import fs from 'fs';
import path from 'path';
import { Compiler, WebpackPluginInstance } from 'webpack';

export class GenerateMustacheScriptsPartial implements WebpackPluginInstance  {
	apply(compiler: Compiler) {
		compiler.hooks.done.tap('generateMustacheScriptsPartial', (statsData) => {
			const stats = statsData.toJson();
			const headerLinks: Array<string> = [];
			const bodyLinks: Array<string> = [];

			if (!stats.errors?.length) {
				stats.assets?.forEach(asset => {
					if (asset.name.match(/chunk/)) return;
					if (asset.name.match(/bundle.js$|npm\.*.*\.js$/)) {
						bodyLinks.push(this._generateScriptTag(asset.name));
					}
					else if (asset.name.match(/bundle.css$|npm\.*.*\.css$/)) {
						headerLinks.push(this._generetaStyleTag(asset.name))
					}
					else if (asset.name.match(/external-vendors-body\S+/)) {
						if (asset.name.match(/.css$/)) {
							bodyLinks.push(this._generetaStyleTag(asset.name.replace('/', '')));
						}
						else if (asset.name.match(/.js$/)) {
							bodyLinks.push(this._generateScriptTag(asset.name.replace('/', '')));
						}
					}
					else if (asset.name.match(/external-vendors-head\S+/)) {
						if (asset.name.match(/.js$/)) {

							bodyLinks.push(this._generateScriptTag(asset.name.replace('/', '')));
						}
						else if (asset.name.match(/.css$/)) {
							headerLinks.push(this._generetaStyleTag(asset.name.replace('/', '')));
						}
					}
				});

				fs.writeFileSync(path.join(__dirname, '../../', 'bundledScripts.partial'), headerLinks.join('\n'));
				fs.writeFileSync(path.join(__dirname, '../../', 'bundledScriptsBody.partial'), [...bodyLinks.filter(e => !e.match(/qcloud/)), ...bodyLinks.filter(e => e.match(/qcloud/))].join('\n'));
			}
		});
	}

    private _generateScriptTag (fileName: string) {
        return `<script src=".static/dist/${fileName}"></script>`;
    } 

    private _generetaStyleTag (fileName: string) {
        return `<link href=".static/dist/${fileName}" rel="stylesheet" type="text/css">`;
    } 
}