import { from, Subject } from "rxjs";
import { CustomStoreArray } from "../../grid/CustomStoreArray";
import { AddColumnButtonsCadastro, createdxDatasource, cxGridIncluirFiltroMaster, formatWithDataType, MontarGridCustomToolButtons } from "../../nGrid";
import dxDataGrid from 'devextreme/ui/data_grid';
import { nExpressionExecutor } from "../../nExpressionExecutor";
import { GridExpressions } from "../../grid/GridExpressions";
import { DateConfigurator } from "../../grid/DateConfigurator";
import gridManager from "../../../ts/components/grid-manager/grid-manager";
import { createProcessDataSourceOnDemand } from "../../../ts/components/widgets/tw-cx-grid/grid/create-process-datasource-on-demand";
import { ConsultaAjaxDeleteReg } from "../../nWebUtil";

export function buildProcessGrid(args) {
	if (FormManager.afterCreateAllForms.isStopped) $(() => ProcessGrid.buildGrid(args));
	else FormManager.afterCreateAllForms.subscribe(() => ProcessGrid.buildGrid(args));
}

export class ProcessGrid {
	static buildGrid({ element = null, grid, params: Params, definicaoObjetos: DefinicaoObjetos }) {
		const valOrEmpty = (val) => (val || '');
		const Class = grid.Class ? valOrEmpty(grid.Class) : valOrEmpty(DefinicaoObjetos.Class);
		const Alias = grid.Class ? valOrEmpty(grid.Alias) : valOrEmpty(DefinicaoObjetos.Alias);

		if (grid.nomegrid === 'nClientDataSet' && DefinicaoObjetos.ntfCadGrid) grid.nomegrid = `${Class}${Alias}`;

		evaluateGridConfig(grid);

		let configProcess = ProcessGrid.getConfigFromProcess({ grid, DefinicaoObjetos, Class, Alias, Params });

		let onEditingStart = ProcessGrid.getOnEditingStart();

		let onEditorPreparingMask = ProcessGrid.getOnEditorPreparingForMask(grid);
		let onEditorPreparingExpression = ProcessGrid.getOnEditorPreparingForExpressions({...grid, ...onEditorPreparingMask });
		let onEditorPreparingDate = DateConfigurator.onEditorPreparing({ ...grid, ...onEditorPreparingExpression });

		if (DefinicaoObjetos.ntfCadGrid) {
			AddColumnButtonsCadastro(grid.columns, Class, grid.keyExpr.join(';'), DefinicaoObjetos.ToolButtons, grid.DisableButtons);
		}

		var config = prepareGridConfig(configProcess, grid, onEditorPreparingDate, onEditingStart);

		gridManager.resetActions(grid.nomegrid);

		const instance = createGrid(element || `dataGridConteudo${grid.nomegrid}`, config);

		ProcessGrid.setGridIfCadGrid(DefinicaoObjetos.ntfCadGrid, instance, Alias, Class);

		ProcessGrid.triggerEvents({
			gridInstance: instance,
			Class,
			Alias,
			gridName: grid.nomegrid,
		});

		return instance;
	}

	static setGridIfCadGrid(isCadGrid, gridInstance, Alias, Class) {
		if (!isCadGrid) return;

		gridInstance.option('remoteOperations', {
				paging: true,
				filtering: true,
				sorting: true,
			});

			const dataSource = createdxDatasource({
				_AAlias: Alias,
				_AClassName: Class,
				_AFuncaoFiltroCX: cxGridIncluirFiltroMaster,
			});

		gridInstance.option('dataSource', dataSource);
		gridInstance.refresh();

		FormManager
			.getForm(Class, Alias)
			.then((form) => form.grid = gridInstance)
			.catch((error) => console.error(error));
		}
		
	static triggerEvents(triggerOptions) {
		if (FormManager.afterCreateAllForms.isStopped) {
			ProcessGrid.triggerEventsElementWhenFound(triggerOptions);
		}
		else {
			FormManager.afterCreateAllForms.subscribe(() => {
				ProcessGrid.triggerEventsElementWhenFound(triggerOptions);
		});
		}
	}

	static triggerEventsElementWhenFound({ gridInstance, Class, Alias, gridName }) {
		const gridElement = gridInstance.element().get(0);

		const triggerEvents = () => {
			const form = FormManager.byname(Class, Alias);

				form.afterCreateGridConteudo.next(`dataGridConteudo${gridName}`);
					
				gridInstance.on('contentReady', () => {
					form.gridContentReady.next(`dataGridConteudo${gridName}`)
				});
			}
		//};

		if (document.contains(gridElement)) {
			triggerEvents();
		}
		else {
			(new MutationObserver((_, observer) => {
				if (!document.contains(gridElement)) return;
				
				observer.disconnect();
				
				triggerEvents();

			})).observe(document.documentElement, {
				childList: true,
			});
		}
	}

	static getConfigFromProcess({ grid, DefinicaoObjetos, Class, Alias, Params }) {
		return {
			form: { Class, Alias, Params },
			onRowRemoving: ConsultaAjaxDeleteReg,
			grouping: { contextMenuEnabled: true, autoExpandAll: false },
			groupPanel: { visible: "auto" },
			cacheEnabled: true,
			dataSource: DefinicaoObjetos.ntfCadGrid ? [] : ProcessGrid.createDataSource({ grid, Class, Params }),
			stateStoring: ProcessGrid.getStateStoring(Class, grid.nomegrid),
			onCellClick: ProcessGrid.getOnCellClick(grid),
			onRowUpdating: ProcessGrid.getOnRowUpdating(),
			onToolbarPreparing: ProcessGrid.getOnToolbarPreparing(DefinicaoObjetos, grid, Params),
		}
	}

	static createDataSource(props) {
		const { grid, Class, Params } = props;
		const { dataSettings, GridButtons, nomegrid, iddata } = grid;

		if (![null, undefined].includes(iddata)) { 
			return createProcessDataSourceOnDemand({
				className: Class,
				gridName: nomegrid,
				dataId: iddata,
			});
		}

		if (dataSettings.UsarDataset) {
			return ProcessGrid.createCustomDataSourceArray({
				...props,
				dataSettings: ProcessGrid.mapDataSettingsConfig(dataSettings),
			});
		}

		const customSaveButton = GridButtons?.find(({ CustomSave }) => CustomSave);

		if (customSaveButton) {
			return ProcessGrid.createCustomDataSourceArray({
				...props,
				dataSettings: ProcessGrid.mapDataSettingsConfig({
					...customSaveButton,
					...customSaveButton.Opcoes,
				}),
			})
		}

		return grid.items;
	}

	static createCustomDataSourceArray({ grid, Class, Params, dataSettings }) {
		return new CustomStoreArray({
			key: grid.keyExpr,
			customStoreArrayOptions: {
				Class,
				Params,
				columnOptions: grid.columns,
				dataSourceArray: grid.items,
				...dataSettings,
			},
		});
	}
	
	static mapDataSettingsConfig({ DataReturn, DataSend, Name, datasetNames }) {
		return {
			DataReturn,
			Name,
			datasetNames,
			requestDataType: DataSend,
		};
	}

	static getStateStoring(Class, gridName) {
		return {
			enabled: true,
			type: "custom",
			ignoreColumnOptionNames: [],
			customLoad: async function () { return await qGrid.loadGridState(`${Class}${gridName}`); },
			customSave: (state) => { return qGrid.saveGridState(`${Class}${gridName}`, state); }
		}
	}

	static getOnCellClick(grid) {
		if (!grid.editing.allowUpdating || grid.selection.mode == "multiple")
			 return null
		else {
			return  (e) => e.component.selectRowsByIndexes([e.rowIndex]);
		}			 
	}

	static getOnRowUpdating() {
		return (ope) => { ope.newData = { ...ope.oldData, ...ope.newData }; }
	}

	static getOnToolbarPreparing(DefinicaoObjetos, grid, Params) {
		const { ToolButtons, Class, Alias, ntfCadGrid } = DefinicaoObjetos;
		const { GridButtons } = grid;

		return (e) => {
			let toolbarItems = e.toolbarOptions.items;

			MontarGridCustomToolButtons({
				Alias: Alias,
				Class: Class,
				hasInsertButton: ntfCadGrid && ToolButtons.find(({ Name, disabled }) => (Name == 'actDataSetInsert' && disabled != 'disabled')),
				GridButtons: GridButtons?.filter(({ CustomSave }) => !CustomSave),
				gridComponent: e.component,
				gridToolbarItems: toolbarItems,
				Params,
			});

			e.toolbarOptions.items = [...toolbarItems, ...ProcessGrid.getDefaultToolButtons(e, grid.nomegrid)];
		}
	}

	static getDefaultToolButtons(gridEvent, gridName) {
		return [
			ProcessGrid.getRefreshButton(gridName),
			ProcessGrid.exportButton(gridEvent),
			ProcessGrid.gridOptionsButton(gridName)
		]
	}

	static getRefreshButton(gridName) {
		return {
			widget: "dxButton", 
			locateInMenu: "after",
			options: { 
				icon: "refresh", 
				hint: "Atualizar", 
				onClick: function() { 
					const datagrid = dxDataGrid.getInstance(`#dataGridConteudo${gridName}`);
					const temp = datagrid.state();
		
					datagrid.refresh();
					datagrid.state(temp);
				} 
			},
			location: "after",
		};
	}

	static exportButton(gridEvent) {
		return {
			location: "after",
			widget: "dxDropDownButton",
			options: {
				dropDownOptions: {
					width: 140
				},
				displayExpr: "text",
				icon: "la la-download",
				hint: "Exportar",
				items: [
					{ text: "Exportar para Excel", action: "excel"},
					{ text: "Exportar para PDF", action: "pdf"}
				],
				onItemClick(data, ...args) {
					exporter.export(gridEvent, data.itemData.action);
				}
			}
		}
	}

	
	static gridOptionsButton(gridName) {
		return {
			location: "after",
			widget: "dxDropDownButton",
			options: {
				dropDownOptions: {
					width: 140
				},
				displayExpr: "text",
				icon: "las la-th",
				hint: "Opções do grid",
				items: [
					{ text: "Restaurar colunas", action: "restaurar"},
					{ text: "Selecionar colunas", action: "selecionar"}
				],
				onItemClick(data) {
					if (data.itemData.action == 'restaurar'){
						dxDataGrid.getInstance($(`#dataGridConteudo${gridName}`)).state({});
					} else {
						dxDataGrid.getInstance($(`#dataGridConteudo${gridName}`)).showColumnChooser();
					}
				}
			}
		}
	}

	/**
	 * 
	 * @param {*} gridConfig config from request
	 * 
	 * Composition of onEditorPreparing 
	 */
	static getOnEditorPreparingForExpressions(gridConfig) {
		const { onEditorPreparing } = gridConfig;

		return {
			onEditorPreparing: (e) => {
				const expressionExecutor = new nExpressionExecutor();

				if (onEditorPreparing) onEditorPreparing(e);

				if (e.parentType === "dataRow") {
					let { component, dataField, row: { data, rowIndex } } = e;

					e.editorOptions.onValueChanged = GridExpressions.onChange({
						data,
						rowIndex,
						expressionExecutor,
						editorField: dataField,
						gridComponent: component,
					});
				}
			}
		}
	}

	static getOnEditorPreparingForMask(gridConfig) {
		const { onEditorPreparing } = gridConfig;

		return {
			onEditorPreparing: (e) => {
				if (onEditorPreparing) onEditorPreparing(e);

				if (e.dataType === "string" && e.mask) {
					e.editorOptions.mask = e.mask;
				}
			}
		}
	}

	static getOnEditingStart() {
		// pode ser utilizada para alterar o valor da celula a ser editada
		// por exemplo para remover os carecteres html das strings (&nbsp...), 
		return {
			onEditingStart: (e) => {
				const { column, component, data, key, cancel } = e
				const { dataType, dataField } = column;

				let newData = { ...data };

				if (dataType === 'string') {
					formatWithDataType(dataType, newData, dataField, newData[dataField])

					if (newData[dataField] !== data[dataField]) {
						component.cellValue(component.getRowIndexByKey(key), column.dataField, newData[column.dataField]);
					}
				}
			}
	    }
	}
}