import FormManager from "../../FormManager";
import { saveLookupFields, fixLookupFields, recoverLookupFields, initLookupStyle } from "../../nLookUp";
import { nFraControl } from "../../nFraControl";
import { CarregarDataCadastro } from "../../nWebUtil";
import { CarregarMenu, invalidateMenuCache } from "../../../ts/components/menu/menu";
import { PostWithForm } from "../../nRequests";

export function runEndPageScript(args) {
    if (FormManager.afterCreateAllForms.isStopped) ScriptCompanySelection.script(args);
    else FormManager.afterCreateAllForms.subscribe(() => ScriptCompanySelection.script(args));
}

class ScriptCompanySelection {
    static script({ scope, isMultiColumn, hasModules }) {
        ScriptCompanySelection.handleParams(scope);
        ScriptCompanySelection.createReturnUrlParam(scope);
        ScriptCompanySelection.handleRequiredFields(scope);
        ScriptCompanySelection.handleClickSubmit(hasModules);

        initLookupStyle(isMultiColumn, true);
    }

    static handleFiltros({ Filtros, Class }) {
        if (Filtros) {
            FormManager.loadCadastro({ _ACadastro: Class, _AFiltros: Filtros });
        }
    }

    static handleParams({ Params }) {
        if (Params) {
            CarregarDataCadastro(Params);
        }
    }

    static createReturnUrlParam({ RedirectURL, RedirectParams, VerificaSelecao }) {
        if (RedirectURL) {
            const form = FormManager.byname('TnDMSelecao');

            if (!form) return;

            const redirectParamsInput = $('<input type="hidden" name="p.REDIRECTPARAMS">');

            redirectParamsInput.val(JSON.stringify(RedirectParams));
            
            form
                .element
                .append($(`<input type="hidden" name="p.REDIRECTURL" value="${RedirectURL}">`))
                .append($(`<input type="hidden" name="p.VERIFICASELECAO" value="${VerificaSelecao}">`))
                .append(redirectParamsInput);
        }
    }

    static handleRequiredFields({ VerificaSelecao }) {
        if (VerificaSelecao) {
            const form = FormManager.byname('TnDMSelecao');

            if (!form) return;

            VerificaSelecao
                .split(';')
                .forEach((fieldName) => {
                    form.setrequired(fieldName.toUpperCase(), true);        
                });
        }
    }

    static handleClickSubmit(hasModules) {
        $("#btnDefaultSubmit").on('click', function(event) {
            const form = $(event.currentTarget.form).data('q-form');

            if (!nFraControl.ValidarCamposObrigatorios(form)) {
                event.preventDefault();
            }
            else {
                $('#q_dashboard_menu').remove();
                CarregarMenu();
                invalidateMenuCache(undefined, hasModules);

                KTApp.blockPage();

                ScriptCompanySelection.submitForm(form);
            }
        });
    }

    static submitForm(form) {
        const inputValues = Object
            .entries(form.getValues())
            .map(([key, value]) => [`p.${key}`, value]);

        const redirectValues = ['p.REDIRECTPARAMS', 'p.REDIRECTURL', 'p.VERIFICASELECAO'].map((inputName) => {
            const inputEl = $(`[name="${inputName}"]`, form.element)
            
            if (inputEl.length) return [inputName, inputEl.val()];
        });

        const postwithForm = new PostWithForm(form.element.attr('action'));
        
        [...inputValues, ...redirectValues].filter((input) => input).forEach(([name, value]) => postwithForm.addParam(name, value)); 
        
        postwithForm.submit();
        postwithForm.remove();
    }
}