import _ from "lodash";
import { FormatErrorMessage } from "../../../nDialog";

export const onSelectionLancamentos = (gridOposta) => {
	const isConciliado = (data) => Boolean(parseInt(data['CONCILIADO']));
	const hasConciliado = (arr) => arr.reduce((acc, item) => acc || isConciliado(item), false);
	var customSelect = false;

	return async ({ selectedRowsData, selectedRowKeys, currentSelectedRowKeys, currentDeselectedRowKeys, component, element }) => {
		let previousSelectedKeys = _.differenceBy(selectedRowKeys, currentSelectedRowKeys, 'ID');
		let previousSelectedData = _.differenceBy(selectedRowsData, currentSelectedRowKeys, 'ID');
		let currentSelectedRowData = _.intersectionBy(selectedRowsData, currentSelectedRowKeys, 'ID');

		const deselectRows = ({ grid, useStopPropagation, rows }) => {
			if (!rows.length) return;
			if (useStopPropagation) customSelectLancamento = useStopPropagation;

			grid.deselectRows(rows)
		}

		const deselectAll = ({ grid, useStopPropagation }) => {
			let hasSelected = grid.getSelectedRowKeys().length;

			if (!hasSelected) return;
			if (useStopPropagation) customSelectLancamento = useStopPropagation;

			grid.deselectAll();
		}

		const selectRows = ({ grid, useStopPropagation, rows }) => {
			if (!rows.length) return;
			if (useStopPropagation) customSelectLancamento = useStopPropagation;

			grid.selectRows(rows);
		}

		if (customSelect) return;
		if (customSelectExtrato) {
			customSelectExtrato = false;
			return;
		}
		customSelect = true;

		if (hasConciliado(currentSelectedRowData)) {
			const undoLastSelectionWithErrorMessage = () => {
				FormatErrorMessage('Item conciliado e existente na seleção');

				deselectRows({
					grid: component,
					rows: currentSelectedRowKeys
				});

				selectRows({
					grid: component,
					rows: [...currentDeselectedRowKeys, ...previousSelectedKeys]
				});
			};

			if (!previousSelectedKeys.length && !currentDeselectedRowKeys.length) {
				const isFirstItemSelected = (rows) => (!hasConciliado(rows) && rows.length);
				let selectedRowsDataGridOposta = gridOposta.getSelectedRowsData();

				if (isFirstItemSelected(selectedRowsDataGridOposta)) {
					undoLastSelectionWithErrorMessage();
				}
				else {
					selectRows({
						grid: gridOposta,
						useStopPropagation: true,
						rows: currentSelectedRowKeys
					})
				}
			}
			else {
				if (hasConciliado(previousSelectedData) || hasConciliado(await component.getDataByKeys(currentDeselectedRowKeys))) {
					let itemToSelect = currentSelectedRowData.find((item) => Boolean(parseInt(item['CONCILIADO'])))

					selectRows({
						grid: component,
						rows: [itemToSelect]
					});

					selectRows({
						grid: gridOposta,
						useStopPropagation: true,
						rows: [itemToSelect]
					});
				}
				else {
					undoLastSelectionWithErrorMessage();
				}
			}
		}
		else {
			if (hasConciliado(previousSelectedData)) {
				selectRows({
					grid: component,
					rows: currentSelectedRowKeys
				});

				deselectAll({
					grid: gridOposta,
					useStopPropagation: true
				});
			}
			else {
				if (hasConciliado(await component.getDataByKeys(currentDeselectedRowKeys))) {
					deselectRows({
						grid: gridOposta,
						useStopPropagation: true,
						rows: currentDeselectedRowKeys
					});
				}
			}
		}
		customSelect = false
	}
}

export const onSelectionExtratos = (gridOposta) => {
	const isConciliado = (data) => Boolean(parseInt(data['CONCILIADO']));
	const hasConciliado = (arr) => arr.reduce((acc, item) => acc || isConciliado(item), false);
	var customSelect = false;

	return async ({ selectedRowsData, selectedRowKeys, currentSelectedRowKeys, currentDeselectedRowKeys, component, element }) => {
		let previousSelectedKeys = _.differenceBy(selectedRowKeys, currentSelectedRowKeys, 'ID');
		let previousSelectedData = _.differenceBy(selectedRowsData, currentSelectedRowKeys, 'ID');
		let currentSelectedRowData = _.intersectionBy(selectedRowsData, currentSelectedRowKeys, 'ID');

		const deselectRows = ({ grid, useStopPropagation, rows }) => {
			if (!rows.length) return;
			if (useStopPropagation) customSelectExtrato = useStopPropagation;

			grid.deselectRows(rows)
		}

		const deselectAll = ({ grid, useStopPropagation }) => {
			let hasSelected = grid.getSelectedRowKeys().length;

			if (!hasSelected) return;
			if (useStopPropagation) customSelectExtrato = useStopPropagation;

			grid.deselectAll();
		}

		const selectRows = ({ grid, useStopPropagation, rows }) => {
			if (!rows.length) return;
			if (useStopPropagation) customSelectExtrato = useStopPropagation;

			grid.selectRows(rows);
		}

		if (customSelect) return;
		if (customSelectLancamento) {
			customSelectLancamento = false;
			return;
		}
		customSelect = true;

		if (hasConciliado(currentSelectedRowData)) {
			const undoLastSelectionWithErrorMessage = () => {
				FormatErrorMessage('Item conciliado existente na seleção');

				deselectRows({
					grid: component,
					rows: currentSelectedRowKeys
				});

				selectRows({
					grid: component,
					rows: [...currentDeselectedRowKeys, ...previousSelectedKeys]
				});
			};
			const selectItemConciliadoInBothGrids = () => {
				selectRows({
					grid: component,
					rows: [currentSelectedRowKeys[0]]
				});

				selectRows({
					grid: gridOposta,
					useStopPropagation: true,
					rows: [currentSelectedRowKeys[0]]
				});
			};

			if (!previousSelectedKeys.length && !currentDeselectedRowKeys.length) {
				const isFirstItemSelected = (rows) => (!hasConciliado(rows) && rows.length)
				let selectedRowsDataGridOposta = gridOposta.getSelectedRowsData();

				if (isFirstItemSelected(selectedRowsDataGridOposta)) {
					undoLastSelectionWithErrorMessage()
				}
				else {
					selectItemConciliadoInBothGrids();
				}
			}
			else {
				if (hasConciliado(previousSelectedData) || hasConciliado(await component.getDataByKeys(currentDeselectedRowKeys))) {
					selectItemConciliadoInBothGrids();
				}
				else {
					undoLastSelectionWithErrorMessage();
				}
			}
		}
		else {
			selectRows({
				grid: component,
				rows: [currentSelectedRowKeys[0]]
			});

			if (hasConciliado(previousSelectedData)) {
				deselectAll({
					grid: gridOposta,
					useStopPropagation: true
				});
			}
			else {
				if (hasConciliado(await component.getDataByKeys(currentDeselectedRowKeys))) {
					deselectRows({
						grid: gridOposta,
						useStopPropagation: true,
						rows: currentDeselectedRowKeys
					});
				}
			}
		}
		customSelect = false
	}
}

export var customSelectLancamento = false;
export var customSelectExtrato = false