import template from './twImageButton.component.html';
import style from './twImageButton.component.scss';
import './twImageButton.component.scss';
import WidgetBase from '../Widget.base';
import { DynamicImport } from '../../../Utils/DynamicImport';

export const buildWidget = args => new ImageButton(args);

class ImageButton extends WidgetBase {
	constructor(args) {
		super();

		this.buildWidget(args);
	}

	async buildWidget({ scope, className, notifications, element }) {
		let widgetElement = $(this.render({ template, scope, style }));

		await this.setAction({ widgetElement, scope, className, notifications });
		
		this.setImages({ widgetElement, scope });
		this.setTooltip({ widgetElement, scope });

		$(element).replaceWith(widgetElement);
		setTimeout(() => widgetElement.css('opacity', 1));
	}

	async setAction({ widgetElement, scope, className, notifications }) {
		let { Action } = scope;

		if (Action) {
			try {

				let buttonAction = (await DynamicImport.Import(`Actions/${Action}`)).default;
				
				widgetElement.on('click', (event) => buttonAction({ 
					event, 
					className, 
					scope: {
						...scope,
						notifications
					}
				}));
			}
			catch {
				// console.error(e);
			}
		}
	}

	setImages({ widgetElement, scope }) {
		let { ImageMouseEnter, ImageMouseLeave } = scope;
		let imgElement = widgetElement.find('img');

		imgElement.attr('src', `${ImageMouseLeave}`);
		widgetElement.on('mouseenter', () => imgElement.attr('src', `${ImageMouseEnter}`));
		widgetElement.on('mouseleave', () => imgElement.attr('src', `${ImageMouseLeave}`));
	}

	setTooltip({ widgetElement, scope }) {
		if (scope.Name == "HomeButton") {
			widgetElement.attr({
				title: `Retornar à página inicial da área de trabalho`,
				'data-direction': 'left',
				'data-toggle': 'kt-tooltip',
				'data-width': 'auto',
			})
		}
	}
}
