import nAction from './form/actions/nActions';
import Utils from './Utils/Utils';
import { Formulario } from './nFormulario';
import { FraControlPropertiesExtractor as PropExtractor } from './form/PropertiesExtractor/FraControlPropertiesExtractor';
import { nFraControl } from './nFraControl';
import { PropertiesExtractor } from './form/PropertiesExtractor/PropertiesExtractor';
import { Subject } from 'rxjs';
import { first } from 'rxjs/operators';
import { RequestDadosCadastro } from './nRequests';
import { FormRepeaterForm } from '../ts/components/form-repeater/form-repeater';
import { CarregarDataCadastro } from './nWebUtil';
import { setDatePicker } from './setDatepicker';
import { Shortcut } from '../ts/components/toolbuttons/util/shortcuts';

class FormManager {
	constructor() {
		this._modalFilter = null;
		this._items = this._initItems();
		this._afterCreateAllForms = new Subject();
		this._afterCreateDynamicForm = new Subject();
		this._afterSetModalFilter = new Subject(); 
		this._linkConsultaOrigem = '';
		this._lastValidFormData = null;
		this._canChangeFields = true;
		this._quemChamou = null;
		this._lookupToFix = [];
		this._filterForCadGrid = null;
		this._reportCustomEvent = new Map();
		this._shortCuts = new Shortcut();
	}

	_initItems() {
		let items = [];

		Object.defineProperty(items, 'firstUsable', {
			value: function () {
				return this.find((form) => {
					return !form.isFormRepeater && form.classe != 'TnDMSelecao'
				})
			}
		})

		return items;
	}

	get lookupToFix() {
		return this._lookupToFix;
	}

	set lookupToFix(e) {
		this._lookupToFix = e;
	}

	get QuemChamou() {
		return this._quemChamou;
	}

	set QuemChamou(quemChamou) {
		this._quemChamou = quemChamou;
	}

	get FiltroModal() {
		return this._modalFilter;
	}

	set FiltroModal(filter) {
		this._modalFilter = filter;

		this._afterSetModalFilter.next(filter);
		this._afterSetModalFilter.complete();
	}

	get afterSetModalFilter() {
		return this._afterSetModalFilter;
	}

	get itens() {
		return this._items;
	}

	get afterCreateAllForms() {
		return this._afterCreateAllForms;
	}

	get afterCreateDynamicForm() {
		return this._afterCreateDynamicForm;
	}

	get LinkConsultaOrigem() {
		return this._linkConsultaOrigem;
	}

	set LinkConsultaOrigem(link) {
		this._linkConsultaOrigem = link;
	}

	get ultimosDadosCadastro() {
		return this._lastValidFormData;
	}

	set ultimosDadosCadastro(data) {
		this._lastValidFormData = data;
	}

	get podeEventoChangeFields() {
		return this._canChangeFields;
	}

	set podeEventoChangeFields(value) {
		this._canChangeFields = value;
	}

	get filterForCadGrid() {
		return this._filterForCadGrid;
	}

	set filterForCadGrid(value) {
		this._filterForCadGrid = value; 
	}

	get shortCuts() {
		return this._shortCuts;
	}

	insertForm(form) {
		this._items.push(form);
	}

	removeForm(name) {
		let itemIndex = this._items.findIndex((item) => item.nome.toUpperCase() === name.toUpperCase());

		this._items.splice(itemIndex, 1);
	}

	byname(classe, alias) {
		if (alias != null) {
			var anome = classe + alias;
			for (var i = 0; i < this._items.length; i++) {
				if ((this._items[i].nome.toUpperCase() == anome.toUpperCase()) ||
					(this._items[i].nome.toUpperCase() == 'T' + anome.toUpperCase()) ||
					('T' + this._items[i].nome.toUpperCase() == anome.toUpperCase())) {
					return this._items[i];
				}
			}
			for (var i = 0; i < this._items.length; i++) {
				if ((this._items[i].nome.split('_')[0].toUpperCase() == anome.toUpperCase()) ||
					(this._items[i].nome.split('_')[0].toUpperCase() == 'T' + anome.toUpperCase()) ||
					('T' + this._items[i].nome.split('_')[0].toUpperCase() == anome.toUpperCase())) {
					return this._items[i];
				}
			}
		} else {
			var anome = classe;
			if (classe == null) {
				return this._items[0];
			}
			for (var i = 0; i < this._items.length; i++) {
				if ((this._items[i].classe.toUpperCase() == anome.toUpperCase()) ||
					(this._items[i].classe.toUpperCase() == 'T' + anome.toUpperCase()) ||
					('T' + this._items[i].classe.toUpperCase() == anome.toUpperCase())) {
					return this._items[i];
				}
			}
		}
		return null;
	}

	/**
	 * 
	 * @param {*} element 
	 * 
	 * Pass an element contained by the desired form
	 */
	getFormByChildElement(element) {
		if (!element || !element.length) return null;

		return element.closest('form').data('q-form');
	}

	getFormByFieldElement(fieldElement) {
		if (!fieldElement || !fieldElement.length) return null;

		return fieldElement.data('q-field').InstanciaFormulario;
	}

	createDynamicDetailForm({ Class, Alias, contextElement, elementMissingProps }) {
		const created = [];

		const addMissingProperties = (element, properties) => {
			const { classe, alias } = element.data();
			
			if (`${Class}${Alias}` == `${classe}${alias}`) element.data(properties);	
		};
		
		$('[data-script-form-create=true]', contextElement).each((index, el) => {
			addMissingProperties($(el), elementMissingProps);

			const form = this.createForm(el);

			created.push(form);

			this.createAllFields(form);
			this._createDefaultAction(form);

			$(el).remove();
		});

		$('[data-create-naction=true]', contextElement).each((index, el) => {
			this.createAction(el);
		});

		setDatePicker();
		
		created.forEach((form) => this._afterCreateDynamicForm.next(form));
	}

	createAllForms() {
		$('[data-script-form-create=true]').each((index, el) => {
			let form = this.createForm(el);

			this.createAllFields(form);
			this._createDefaultAction(form);

			$(el).remove();
		});

		$('[data-create-naction=true]').each((index, el) => {
			this.createAction(el);
		});

		this._afterCreateAllForms.next(this._items);
		this._afterCreateAllForms.complete();
	}

	createForm(el) {
		let options = PropertiesExtractor.extractOptionsForm($(el));

		return this._buildForm({ ...options, element: $(`form[data-class='${options.classe}']`) });
	}

	_buildForm(options) {
		let newForm = new Formulario(options)
		this._items.push(newForm);

		return newForm;
	}

	createAllFields(/** @type {Formulario} */ form, additionalOptions = {}) {
		/** @type {JQuery<HTMLElement>} */
		const rootElement = additionalOptions.rootElement;

		(rootElement || form.element)
			.find(`[data-fracontrol-script-create="true"][data-classname="${form.classe}"][data-alias="${form.alias}"]`)
			.each((index, fieldEl) => {
				this.createField(form, fieldEl, additionalOptions);
			});

		form.afterCreateStaticFields.next();

		if (!form.isFormRepeater) {
			form.afterCreateStaticFields.complete();
		}
	}

	/**
	 * 
	 * @param {*} form 
	 * @param {*} el 
	 * @param additionalOptions.rootElement Base element to search for field input  
	 */
	createField(form, el, additionalOptions) {
		let options = this.extractFormFieldOptions(form, el, additionalOptions);

		this._addFieldToForm(form, options);

		el.parentNode.removeChild(el);
	}

	extractFormFieldOptions(form, el, additionalOptions) {
		let options = { form, ...PropertiesExtractor.extractOptionsField($(el), additionalOptions) };

		if (form) {
			switch (options.ntcType) {
				case "ntcLinkEdit": return PropExtractor._ntcLinkEditProperties(options, additionalOptions);
				case "ntcCheckBox": return PropExtractor._ntcCheckBox(options, additionalOptions);
				case "ntcCheckListBox":
				case "ntcCheckListBoxScroll":
				case "ntcCheckListBoxHorizontal": return PropExtractor._ntcCheckListBoxProperties(options, additionalOptions);
				default: return PropExtractor._ntcDefaultProperties(options, additionalOptions);
			}
		}
	}

	_addFieldToForm(form, extractedOptions) {
		extractedOptions.forEach(option => form.addfield(option));
	}

	_createDefaultAction(form) {
		let refresh = {
			config: {
				alias: form.alias,
				class: form.classe,
			},
			form: form,
			name: "actDataSetRefreshExecute",
			type: "click",
			isDefault: true
		}

		form.setAction(refresh);
	}

	createAction(el) {
		let options = PropertiesExtractor.extractOptionsAction($(el))
		let form = this.byname(options.class, options.alias);

		options.actions.forEach(action => {
			if (action.name) {
				if (form) {
					form.setAction({
						...action,
						form: form,
						elementTrigger: el
					});
				}
				else { // caso ser uma função que não precisa do form por exemplo a função que abre um menu na area de trabalho
					new nAction[action.name]({ ...action, elementTrigger: el });
				}
			}
		})
	}

	IncluirValoresFixosModal(form) {
		let FiltroModal = this._modalFilter;

		if (FiltroModal && form != undefined) {
			$.each(form.detailfields, (i, field) => {
				if (FiltroModal.hasOwnProperty(field)) {
					form.setfieldvalue(field, FiltroModal[field]);
				}
			});
		};

		if (this.SuggestionFieldValues && form && form.suggestionFields) { // "Sugere" valores vindos do link "clique aqui" da treeview
			form.suggestionFields.forEach((field) => {
				if (this.SuggestionFieldValues.hasOwnProperty(field)) {
					form.setfieldvalue(field, this.SuggestionFieldValues[field]);
				}
			})
		}
	}

	CadastroClearFields({ form, clearDetails = false }) {
		if (form.ehdetalhe) {
			form.clearfields();
		}
		else {
			this.itens.forEach(function (form) {
				if (form.classe != 'TnDMSelecao') {
					if (form.ehdetalhe) {
						if (clearDetails) {
							form.clearfields();
						}
					} else {
						form.clearfields();
					}
				}
			});
		}
	}

	SetarValoresInputs(dados) {
		const form = this.byname(dados.Class, dados._AAlias);

		if (dados.StatusRegistro == 'naoencontrado') {
			if (dados._AOrigem != 'anterior' && dados._AOrigem != 'proximo') {
				this.CadastroClearFields({ form: form, clearDetails: false });
			};
		}
		else {
			(form.isFormRepeater && dados.EhLookup !== 'sim') 
				? this._loadFormRepeater(form, dados)
				: form.loadfromdata(dados);
		};
	}

	/**
	 * 
	 * By definition the first instance of the form repeater it's already on FormManager.
	 * 
	 * Get FormManager instance and create or update data.
	 */
	_loadFormRepeater(form, data) {
		/** @type {FormRepeaterForm} */
		let formRepeater = form.formRepeaterInstance;

		formRepeater.loadData(data);
	}

	loadCadastro({ _ACadastro, _AAlias, _AFiltros, _AOrigem }) {
		const getMasterPrimaryKey = ({ ehdetalhe, detailfields, filtrofixo, masterclass }) => {
			if (ehdetalhe) {
				const masterForm = masterclass ? this.byname(masterclass) : null;

				return JSON.stringify(masterForm ? masterForm.chaveprimaria : filtrofixo);
			}

			return detailfields ? JSON.stringify(detailfields) : '';
		};

		let FormToLoad = this.byname(_ACadastro, _AAlias);
		let arequest = new RequestDadosCadastro(_ACadastro, _AAlias);

		KTApp.blockPage();

		arequest.filtros = _AFiltros;
		arequest.origem = _AOrigem;
		arequest.idrequest = FormToLoad.idrequest ?? 0;
		arequest.detalhes = this._getDetailToLoad(FormToLoad);
		arequest.SetChavePrimariaMestre(getMasterPrimaryKey(FormToLoad));

		return arequest.executar(CarregarDataCadastro);
	}

	_getDetailToLoad(formToLoad) {
		if (formToLoad.ehdetalhe) return null;

		const translateMasterFields = (masterFields, detailFields) => {
			if (!(masterFields && detailFields)) return {};

			return masterFields
				.reduce((keyObject, masterField, index) => ({
					...keyObject,
					[masterField]: detailFields[index]
				}), {});
		};

		const getGridFilter = (form, detail) => {
			if (!(form.grid || form.isFormRepeater)) return null;

			return {
				_AClass: form.classe,
				skip: 0,
				take: form.isFormRepeater ? 200 : (form.grid ? form.grid.pageSize() : 10),
				filter: ''
			}
		};

		return this.itens
			.filter((form) => (form != formToLoad && form.ehdetalhe))
			.map((form) => ({
				alias: form.alias,
				class: form.classe,
				filtrofixo: form.filtrofixo || {},
				masterfields: translateMasterFields(form.masterfields, form.detailfields),
				filtrogrid: getGridFilter(form)
			}));
	}

	/**
	 * 
	 * @param {str} name Name of an entity under Utils category
	 * 
	 * Intended for use in specific javascript
	 */
	getUtil(name) {
		return Utils[name];
	}

	runAfterCreateAllForms(func) {
		if (typeof func !== 'function') return;

		if (this._afterCreateAllForms.isStopped) func(this, this._items);
		else this._afterCreateAllForms.subscribe((items) => func(this, items));
	}

	runAfterSetModalFilter(func) {
		if (typeof func !== 'function') return;

		if (this._afterSetModalFilter.isStopped) func(this._modalFilter);
		else this._afterSetModalFilter.subscribe((modalFilter) => func(modalFilter));
	}

	getForm(Class, Alias) {
		return new Promise((resolve, reject) => {
			const form = this.byname(Class, Alias);

			if (form) return resolve(form);

			this._afterCreateDynamicForm
				.pipe(
					first((form) => `${form.classe}${form.alias || ''}` == `${Class}${Alias || ''}`),
				)
				.subscribe(resolve, reject);
		});
	}

	setReportCustomEvent({ Class, Alias = '', reportName, fn }) {
		this._reportCustomEvent.set(`${Class}${Alias}${reportName}`.toLowerCase(), fn);
	}
	
	getReportCustomEvent({ Class, Alias = '', reportName, }) {
		return this._reportCustomEvent.get(`${Class}${Alias}${reportName}`.toLowerCase());
	}

}

export default (new FormManager());