import { lookupAlphaNumeric } from '../../lookup';

export class FraControlPropertiesExtractor {
	static _fixDotInName(name) {
		return (name || "").replace(/\./g, "\\.");
	}

	static _jQueryElmentProps(search, rootElement) {
		return {
			label: $(search.sLabel, rootElement), 
			componente: $(search.sComponente, rootElement), 
			linkbutton: $(search.sLinkbutton, rootElement), 
			resultcomponente: $(search.sResultComponent, rootElement), 
			resultcomponentetext: $(search.sResultComponentText, rootElement),
			componenteimage: search.sComponenteimage ? $(search.sComponenteimage, rootElement) : null
		}
	}

	static _commonProps(options) {
		return {
			nome: options.name,
			visible: options.visible, 
			index: options.index, 
			isMultiColumn: options.isMultiColumn, 
			isGroupColumn: options.isGroupColumn, 
			expressoes: options.expressoes,
			controlType: options.ntcType
		}
	}

	static _ntcLinkEditProperties(options, additionalOptions = {}) {
		const rootElement = additionalOptions.rootElement || options.form.element;

		let properties = [];

		if (options.ntcSubtype == "lookupAlphaNumeric") {
			let sComponente = `#ctrl${this._fixDotInName(options.name)}`;
			let sLinkbutton = `#btn${options.name}`;
			let sResultComponent = `#ctrl${options.name}RESULT`;
			let sResultComponentText = `#ctrl${options.name}RESULTTEXT`;
			
			properties.push({
				...this._commonProps(options),
				...this._jQueryElmentProps({sComponente, sLinkbutton, sResultComponent, sResultComponentText}, rootElement),
				lookupAlpha: new lookupAlphaNumeric({ 
					element: $(`[data-lookupid=ctrl${options.name}]`, rootElement),
					form: options.form
				})
			});
		}
		else {
			properties.push(...this._ntcDefaultProperties(options, additionalOptions));
		}

		return properties;
	}

	static _ntcCheckListBoxProperties(options, additionalOptions = {}) {
		const rootElement = additionalOptions.rootElement || options.form.element;
		let itemname = `${options.name}_checkbox_${options.nvValor}`;
		let sComponente = `#ctrl${this._fixDotInName(itemname)}_ctrl${options.name}`;
		let sLinkbutton = `#btn${itemname}`;
		let sLabel = `#labelctrlopt${itemname}`;
		let sResultComponent = `#ctrl${itemname}RESULT`;
		let sResultComponentText = `#ctrl${itemname}RESULTTEXT`;

		return [{
			...this._commonProps(options),
			...this._jQueryElmentProps({sComponente, sLinkbutton, sLabel, sResultComponent, sResultComponentText}, rootElement),
			nome: itemname
		}];
	}

	static _ntcDefaultProperties(options, additionalOptions = {}) {
		const rootElement = additionalOptions.rootElement || options.form.element;
		let sComponente = `#ctrl${this._fixDotInName(options.name)}`;
		let sLinkbutton = `#btn${options.name}`;
		let sLabel = `#labelctrlopt${options.name}`;
		let sResultComponent = `#ctrl${options.name}RESULT`;
		let sResultComponentText = `#ctrl${options.name}RESULTTEXT`;
		let sComponenteimage = options.isImage ? `#ctrlImg${options.name}` : null;

		return [{
			...this._commonProps(options),
			...this._jQueryElmentProps({sComponente, sLinkbutton, sLabel, sResultComponent, sResultComponentText, sComponenteimage}, rootElement)
		}];
	}

	static _ntcCheckBox(options, additionalOptions = {}) {
		const rootElement = additionalOptions.rootElement || options.form.element;
		let sComponente = `#ctrl${this._fixDotInName(options.name)}`;
		let sLabel = `#labelctrlopt${options.name}`;


		return  [{
			...this._commonProps(options),
			...this._jQueryElmentProps({sComponente, sLabel}, rootElement)
		}];
	}
}