import { BaseAction } from "./BaseAction";
import { saveLookupFields, fixLookupFields, recoverLookupFields } from "../../nLookUp";
import { FormState, getBtnSalvar } from '../../nCadastro';
import { nFraControl } from "../../nFraControl";
import FormManager from "../../FormManager";
import { OperacaoCadastro } from "../../nWebUtil";
import qNotifier from "../../../ts/components/notifier/q-notifier";

export class actDataSetPostExecute extends BaseAction {
	constructor(options) {
		super(options);
	}

	execute(options = {}) {
		const saveButton = getBtnSalvar(this._form);
		 
		if (saveButton.length > 0 && saveButton.attr('disabled') && $(this._element).data('disabled')) {
			return
		}

		if (this._form.BeforePost) {
			this._form
				.BeforePost()
				.then(() => {
					this._DoPostExecute(options);
				});
		} else {
			this._DoPostExecute(options);
		}
	}

	_DoPostExecute(options) {
		this._form.limparAtencaoFields();
		let savedLookups = saveLookupFields(this._form);
		fixLookupFields(savedLookups);
		FormManager.IncluirValoresFixosModal(this._form);

		if (this._ValidarEstadoMestre() && nFraControl.ValidarCamposObrigatorios(this._form)) {
			OperacaoCadastro({
				_AClasse: this._form.classe,
				_Aoperacao: 'CadastroSalvar',
				_AAlias: this._form.alias,
				_AAfterPostAction: this._FindAfterPostAction(options)
			});

		}
		recoverLookupFields(savedLookups);
	}

	verificarAbasLctoCtb() {
	  return (this._form.classe == "TnCtbDMLctoCtb" && $("[data-id=tabTnCtbDMLctoGercredito]:visible, [data-id=tabTnCtbDMLctoGerdebito]:visible, [data-id=tabTnCtbDMLctoAtivDiferenccredito]:visible, [data-id=tabTnCtbDMLctoAtivDiferencdebito]:visible").length == 0);
	}
	
	verificarAbasTabSSimples() {
		return (this._form.classe == "TNFISDMTABSSIMPLES" && $("[data-id=tabTnFisDMTabSSimplesFaixaConfig]:visible, [data-id=tabTnFisDMTabSSimplesFaixa]:visible").length == 0);
	}

	verificarAdmissao() {		
		return !(this._form.classe == "TNFPADMFUNCCONTRATOPREAREATRAB" || this._form.classe == "TNFPADMDADOSOBRIGATORIOSADMISSAO" || this._form.classe == "TNFPADMDADOSNAOOBRIGATORIOSADMISSAO");
	}
	

	_FindAfterPostAction(options) {
		const { forcePostType } = options;
		const { posttype } = $(this._element).data();
		const isDropdwonItem = $(this._element).hasClass('dropdown-item');

	    if (!isDropdwonItem && !forcePostType && !this._form.isFormRepeater && posttype == '1' && 
			this._form.estado == "dsInsert" && this.verificarAdmissao() &&
			(FormManager.itens.filter(item => (item.masterclass != null && item.classe != null && item.masterclass.toUpperCase() === this._form.classe.toUpperCase())).length == 0 || this.verificarAbasLctoCtb() || this.verificarAbasTabSSimples())) {
			return 0;
		} else {
			return posttype;
		};
	}

	_ValidarEstadoMestre() {
		if (this._form.masterclass != null) {
			let masterform = FormManager.byname(this._form.masterclass);
			
			if (masterform.estado != FormState.cadNavegando) {
				qNotifier.error('É necessário salvar o cadastro "' + masterform.caption + '" antes de salvar "' + this._form.caption + '"');
				return false;
			}
		}
		return true;
	}
}