import { BaseAction } from "./BaseAction";
import { nFraControl } from "../../nFraControl";
import { ProcessarDialogsRetorno } from "../../nDialog";
import { Timer } from "../../Timer";
import { TnCaptcha } from "../../nCaptcha";
import FormManager from "../../FormManager";
import qNotifier from "../../../ts/components/notifier/q-notifier";
import { ScriptTaskExecutor } from "../../../ts/utils/script-task-executor/script-task-executor";
import { verifyAxiosRedirect } from "../../nRequests";

export class ActionExecutarExecute extends BaseAction {
	constructor(options) {
		super(options);
	}

	/**
	 * @param {*} form Formulario Instance
	 * @param {*} modalFilter FormManager.FiltroModal
	 */
	_getValuesModalFilter(form, modalFilter) {
		return !modalFilter
			? {}
			: form.fields
				.filter((field) => field.disabled() && modalFilter.hasOwnProperty(field.nome))
				.reduce((values, field) => ({
					...values,
					[field.nome]: modalFilter[field.nome]
				}), {});
	}

	execute() {
		KTApp.blockPage();
		
		if (nFraControl.ValidarCamposObrigatorios(this._form)) {
			const formElement = this._form.element;
			const timer = new Timer();

			this._form.limparAtencaoFields();

			const modalFilter = this._getValuesModalFilter(this._form, FormManager.FiltroModal);
			const Params = getFormValue(this._form);

			if (this._form.idrequest != 0) {
				var runcaptcha = true;

				Params.IDREQUEST = this._form.idrequest;
			}

			const requestConfig = {
				url: formElement.attr('action'),
				data: {
					Class: this._form.classe,
					...modalFilter,
					...Params,
					...this.getSplitParams(),
				},
			};

			this.executeProcess(requestConfig, { timer });

			if (runcaptcha) {
				TnCaptcha.getInstance().start(this._form.idrequest, timer);
			}

			timer.IniciarExecucao();
		}
		else {
			KTApp.unblockPage();
		}
	}

	getSplitParams() {
		if (this._form.tipoObjeto !== 'ntoRelatorio') return {};

		const splitReportParamValue = this._form?.splitReportParam?.getValues() || '';

		if (!splitReportParamValue) return {};

		return {
			['pSepararArquivosPublicacao'.toUpperCase()]: splitReportParamValue,
		};
	}

	executeProcess({ url,  data }, { timer }) {
		const formElement = this._form.element;
		const formClass = this._form.classe;

		axios
			.post(url, data)
			.then((response) => {
				const { data: responseData } = response;

				if (verifyAxiosRedirect(response)) return;

				try {
					const datadialogs = {
						data: {
							...responseData,
							dialogs: responseData.mensagem
						}
					};

					timer.FinalizarExecucao();

					ProcessarDialogsRetorno(datadialogs, (Scope) => {
						timer.IniciarExecucao();

						const requestConfig = {
							url,
							data: {
								...data,
								dialogs: Scope.dialogs,
							},
						};

						this.executeProcess(requestConfig, { timer });
					});
				}
				catch {
					TnCaptcha.getInstance().stop();

					timer.FinalizarExecucao();

					let conteudo = formElement.find('#Conteudo').exists() || formElement.parent().find('#Conteudo');

					conteudo.empty();

					conteudo.append(responseData);
					
					ScriptTaskExecutor();

					const triggerGridMutationObserverElement = $('<div>');
					
					$(document.documentElement).append(triggerGridMutationObserverElement);
				
					triggerGridMutationObserverElement.remove();

					setTimeout(() => {
						conteudo.get(0)?.scrollIntoView()
					});
				}
				finally {
					let formProcess = FormManager.byname(formClass);

					formProcess.AfterExecuteActionProcess.next();
				}
			})
			.catch((error) => {
				console.log(error);

				TnCaptcha.getInstance().stop();

				timer.FinalizarExecucao();

				qNotifier.error('Ocorreu um erro ao executar o processo: ' + error);
			})
	};
}

/** * 
 * @param {*} form Formulario instance 
 */
function getFormValue(form) {
	const isResult = (field) => field.nome.match(/resulttext/i);

	return form.fields
		.filter((field) => !field.componente.is(':disabled') && !field.Lookup && !isResult(field))
		.reduce((fieldsValue, field) => ({
			...fieldsValue,
			[`${field.nome}`]: field.getvalue(),
		}), {});
}