import { Subject } from "rxjs";
import { Formulario } from "../../nFormulario";

export class BaseAction {
	constructor({ form, name, type, elementTrigger, idElement, isDefault, handler, onExecuted }) {
		/** @type {Formulario} */ 
		this._form = form;
		
		this._element = elementTrigger;
		this._name = name;
		this._type = type;
		this._idElement = idElement;
		this._isDefault = isDefault;
		this._onExecuted = new Subject();

		this._registerHandler(type, handler);
		this._registerEvents({
			onExecuted
		});
	}

	get idElement() {
		return this._idElement;
	}

	_registerHandler(type, handler) {
		if (handler) {
			$(this._element).on(handler.type, (event) => handler.action(this, event));
		}
		else {
			$(this._element).on(type, (event) => this.execute(event));
		}
	}

	_registerEvents({ onExecuted }) {
		if (onExecuted) this._onExecuted.subscribe(onExecuted);
	}

	isToolbuttonDisabled() {
		return Boolean(((typeof this._element == 'object') && this._element.dataset["disabled"] || !this._element) && !this._isDefault);
	}

	setEnabled(isEnabled = true) {
		if (this._element) {
			$(this._element)
				.toggleClass('disabled', !isEnabled)
				.attr('disabled', !isEnabled);

			if (isEnabled) {
				this._element.removeAttribute('data-disabled');
			}
			else {
				this._element.dataset['disabled'] = true;
			}
		}
	}
}
