import dxDataGrid from 'devextreme/ui/data_grid';
import 'devextreme/localization/intl/date';
import 'devextreme/localization/intl/number';
import gridDefaultOptions from "./dxDataGridDefaultOptions";
import $ from 'jquery';
import "devextreme/integration/jquery"; // Necessary for return jQuery elements on grid methods
import DataSource from 'devextreme/data/data_source';
import ArrayStore from 'devextreme/data/array_store';
import { createExpressionButtonVisible, fixRowValues } from './nGrid';
import { CustomStoreArray } from './grid/CustomStoreArray';
import { buttonCellTemplateRef, configureFormat, getEditCellTemplate } from './grid/gridTemplates';
import gridManager from '../ts/components/grid-manager/grid-manager';

export function evaluateGridConfig(config) {
  /** Function References must be reassigned for eval work with correct scope  */
  const buttonCellTemplate = buttonCellTemplateRef;

  const evaluate = (conf, prop) => conf[prop] && (conf[prop] = eval("(" + conf[prop] + ")"));
  const evaluateButton = (button) => (button.visible = button.ExpressaoVisible ? createExpressionButtonVisible(button.ExpressaoVisible) : true);
  const evaluateColumnButton = (column) => (column.buttons.map(i => i.inDropDown).reduce((accumulator, current) => accumulator || current) ? evaluate(column, 'cellTemplate') : delete column.cellTemplate);
  const evaluateKeyExpr = (conf, prop) => conf[prop] && (conf[prop] = conf[prop].split(';'));
  const evaluateEditCellTemplate = (column, prop) => column[prop] = getEditCellTemplate(column);
  const evaluateDateFormat = (column, prop) => column[prop] && configureFormat(column);

  evaluate(config, 'onRowPrepared');
  evaluate(config, 'onEditorPreparing');
  evaluateKeyExpr(config, 'keyExpr');

  for (let column of config.columns) {
    if (column.type) {
      evaluateColumnButton(column)
      for (let button of column.buttons) {
        evaluate(button, 'onClick');
        evaluateButton(button);
      }
    } 
    else {
      evaluate(column, 'calculateDisplayValue');
      evaluate(column, 'cellTemplate');
      evaluate(column, 'headerCellTemplate');
      evaluate(column, 'calculateFilterExpression');  
      evaluateEditCellTemplate(column, 'editCellTemplate');
      evaluateDateFormat(column, 'mask');
    }
  }
}

/**
 * Utility function for grid config, needs to be adapted on migration to base64 data-attribute
 * @param {*} configFromPage 
 * @param {*} configFromRequest 
 */
export function prepareGridConfig(configFromPage, configFromRequest, ...additionalConfig) {
  let configObject = configRemoteOperations({
    ...gridDefaultOptions(),
    ...configFromPage,
    ...configFromRequest,
    ...evaluateGridDetailConfig(configFromRequest.Detail),
    ...additionalConfig.reduce((addConfig, config) => ({...addConfig, ...config}), {})
  });

  return configObject;
}

export function evaluateGridDetailConfig(detailRequest) {
  if (!detailRequest) return {};
  if (detailRequest.outsideGrid) return {};

  let gridDetailConfig = {
    selection: {mode: 'single'},
    cacheEnabled: true,
    grouping: { contextMenuEnabled: true },      
  }
  
  evaluateGridConfig(detailRequest);
  var detailConfig = prepareGridConfig(gridDetailConfig, detailRequest);

  let masterDetailConfig = {
    masterDetail: {
      enabled: true,
      template: function(container, options) {
        let dataSource = new DataSource({
          store: new ArrayStore({
            key: detailConfig.keyExpr,  
            data: detailConfig.items
          }),
          filter: prepareMasterDetailFilter(detailConfig.detailKeyFields.split(";"), detailConfig.masterKeyFields.split(";"), options.data, options.component),
        });

        delete detailConfig.keyExpr;

        let el = (new dxDataGrid('<div>', {
          ...detailConfig,
          dataSource,
          remoteOperations: false
        }));

        el.element().appendTo(container);
          
        return el;
      }
    }
  }

  return masterDetailConfig;
}

/**
 * 
 * @param {Object} config dxDataGrid configuration object 
 * @returns {Object} dxDataGrid configuration object with remoteOperations fixed for dataSource of type array or store
 */
function configRemoteOperations(config) {
	let { dataSource, remoteOperations } = config;

  if (dataSource && (dataSource.hasOwnProperty('length') || (dataSource instanceof CustomStoreArray))) {
    config.remoteOperations = false;
  }
  else if (config.iddata) {
    config.remoteOperations = {
      ...config.remoteOperations,
      grouping: true,
      groupPaging: true,
      summary: true,
    };
  }

	return config;
}

export function prepareMasterDetailFilter(detailKeys, masterKeys, masterData, gridInstance) {
  const getFixedValue = (gridInstance, row, columnName) => {
    const { dataType } = gridInstance.columnOption(columnName);

    return dataType == 'date' ? new Date(row[columnName]) : row[columnName];
  };

  const filter = [];

  for (let i = 0; i < detailKeys.length; i++) {
    if (filter.length > 0) filter.push('and');

    const value = getFixedValue(gridInstance, masterData, masterKeys[i].toUpperCase());

    filter.push([detailKeys[i].toUpperCase(), '=', value]);
  }

  return filter;
}

export function createGrid(id, config) {
  const gridInstance = new dxDataGrid(typeof id == 'string' ? document.getElementById(id) : id, config);

  gridManager.set({
    name: config.nomegrid,
    instance: gridInstance
  });

  return gridInstance;
}

export function createGridBase64() {
  let grids = document.querySelectorAll("[data-dxdatagrid-config");
  
  for (let grid of grids) {
      let config = JSON.parse(atob(grid.dataset["dxdatagridConfig"]));
    
      evaluateGridConfig(config);
      config = prepareGridConfig(config, config.items);
      createGrid(grid, config);
  }
}