import $ from 'jquery';
import { QuickSearch } from './quickSearch';
import { pesquisarModulos } from './searchModulos';
import FormManager from './FormManager';
import moment from 'moment';
import { DynamicImport } from './Utils/DynamicImport';
import { CarregarTreeList } from './treeview/treeview';
import { togglePortletFullscreen } from './fullscreen';
import { FieldChooser } from './form/FieldChooser';
import { createContextMenu } from './ContextMenu';
import { qApp } from './app';
import { CarregarMenu, invalidateMenuCache } from '../ts/components/menu/menu';
import { AvancarEnabled, setarevento, VoltarEnabled } from './nWebUtil';
import qNotifier from '../ts/components/notifier/q-notifier';
import bsCustomFileInput from 'bs-custom-file-input';
import Swal from 'sweetalert2';
import { resizeIframe } from './nIframeUtils';
import { TnCaptcha } from './nCaptcha';
import { LoadCadPageDynamicTab } from '../ts/components/cad-page/cad-page-dynamic-tab';
import SessionManager from './SessionManager';
import CookiesManager from './CookiesManager';
import { ScriptTaskExecutor } from '../ts/utils/script-task-executor/script-task-executor';
import favoriteButton from '../ts/components/favorite-button/favorite-button';
import { verifyCreateView } from '../ts/components/dashboard-products/dashboard-products';

export default class Initializer {
	static init() {
		Initializer._initializerWrapper();
	}

	static _initializerWrapper() {

		Initializer._initFormManagerEvents();

		Initializer._initScriptsTasks();

		Initializer._initSession();

		Initializer._initMenuEvents();

		Initializer._initFavoriteButton();

		Initializer._initSuporteOnline();

		Initializer._initQuickSearch();

		Initializer._initSearchModulos();

		Initializer._initSubheaderDate();

		Initializer._initVerticalTabs();

		Initializer._initSweetAlert();
		
		Initializer._initQApp();

		Initializer._createForms();

		Initializer._initPosicionarWizards();

		Initializer._initClassActive();

		Initializer._initTreeview();

		Initializer._initFieldChooser();

		Initializer._initContextMenuChecklist();

		Initializer._initDynamicDependency();

		Initializer._initBlockPageEvents();

		Initializer._initCookieEvents();

		Initializer._initResizeModalEvents();

		Initializer._initTnCaptchaModal();

		Initializer._initDynamicLoadCadPage();

		Initializer._initTabChangeResetGrid();

		Initializer._createProductView();

		Initializer._initCustomFileInput(); // Deixar por último para executar os handles para arquivos na ordem correta
	}

	static _initFormManagerEvents() {
		const copyEventFromProxy = (array, propName, isSubscriber = true) => {
			while(array.length) {
				const func = array.pop();

				if (isSubscriber) FormManager[propName].subscribe(func);
				else FormManager[propName](func);
			}
		};

		copyEventFromProxy(window.FormManager.afterCreateAllFormsArray, 'afterCreateAllForms');
		copyEventFromProxy(window.FormManager.afterCreateDynamicFormArray, 'afterCreateDynamicForm');
		copyEventFromProxy(window.FormManager.afterSetModalFilterArray, 'afterSetModalFilter');
		copyEventFromProxy(window.FormManager.runAfterCreateAllFormsArray, 'runAfterCreateAllForms', false);
		copyEventFromProxy(window.FormManager.runAfterSetModalFilterArray, 'runAfterSetModalFilter', false);
		
		delete window.FormManager;

		window.FormManager = FormManager;
	}

	static _initScriptsTasks() {
		// run js originally called on script tag
		ScriptTaskExecutor();
	}

	static _initSession() {
		SessionManager.init();
	}

	static _initMenuEvents() {		
		CarregarMenu();

		$('a.link-modulo').on('click', (event) => {
			let moduleName = $(event.currentTarget).attr('href').split('/')[1]; // href="/{{Nome}}/Login"
	
			invalidateMenuCache(moduleName);
		});
	}

	static _initFavoriteButton() {
		favoriteButton.init();
	}

	static _initSuporteOnline() {
		let suporteOnlineEl = $('#suporte-online').exists();
		if (suporteOnlineEl) {				
			suporteOnlineEl.click(function(e) {
				KTApp.blockPage();
				axios.get('nHttpServicos/SuporteOnline')
					.then(
						function(response) {
							KTApp.unblockPage();
							const suporteOnline = response.data.Result;
							if (suporteOnline.HabilitarSuporteOnline) {
								if (suporteOnline.Error) {
									qNotifier.error(suporteOnline.Error, 'Suporte Online');
								} else {
									window.open(suporteOnline.URL);
								}
							}	else {
								qNotifier.warning('Movidesk não está habilitado!', 'Suporte Online');
							}
						})
					.catch(verifyAxiosError);
			});
		} 
	}

	static _initSearchModulos() {				
		$('#search-modulos').on('keyup', (e) => pesquisarModulos());
	}

	static _initQuickSearch() {
		if (KTUtil.getById('kt_quick_search_d')) {
			QuickSearch().init(KTUtil.getById('kt_quick_search_d'));
		}
	}

	static _initSubheaderDate() {
		let subheader = $('#todaySubheader').exists();

		if (subheader) {
			moment.locale('pt-br');
			let now = moment().format('dddd, DD [de] MMMM [de] YYYY');
			let user = subheader.data('user');

			subheader.html(`<span class="font-size-sm">${user.charAt(0).toUpperCase() + user.slice(1)}, ${now.toLowerCase()}. </span>`);
		}
	}

	static _initVerticalTabs() {
		Initializer._onShowBsTabs();
		Initializer._afterShowBsTabs();
	}

	static _onShowBsTabs() {
		$('.kt-wizard__head a[data-toggle="tab"]').on('show.bs.tab', function (e) {
			let dataId = $(e.target).closest('.nav-item').data('id');

			$('#kt_content [data-id="tabitens"]').find('.active,.kt-nav__item--active').removeClass('active kt-nav__item--active show');
			$(`#kt_content [data-id="${dataId}"]`).addClass('kt-nav__item--active show')
		});
	}

	static _afterShowBsTabs() {
		const scrollTo = (selector) => {
			const menuHeight = $('#kt_header').height() + $("#kt_subheader").height();
            
			KTUtil.scrollTo($(selector).get(0), $(selector).offset().top - menuHeight, 500);
		};

		const updateTabMenuHeight = (elem) => {
			let href = $(elem || '[data-id="tabitens"] li:first-child a').attr('href');
			let closest = $(elem).closest('.kt-scroll');
			let maxHeight = $('#kt_aside_menu').height() - ($('#kt_header').height() + 55) - 30;

			closest.css('max-height', $(href).height() > maxHeight ? maxHeight : '500px');
		};

		$('[data-id="tabitens"] a[data-toggle="tab"]').on('shown.bs.tab', function (e) {
			const jQdata = prop => form.data(prop);
			var href = $(this).attr('href');
			var pHeadElem = `${href} .card-header`;
			var form = $(`${href} form`);

			$('#kt_content [data-duplicate-portlet-page="true"]').hide();
			$(href).closest('#kt_content [data-duplicate-portlet-page="true"]').show();

			if ($(pHeadElem).length) scrollTo(pHeadElem);

			updateTabMenuHeight(this);
		});

		updateTabMenuHeight();
	}

	static _initSweetAlert() {
		window.swal = Swal.mixin({
			width: 400,
			heightAuto: false,
			padding: "2.5rem",
			buttonsStyling: false,
			confirmButtonClass: "btn btn-q-primary m-btn m-btn--custom",
			confirmButtonColor: null,
			cancelButtonClass: "btn btn-q-secondary m-btn m-btn--custom",
			cancelButtonColor: null,
			confirmButtonText: 'Sim',
			cancelButtonText: 'Não'
		});
	}

	static _createForms() {
		FormManager.createAllForms();
		setarevento();
	}

	static _initPosicionarWizards() {
		//Carregando treeLists
		$('.data-tree').each(function (index, element) {
			CarregarTreeList(element.id);
		});
		//Posicionando wizards
		$('[data-wizard="true"]').each((i, e) => {
			let startStep = $(e).find('[data-wizard-state="current"]').attr("data-step-index");
			new KTWizard(e, { startStep: startStep || 1 });
			VoltarEnabled(e.dataset['progressposition'] > 2);
			AvancarEnabled(e.dataset['progressposition'] != 100);
		});
	}

	static _initClassActive() {
		$(".twMenu").on("click", function () {
			$(".twMenu").removeClass("active");
			this.classList.add("active");
		})
	}

	static _initTreeview() {
		CarregarTreeList();
	}

	static _initContextMenuChecklist() {
		createContextMenu();
	}
	
	static _initDynamicDependency() {
		Initializer._requestDynamicDependency(window.dynamicDependencies);
	}

	static _clearScripts() {
		$('script[data-remove-script="true"]').remove();
	}

	static _searchDependencies(context) {
		const extractData = ({ moduleName, importName, execute, ...params }) => ({ moduleName, importName, execute, params });
		const convertData = (index, el) => extractData($(el).remove().data());

		return $('[data-dynamic-dependency="true"]', context).map(convertData).toArray();
	}

	static async _requestDynamicDependency(dependencies = []) {
		const promises = [];

		const getDepencency = ({ moduleName, importName, execute, params }) => {
			return execute ?
				DynamicImport.execute(DynamicImport.Import(moduleName, importName), params) :
				DynamicImport.Import(moduleName, importName);
		};

		while(dependencies.length) {
			promises.push(getDepencency(dependencies.pop()));
		}

		Promise
			.all(promises)
			.catch((e) => console.error(e));
	}

	static _initCustomFileInput() {
		bsCustomFileInput.init();
	}

	static _initFieldChooser() {
		FieldChooser.init()
	}

	static _initQApp() {
		qApp.init();
	}

	static _initCookieEvents() {
		$('.kt_login_signin_form').on('submit', () => {
			return CookiesManager.verificarPermitiuCookies();
		});

		$('.cookies-button').on('click', () => {
			CookiesManager.permitirCookies();
		});

		CookiesManager.verificarMostrarCookiesBar();
	}

	static async _initResizeModalEvents() {
		$('[id=framemodal]').on('load', function() { resizeIframe(this); });
	}

	static async _initTnCaptchaModal() {
		$('#CaptchaModal #verify_captcha_modal').on('click', () => {
			TnCaptcha
				.getInstance()
				.RespostaCaptcha($('#captcharesposta').val());
		});
	}

	static _initDynamicLoadCadPage() {
		$('[data-dynamic-load-cad-page-options]').each((index, el) => {
			const config = $(el).data('dynamic-load-cad-page-options');

			el.removeAttribute('data-dynamic-load-cad-page-options');
			
			$(el).on('show.bs.tab', () => {
				LoadCadPageDynamicTab(config);
			});
		});
	}

	static _initTabChangeResetGrid() {
		$(document).on('shown.bs.tab', '[data-repaint-grid-on-click]', ({ target }) => {
			const id = $(target).data('repaint-grid-on-click');

			$(`#${id}`)
				.find('[id*=dataGrid]')	
				.each((index, el) => {	
					if ($(el).length) {
						dxDataGrid.getInstance($(el)).state({});
					}
				});
		});
	}

	static _createProductView() {
		verifyCreateView();
	}

	static _initBlockPageEvents() {
		$(document).on('click', '[data-block-page-on-click]', ({ target, ctrlKey, shiftKey }) => {
			const isLink = target.tagName.match(/a/i) || target.closest('a');
			
			if (isLink && (ctrlKey || shiftKey)) return;

			KTApp.blockPage()
		});
	}
}