import SHA1 from "crypto-js/sha1";
import EncLatin1 from "crypto-js/enc-latin1";
import EncHex from "crypto-js/enc-hex";
import { exportPdf } from "../ts/utils/grid/export-pdf";
import { nFraControl } from "./nFraControl";
import qNotifier from "../ts/components/notifier/q-notifier";

export const exporter = (() => {

	const getAllData = async (gridInstance) => {
		const exportData = await gridInstance.getDataProvider().ready();

		return exportData.filter((row) => row.rowType == 'data').map(({ data }) => ({ ...data }));
	};

	const excel = e => {
		const selectionOnly = false;
		e.component.exportToExcel(selectionOnly);
	};

	const pdf = e => {
		const expPdf = data => {
			KTApp.blockPage();

			setTimeout(async () => { // visual delay for KTApp.blockPage
				try {
					await exportPdf({
						instance: e.component,
						data: data,
						fileName: FormManager.itens.firstUsable().classe,
					});
				}
				catch (e) {
					console.error(e);
				}
				finally {
					KTApp.unblockPage();
				}
			}, 200);
		};

		const hasGroup = (gridComponent) => {
			return gridComponent
				.option('columns')
				.reduce((isGrouped, { dataField }) => (isGrouped || Number.isInteger(gridComponent.columnOption(dataField)?.groupIndex)), false);
		}

		const exportAllData = async (gridComponent) => {
			const { _array, _dataSourceArray } = gridComponent.getDataSource().store();

			expPdf(_array || _dataSourceArray || await getAllData(gridComponent));
		}

		const isConsulta = window.location.pathname.includes('Consulta');

		if (isConsulta) {
			expPdf(
				e.component
					.getVisibleRows()
					.filter(row => row.rowType == 'data')
					.map(row => row.data)
			);
		} else {
			if (hasGroup(e.component)) {
				exportAllData(e.component);
			}
			else {
				swal
					.fire({
						text: 'Deseja exportar para PDF os ' + e.component.option("paging.pageSize") + ' registros selecionados, ou exportar todos registros(pode demorar alguns minutos)?',
						showCancelButton: true,
						width: '800px',
						confirmButtonText: 'Selecionados',
						confirmButtonClass: 'btn btn-sm btn-q-primary',
						cancelButtonText: 'Todos',
						cancelButtonClass: 'btn btn-sm btn-q-secondary'
					})
					.then(async (result) => {
						if (result.value) {
							expPdf(e.component.getVisibleRows().map(row => row.data));
						} else {
							exportAllData(e.component);
						}
					});
			}
		}
	};

	return {
		export(e, action) {
			if (action !== "excel" && action !== "pdf")
				return null;

			return (action === "excel") ? excel(e) : pdf(e);
		}
	};

})();

export const downloadArquivoTemp = function (arquivo) {
    const a = document.createElement('a');
    
	a.href = 'ArquivoTemp/' + arquivo.Arquivo;
    a.download = arquivo.Caption;
    a.style.display = 'none';
    
	document.body.appendChild(a);
    
	a.click();
    a.parentNode.removeChild(a);
    
	KTApp.unblockPage();
}

export const IncluirArquivos = function (formdata, form) {
    $.each(form.fields, function (i, field) {
        if (field.filebase64) {
            formdata = formdata + '&p.' + field.nome + '=' + JSON.stringify(field.getvalue());
        };
    });

    return formdata;
}

export const downloadFileFromData = function (filename, dataType, data) {
    var element = document.createElement('a');
    element.setAttribute('href', dataType + encodeURIComponent(data));
    element.setAttribute('download', filename);

    element.style.display = 'none';
    document.body.appendChild(element);

    element.click();

    document.body.removeChild(element);
}

export const prepareFile64 = function (file) {
    return new Promise((resolve, reject) => {
        var reader = new FileReader();

        reader.onload = function (e) {
            var control = $('#ctrl' + name);
            control.attr('src', e.target.result);
            var result = reader.result;
            console.log(reader.result);
            var n = result.indexOf(",");
            result = result.slice(n + 1, result.length);

            resolve(result);
        }
        reader.onerror = function () {
            reject("");
        }
        reader.readAsDataURL(file);
    })
}
export const loadBase64File = function (files, name, classname, alias) {
	if (files && files[0]) {
		const form = FormManager.byname(classname, alias);		
		const field = form.fieldbyName(name);
		const extension = $(field.componente).attr('accept').split(',');       
		let extencaovalida = false;    

		for (var i = 0; i < extension.length; i++) {	 	
			if ((extension[i]) && (files[0].name)) {				
				if (files[0].name.toUpperCase().match(new RegExp(`${extension[i].toUpperCase()}$`))) {
					extencaovalida = true;
				}								 
			}												   
		}

		if (extencaovalida) {
			var reader = new FileReader();
			reader.onload = function (e) {
				var control = $('#ctrl' + name);
				control.attr('src', e.target.result);
				var result = reader.result;
				var n = result.indexOf(",");
				result = result.slice(n + 1, result.length);
				field.filebase64 = result;
			}
			reader.readAsDataURL(files[0]);	
		} else {
			qNotifier.error(`Arquivo selecionado inválido! Selecione a extensão ${extension.filter(exp => exp)}`);		
			extencaovalida = true;
			field.setvalue('');
			field.componente.siblings().eq(0).text('Nenhum Arquivo Selecionado.');
		}
	}
}

export function  aplicarCriptografia(e, name, classname, alias) {
	let form = FormManager.byname(classname, alias);
	if (e.target.files) {
	  var reader = new FileReader();
	  var file = e.target.files[0];
	  reader.addEventListener('load',function () {
		  //Executa a conversão do arquivo para SHA1 com encoding latino
		  let hash = SHA1(EncLatin1.parse(this.result));
          //Altera o valor do campo texto para salvar no dataset corretamente
		  form.fieldbyName(name).setvalue(hash.toString(EncHex).toUpperCase());
          //Executa diretamente o evento atribuido ao on change para não ficar em loop
		  nFraControl.FraControlChange(e);
	  });
	  if (file) {
		  reader.readAsBinaryString(file);
	  }
	}
}

export const showThumbnail = function (files, name, classname, alias) {
	if (files && files[0]) {
		if ((files[0].type == "image/jpeg") ||
			(files[0].type == "image/jpg") ||
			(files[0].type == "image/bmp")) {
			var reader = new FileReader();
			reader.onload = function (e) {
				var controlImagem = $('#ctrlImg' + name);
				controlImagem.attr('src', e.target.result);
				var result = reader.result;
				var n = result.indexOf(",");
				result = result.slice(n + 1, result.length);
				let form = FormManager.byname(classname, alias);
				
				form.fieldbyName(name).imagevalue = result;
			}
			reader.readAsDataURL(files[0]);
		}
	}
}