import qNotifier from "../ts/components/notifier/q-notifier";
import FormManager from "./FormManager";

export const carregarDadosDoCEP = function (DMClassName, Alias, LinkLookupResultField, InputName) {
    const form = FormManager.byname(DMClassName, Alias);

    let cep = form.getvalue(InputName).replace(/[^\d]+/g, '');

    if (cep.length === 8) {
        KTApp.blockPage();

        let dTipoLogradouro = new $.Deferred();
        let dEstado = new $.Deferred();

        $.when(dTipoLogradouro, dEstado).done(function () {
            KTApp.unblockPage();
        });

        let endereco = { codigoTipoLogradouro: 0, logradouro: '', descricaoTipoLogradouro: '', bairro: '', uf: '', cidade: '', complemento: '' };

        BuscarCEP(cep, function (retorno) {
            const dados = retorno.data;

            if (!("erro" in dados)) {
                const form = FormManager.byname(DMClassName, Alias);

                endereco.complemento = dados.complemento;
                endereco.bairro = dados.bairro;
                endereco.uf = dados.uf;
                endereco.cidade = dados.localidade;
                endereco.logradouro = dados.logradouro.substring(dados.logradouro.indexOf(' ') + 1);
                endereco.descricaoTipoLogradouro = dados.logradouro.substring(0, dados.logradouro.indexOf(' '));

                LinkLookupResultField.split(';').forEach((campo, index) => {
                    if (campo.indexOf('LOGRADTERC') > -1 || campo.indexOf('LOGRADPESSOA') > -1) {
                        form.setvalue(campo, endereco.descricaoTipoLogradouro);
                    } else if (campo.indexOf('ENDER') > -1 && campo.indexOf('COMPL') === -1) {
                        form.setvalue(campo, endereco.logradouro);
                    } else if (campo.indexOf('COMPL') > -1 && campo.indexOf('ENDER') === -1) {
                        form.setvalue(campo, endereco.complemento);
                    } else if (campo.indexOf('BAIRRO') > -1) {
                        form.setvalue(campo, endereco.bairro);
                    } else if (campo.indexOf('SIGLAESTADO') > -1) {
                        form.setvalue(campo, endereco.uf);
                        let field = form.fieldbyName(campo);
                        $(field.componente).blur();
                    } else if (campo.indexOf('CODIGOTIPOLOGRAD') > -1) {
                        BuscarTipoLogradouroPelaDescricao(dTipoLogradouro, endereco.descricaoTipoLogradouro)
                            .done(function (dataPromise) {
                                if (dataPromise && dataPromise.length > 0) {
                                    let field = form.fieldbyName(campo);
                                    field.resultcomponentetext.text(dataPromise[0].DESCRTIPOLOGRAD);
                                    field.setvalue(dataPromise[0].CODIGOTIPOLOGRAD);
                                }
                            });
                    } else if (campo.indexOf('CODIGOMUNIC') > -1) {
                        BuscarMunicipioDoEstado(dEstado, endereco.uf)
                            .done(function (estados) {
                                if (estados && estados.length > 0) {
                                    let cidade = LimparAcentuacao(endereco.cidade).toUpperCase();
                                    for (var i = 0; i < estados.length; ++i) {
                                        let municipio = estados[i];
                                        let nomeMunicipio = municipio.NOMEMUNIC.replace('D\'O', 'Do O').replace('D\'A', 'Do A').replace(' -', '-').replace('-D', ' D');
                                        nomeMunicipio = LimparAcentuacao(nomeMunicipio).toUpperCase();
                                        nomeMunicipio = nomeMunicipio.replace(/&NBSP/g, ' ');
                                        if (nomeMunicipio == cidade) {
                                            let field = form.fieldbyName(campo);
                                            field.resultcomponentetext.text(municipio.NOMEMUNIC.replace(/&nbsp/g, ' '));
                                            field.setvalue(municipio.CODIGOMUNIC);
                                            break;
                                        }
                                    }
                                }
                            });
                    }
                });
            } else {
                qNotifier.error("CEP não encontrado.");
                KTApp.unblockPage();
            }
        });
    } else {
        qNotifier.error('CEP incorreto!');
    }
}

function BuscarCEP(cep, functionSuccess) {
    $.ajax({
        type: "GET",
        url: "https://bigdata.questor.com.br/api/v1/Localizacao/Cep?cep=" + cep,
        dataType: 'json',
        headers: {
            "Authorization": "Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE2NDUxMDAzOTAsImlzcyI6ImJpZ2RhdGEiLCJhdWQiOiJiaWdkYXRhIn0.3KxPIYOVHgvb0S3hRZX2kzRcFScUtAC_DkOosW-7zrM"
        }
    }).done(functionSuccess)
        .fail(function (error) {
            if (error.responseText.indexOf('errors')) {
                try {
                    let errorResponse = JSON.parse(error.responseText);

                    qNotifier.error(errorResponse.errors[0]);
                }
                catch {
                    qNotifier.error("Servidor indisponível.");
                }
            } else {
                qNotifier.error("Servidor indisponível.");
            }
            KTApp.unblockPage();
        })
}

function LimparAcentuacao(newStringComAcento) {
	let string = newStringComAcento;

	const mapaAcentosHex = {
		a: /[\xE0-\xE6]/g,
		A: /[\xC0-\xC6]/g,
		e: /[\xE8-\xEB]/g,
		E: /[\xC8-\xCB]/g,
		i: /[\xEC-\xEF]/g,
		I: /[\xCC-\xCF]/g,
		o: /[\xF2-\xF6]/g,
		O: /[\xD2-\xD6]/g,
		u: /[\xF9-\xFC]/g,
		U: /[\xD9-\xDC]/g,
		c: /\xE7/g,
		C: /\xC7/g,
		n: /\xF1/g,
		N: /\xD1/g,
	};

	for (let letra in mapaAcentosHex) {
		const expressaoRegular = mapaAcentosHex[letra];
		string = string.replace(expressaoRegular, letra);
	}

	return string;
}

export const PrepararMaskDefault = function () {
    PrepararMaskDicionarios();
    $.applyDataMask();
}

function PrepararMaskDicionarios() {
    FormManager.itens.forEach((form) => {
        form.fields.forEach((field) => {
            if (field.dicionario() == 'CEPESTAB') {
                const fieldestado = form.fieldbyNamerDic('SIGLAESTADO');

                if (fieldestado != null) {
                    fieldestado.componente.change(() => SiglaEstadoCepChange(form));
                } 
                else {
                    field.setarmascara('00.000-000');
                }
            }
        });
    });
}

function SiglaEstadoCepChange(form) {
    const estados = ["AC", "AL", "AP", "AM", "BA", "CE", "DF", "ES", "GO", "MA",
        "MT", "MS", "MG", "PA", "PB", "PR", "PE", "PI", "RJ", "RN",
        "RS", "RO", "RR", "SC", "SP", "SE", "TO"
    ];
    const fieldcep = form.fieldbyNamerDic('CEPESTAB');
    const fieldestado = form.fieldbyNamerDic('SIGLAESTADO');

    const a = estados.indexOf(fieldestado.getvalue());
    fieldcep.setarmascara((a == -1) ? '##########' : '00.000-000');
    $.applyDataMask();
}


//Não Usa
function BuscarMunicipioDoEstado(d, uf) {
	var filterOptions = ["SIGLAESTADO", "=", uf];
	var sortOptions = [{ "selector": "NOMEMUNIC", "desc": false }];
	var params = {
		filter: JSON.stringify(filterOptions),
		skip: 0,
		sort: JSON.stringify(sortOptions),
		take: 1000,
		_AClass: "TnGemDCMunicipio"
	};

	$.getJSON('nHttpServicos/DadosConsulta2', params).done(function (data) {
		d.resolve(data.data.items);
	}).fail(function (response) {
        if (verifyAjaxRedirect(response)) return;

		KTApp.unblockPage();
	});

	return d.promise();
}

function BuscarTipoLogradouroPelaDescricao(d, descricao) {
	if (descricao.trim() == '') {
		descricao = 'Outro';
	}
	var filterOptions = ["DESCRTIPOLOGRAD", "=", descricao];
	var params = {
		filter: JSON.stringify(filterOptions),
		skip: 0,
		take: 10,
		_AClass: "TnArqDCTipoLogradouro"
	};

	$.getJSON('nHttpServicos/DadosConsulta2', params).done(function (data) {
		d.resolve(data.data.items);
	}).fail(function (response) {
        if (verifyAjaxRedirect(response)) return;

		KTApp.unblockPage();
	});

	return d.promise();
}
