import WizardManager from "./WizardManager";
import { DynamicImport } from "./Utils/DynamicImport";
import { RequestOperacaoDataset, verifyAjaxRedirect } from "./nRequests";
import { condExpressionValidator } from "./condExpressionValidator";
import { CustomStoreArray } from "./grid/CustomStoreArray";
import ArrayStore from 'devextreme/data/array_store';
import { clearAllReponseBox, handleResponseBox } from "./grid/actions/messageBox/handleReponseBox";
import { ButtonTypes } from "./grid/ButtonTypes";
import { VerificaConfirmacaoAction } from "./nDialog";
import dxButton from 'devextreme/ui/button';
import { downloadArquivoTemp } from './nArquivos';
import FormManager from "./FormManager";
import { Formulario } from "./nFormulario";
import gridManager from "../ts/components/grid-manager/grid-manager";
import { AbrirCadastro, AbrirProcessoModal, CadastroExecuteAction, FormatErrorMessage, FrameModalLoad, loadwidgetsvalues } from "./nWebUtil";
import qNotifier from "../ts/components/notifier/q-notifier";
import { CustomStoreProcessOnDemand } from "../ts/components/widgets/tw-cx-grid/grid/create-process-datasource-on-demand";

export const MontarGridCustomToolButtons = ({ gridToolbarItems, GridButtons, gridComponent, Params, Class, Alias, hasInsertButton }) => {
	const getButton = (toolButton) => {
		const  { TipoAcao, Name} = toolButton;
		
		if (TipoAcao === 'TnActAbrirProcesso') MontarGridCustomToolButtonAbrirProcesso(gridToolbarItems, toolButton, gridComponent, Class, Alias);
		else if (TipoAcao === 'TnActAbrirCadastro') MontarGridCustomToolButtonAbrirCadastro(gridToolbarItems, toolButton, gridComponent);
		else if (TipoAcao === 'TnActOperacaoDataset') MontarGridCustomToolButtonOperacaoDataset(gridToolbarItems, toolButton, gridComponent, Params);
		else if (TipoAcao === 'TnActDatasetReplicar') MontarGridCustomToolButtonDatasetReplicar(gridToolbarItems, toolButton, gridComponent);
		else if (TipoAcao === 'TnAction') MontarGridCustomToolButtonTnAction(gridToolbarItems, toolButton, gridComponent, Class, Alias);
	};
	
	GridButtons?.forEach(getButton);
	
	if (hasInsertButton) injectInsertButton({ gridToolbarItems, Class, Alias });
};

const injectInsertButton = ({ gridToolbarItems, Class, Alias }) => {
	const isEmpty = (obj) => !Object.entries(obj).length;

	gridToolbarItems.push({
		locateInMenu: "auto",
		showText: "inMenu",
		widget: "dxButton", 
		location: "after",
		options: { 
			icon: "add", 
			text: "Novo Registro",
			hint: "Novo Registro", 
			onClick: function() { 	
				const form = FormManager.byname(Class, Alias);

				let filtrofixo = {};
				let chavemestre = null;

				if (form.masterclass != null) {
					const masterform = FormManager.byname(form.masterclass);

					filtrofixo = form.putmastervaluesin(masterform, filtrofixo);
					chavemestre = form.detailfields;

					if (masterform.grid && isEmpty(filtrofixo)) {
						qNotifier.warning('Selecione um item para inserir um novo registro');

						return;
					}

					if ((!masterform.grid) && (masterform.estado != FormState.cadNavegando)) {
						qNotifier.error(`É necessário salvar o cadastro "${masterform.caption}" antes de inserir no detalhe "${form.caption}"`);

						return;
					}
				}
				else if (form.grid && FormManager.filterForCadGrid) {
					filtrofixo = FormManager.filterForCadGrid;

					chavemestre = Object.keys(FormManager.filterForCadGrid);
				}

				AbrirCadastroModal({
					_AClasse: Class,
					AFiltros: filtrofixo,
					AGrid: form.grid,
					ANovo: true,
					AChaveMestre: chavemestre,
					AForm : form
				});											
			}, 				
		},
	});			
};

function MontarGridCustomToolButtonAbrirProcesso(toolbarItems, toolbutton, dxgrid, Class, Alias) {
	MontarToolOptions(toolbarItems, toolbutton, dxgrid,
		function ({ component }) {
			if (component) {
				VerificaConfirmacaoAction(component.option('confirmacao'), function(){
					var actionOptions = component.option('actionOptions');
					AbrirProcessoModal({
						_AClasse: actionOptions.ClasseProcesso,
						form: FormManager.byname(Class, Alias),
						Executar: actionOptions.AbrirExecutar,
					});
				});
			}
		});
}

function MontarGridCustomToolButtonAbrirCadastro(toolbarItems, toolbutton, dxgrid) {
	const { Opcoes, Confirmacao } = toolbutton;
	MontarToolOptions(
		toolbarItems,
		toolbutton,
		dxgrid,
		function ({ component }) {
			VerificaConfirmacaoAction(Confirmacao, function() {
				const { MasterFields = '', DetailFields = '', ClasseDetalhe = '', OpenModal, InsertOnOpen } = Opcoes;

				AbrirCadastroModal({
					_AClasse: ClasseDetalhe,
					ANovo: InsertOnOpen,
					AGrid: dxgrid,
				});
			});
		},
	);
}

function MontarGridCustomToolButtonOperacaoDataset(toolbarItems, toolbutton, dxgrid, params) {
	MontarToolOptions(toolbarItems, toolbutton, dxgrid,
		function ({ component }) {
			if (component) {
				var options = component.option();
				var grid = component.option('grid');
				if (grid) {
					ActionExecutarOperacaoDataSet({
						_AClasse: options.classe,
						_AAction: options.action,
						_AGrid: [options.grid],
						_ADataset: options.actionOptions.datasetNames,
						_AEhProcesso: FormManager.byname(options.classe).tipoObjeto == "ntoProcesso",
						_AParams: params,
						_AOpcoes: options.actionOptions,
					});
				}
			}
		});
}

function MontarGridCustomToolButtonDatasetReplicar(toolbarItems, toolbutton, dxgrid) {
	const onClick = function ({ component }) {
		VerificaConfirmacaoAction(component.option('confirmacao'), function(){
			let { actionOptions, classe, action, grid } = component.option();
			let fieldsToClone = (actionOptions.camposReplicar || '').split(';');

			ActionReplicarDados(classe, action, [grid], actionOptions.datasetNames, fieldsToClone);
		});
	}

	MontarToolOptions(toolbarItems, toolbutton, dxgrid, onClick);
}

function MontarGridCustomToolButtonTnAction(toolbarItems, toolbutton, dxgrid, Class, Alias) {
	MontarToolOptions(
		toolbarItems,
		toolbutton,
		dxgrid,
		function (event) {
			const { NameEvent, Confirmacao } = toolbutton;

			VerificaConfirmacaoAction(Confirmacao, function() {
				CadastroExecuteAction({
					_AClasse: Class,
					_AAlias: Alias,
					_AMasterClass: Class,
					_AAction: NameEvent,
					_ADxEvent: event,
				});
			});
		},
	);
}

function MontarToolOptions(toolbarItems, toolbutton, dxgrid, evento) {
	const { Opcoes, icon, Hint, Caption, Name, Classe, Confirmacao, CloudTipoAcao } = toolbutton;
    
	if (CloudTipoAcao == 'apNone') return;

	if (toolbutton.icon || CloudTipoAcao == 'apSecundaria') {
		toolbarItems.unshift({
			locateInMenu: CloudTipoAcao == 'apPrincipal' ? 'never' : 'always',
			location: 'after',
            widget: "dxButton",
			template: function (config, index, el) {
				const gridName = dxgrid.option('nomegrid');
				const button = new dxButton('<div>', {
					icon: icon,
					hint: Hint,
					text: Caption,
					grid: dxgrid,
					action: Name,
					actionOptions: Opcoes,
					classe: Classe,
					confirmacao: Confirmacao,
					onClick: evento,
					elementAttr: {
						'data-name': Name,
					},
					...gridManager.getCustomToolbarItemConfig(gridName, toolbutton),
				});

				button
					.element()
					.appendTo(el);

				// As of devexpress v21.2 the onToolbarPreparing event is deprecated in favor of "toolbar" property 
				// In case of devexpress update, the gridManager.setAction may be unnecessary
				gridManager.setAction({
					dxButtonInstance: button,
					actionName: Name,					
					gridName,
				});
			}
        });
    }
	else {
		console.error('Toolbutton must have Icon property: ', toolbutton);
	}
}

export const cxGridIncluirFiltroMaster = function (params) {
	const prepareFilter = (/** @type {Formulario}*/ form, currentFilter, additionalFilter) => {
		const newFilter = ObjetoParaFiltroCX([additionalFilter, form.filtrofixo]);
		
		if (!currentFilter) return newFilter ? `[${newFilter}]` : undefined;

		if (currentFilter.startsWith('[[')) return currentFilter.replace(']]', `],"and",${newFilter}]`);
		
		return `[${currentFilter},"and",${newFilter}]`;
	};

    const /** @type {Formulario}*/ formulario = FormManager.byname(params._AClass, params._AAlias);

	if (formulario.masterclass) {
		const /** @type {Formulario}*/ masterForm = FormManager.byname(formulario.masterclass);

		if (masterForm.grid && (!masterForm.grid.getSelectedRowsData().length)) {
			throw new Error('Filter Grid: master is not selected');
		}

		params.filter = prepareFilter(formulario, params.filter, formulario.putmastervaluesin(formulario, {}));
    }
	else if (formulario.grid && FormManager.filterForCadGrid) {
		params.filter = prepareFilter(formulario, params.filter, FormManager.filterForCadGrid);
	}
};

function ObjetoParaFiltroCX(objects) {
	const valueOrDef = (val, def) => [null, undefined, ''].includes(val) ? def : val;
	const joinFilter = (first = '', second = '') => first ? `${first},"and",${second}` : second;
	let filter = '';

	for (const object of objects) {
		for (const key in object) {
			if (object.hasOwnProperty(key)) {
				filter = joinFilter(filter, JSON.stringify([String(key), '=', valueOrDef(object[key], null)]));
			}
		}
	}

	return filter;
}

const getFieldFromGridRow = function (e, field) {
    return (e.row.data[field] || "");
}

export const AddColumnButtonsCadastro = function (columns, classe, chaveprimaria, toolbuttons, disableButtons = []) {
    let form = FormManager.byname(classe, '');

    if (columns[columns.length - 1].type == "buttons") {
        var obj = columns[columns.length - 1];
    }
    else {
        var obj = {};
        obj.type = 'buttons';
        obj.caption = "Ações";
        obj.buttons = [];
    }

	if (!disableButtons.includes("actDataSetEdit")) {
			const chavemestre = form?.masterclass ? form.detailfields : null;

			obj.buttons = obj.buttons.filter(({ text }) => text !== 'Editar');

			obj.buttons.push({
				text: "Editar",
				icon: "las la-pen",
				hint: "Editar o registro",
				visible: true,
				onClick: function (e) {
					AbrirCadastroModal({
						_AClasse: classe,
						AFiltros: rowDataValues(e, chaveprimaria),
						AGrid: e.component,
						ANovo: false,
						AChaveMestre: chavemestre
					});
				}
			});
		}

    for (let toolbtn of toolbuttons) {
        if ((toolbtn.Custom) && (toolbtn.TipoAcao != "TnAction")) {
            if (toolbtn.TipoAcao == "TnActAbrirCadastro") {
                obj.buttons.push(NewCxButtonAbrirCadastro(toolbtn));
            }
            if (toolbtn.TipoAcao == "TnActAbrirProcesso") {
                obj.buttons.push(NewCxButtonAbrirProcesso(toolbtn, form));
            }
			if (toolbtn.TipoAcao == "TnActOperacaoDataset") {
                obj.buttons.push(NewCxButtonOperacaoDataset(toolbtn, form));
            }			
        }
    }

    for (let toolbtn of toolbuttons) {
        if ((toolbtn.Name == "actDataSetDelete") && (toolbtn.disabled == null)) {
            obj.buttons.push({
                text: "Excluir",
                icon: "trash",
                hint: "Excluir o registro",
                visible: true,
                onClick: function (e) { cxGridDeleteRow(classe, e); }
            });
        }
    }
	
    obj.width = Math.max(55, 31 * obj.buttons.length);

    if (columns[columns.length - 1].type == "buttons") {
        columns[columns.length - 1] = obj;
    } else {
        if (obj.buttons.length > 0) {
            columns[columns.length] = obj;
        }
    }
}

function NewCxButtonAbrirProcesso(toolbtn, form) {
	return {
		text: toolbtn.CleanCaption,
		icon: toolbtn.Opcoes.Icon,
		hint: toolbtn.Hint,
		visible: true,
		cssClass: toolbtn.Opcoes.CssClass,
		onClick: function (e) {
			AbrirProcessoModal({
				form,
				_AClasse: toolbtn.Opcoes.ClasseProcesso,
				AParams: rowDataValues(e, toolbtn.Opcoes.MasterFields, toolbtn.Opcoes.DetailParams),
				AGrid: e.component,
				Executar: toolbtn.Opcoes.AbrirExecutar,
			});
		},
	};
}

export const AddColumnButtonsEdicao = function (columns, classe, chaveprimaria, disableButtons = []) {
	let column = {
		type: 'buttons',
		caption: "Ações",
		width: 64,
		buttons: getButtons({ disableButtons, classe, chaveprimaria }),
	};

	if (columns[columns.length - 1].type == "buttons") {
		columns[columns.length - 1] = column;
	} else {
		columns[columns.length] = column;
	}
}

function getButtons(options) {
	let { disableButtons } = options;
	let buttons = [];

	buttons.push(getEditButton({
		disabled: disableButtons.includes(ButtonTypes.actDataSetEdit),
		...options
	}));
	
	buttons.push(getDeleteButton({
		disabled: disableButtons.includes(ButtonTypes.actDataSetDelete),
		...options
	}));

	return buttons;
}

function getDeleteButton({ classe, disabled }) {
	let templateButton = (templateEl, gridEvent) => {
		let button = (new dxButton($('<div>'), {
			disabled,
			stylingMode: "text",
			icon: 'las la-trash',
			size: '18px',
			onClick: function (e) { cxGridDeleteRow(classe, gridEvent); }
		})).element();
		button.find('.dx-button-content').css('padding', 0);
		button.find('.dx-button-content').find('i').css('color', 'var(--orange-q)');
		return button;
	}

	return {
		template: templateButton,
	}
}

function getEditButton({ classe, chaveprimaria, disabled }) {
	let templateButton = (templateEl, gridEvent) => {
		let button = (new dxButton($('<div>'), {
			disabled,
			stylingMode: "text",
			icon: 'las la-pen',
			size: '18px',
			onClick: function (e) { AbrirCadastro(classe, false, rowDataValues(gridEvent, chaveprimaria)); }	
		})).element();
		button.find('.dx-button-content').css('padding', 0);
		button.find('.dx-button-content').find('i').css('color', 'var(--orange-q)');
		return button;
	}

	return {
		template: templateButton,
	}
}

function cxGridDeleteRow(_AClasse, e) {
    e.component.deleteRow(e.row.rowIndex);
}

function NewCxButtonAbrirCadastro(toolbtn) {
    var newBtn = {};
    newBtn.text = toolbtn.CleanCaption;
    newBtn.icon = toolbtn.Opcoes.Icon;
    newBtn.hint = toolbtn.Hint;
    newBtn.visible = true;
    newBtn.cssClass = toolbtn.Opcoes.CssClass;

    newBtn.onClick = function (e) {
			AbrirCadastroModal({
				_AClasse: toolbtn.Opcoes.ClasseDetalhe,
				AFiltros: rowDataValues(e, toolbtn.Opcoes.DetailFields),
				AGrid: e.component,
				ANovo: false,
				AChaveMestre: toolbtn.Opcoes.MasterFields.split(';')
			});
    };
    return newBtn;
}

function NewCxButtonOperacaoDataset(toolButton, form) {
    return {
        text: toolButton.CleanCaption,
        icon: toolButton.icon,
        hint: toolButton.Hint,
        visible: true,
        cssClass: toolButton.Opcoes.CssClass,
        onClick: function (e) {
            e.component.selectRowsByIndexes([e.row.rowIndex]);

            const action = 
				getToolButtonAction({
					toolButton,
					DefinicaoObjetos: {
						Class: toolButton.Classe,
						TipoObjeto: form.tipoObjeto,
					},
				});
			action();
        },
    };
}
//IMPORTAR NO funcoes.js ???
export const AbrirCadastroModal = function ({ _AClasse, AFiltros, AGrid, ANovo, AChaveMestre, AClasseForm, ACustomWidth, ACustomHeigth, ASugerir, shouldRefresh = true, AForm }) {
		let WizardContext = WizardManager.context;

    $('#detalheModal .modal-dialog').width(ACustomWidth ? ACustomWidth + 'px' : '100%');
    $('#framemodal').data('CustomModalHeigth', ACustomHeigth || null);

    let Scope = {
			Class: _AClasse,
			Filtros: AFiltros,
			Sugerir: ASugerir,
			EhModal: true,
			new: ANovo
		};

	clearAllReponseBox();

    if (WizardContext) {
        if (WizardContext.Params) {
            Scope.IDRequest = WizardContext.Params.idrequest;
        }
    }
    if (AChaveMestre) {
        Scope.ChaveMestre = AChaveMestre;
    }

	$('#detalheModal').on('hide.bs.modal', function hidemodal(e) {
		const form = AClasseForm ? FormManager.byname(AClasseForm) : AForm;

		if (AGrid && shouldRefresh) AGrid.refresh();

		form
			?.AfterModal
			?.($('#framemodal')[0].contentWindow.FormManager.itens.firstUsable(), AGrid);

		$('#detalheModal').off('hide.bs.modal', hidemodal);
	});

    FrameModalLoad('Cadastro?Scope=' + JSON.stringify(Scope));
}

export const loadData = function (_AConsulta, options) {
    var mymodal = $('#myModal');
    $.ajax({
        type: 'GET',
        url: "nHttpServicos/DefinicaoConsulta?_AClassName=" + _AConsulta,
        cache: true,
        beforeSend: function () {
            KTApp.blockPage();
        },
        success: async function (request) {
						try {
							let configGrid = await DynamicImport.Import('Grid/ModalGridConfig', 'BuildModalGrid');

							configGrid(request.Result, options);
						}
						catch(e) {
							console.error(e);
							qNotifier.error('Erro ao carregar dados');
						}
        },
        error: function (jqXHR, textStatus, errorThrown) {
			if (verifyAjaxRedirect(jqXHR)) return;

            qNotifier.error(errorThrown);
        },
        complete: function () {
            KTApp.unblockPage();
        }
    });
}

export const dxdatagridDateFilterRevert = function (filterValue, selectedFilterOperation) {
    var dateconv = function (_Avalue, _Acontext) {
        if (_Avalue == null) {
            return _Avalue;
        }
        function pad(d) {
            return (d < 10) ? "0" + d.toString() : d.toString();
        }
        var datestring = pad((_Avalue.getUTCMonth() + 1)) + "/" + pad(_Avalue.getUTCDate()) + "/" + _Avalue.getUTCFullYear();
        if (_Acontext.dataType === "datetime") {
            datestring += ' ' + _Avalue.toLocaleTimeString('pt-BR');
        }
        return datestring;
    };
    if (selectedFilterOperation === "between" && $.isArray(filterValue)) {
        var filterExpression = [[this.dataField, ">=", dateconv(filterValue[0], this)], "and", [this.dataField, "<=", dateconv(filterValue[1], this)]];
        return filterExpression;
    } else {
        var filterExpression = [this.dataField, selectedFilterOperation, dateconv(filterValue, this)];
        return filterExpression;
    };
}

export const dxdatagridDateFilter = function (filterValue, selectedFilterOperation) {
    var dateconv = function (_Avalue, _Acontext) {
        if (_Avalue == null) {
            return _Avalue;
        }
        function pad(d) {
            return (d < 10) ? "0" + d.toString() : d.toString();
        }
        var datestring = pad(_Avalue.getUTCDate()) + "/" + pad((_Avalue.getUTCMonth() + 1)) + "/" + _Avalue.getUTCFullYear();
        if (_Acontext.dataType === "datetime") {
            datestring += ' ' + _Avalue.toLocaleTimeString('pt-BR');
        }
        return datestring;
    };
    if (selectedFilterOperation === "between" && $.isArray(filterValue)) {
        var filterExpression = [[this.dataField, ">=", dateconv(filterValue[0], this)], "and", [this.dataField, "<=", dateconv(filterValue[1], this)]];
        return filterExpression;
    } else {
        var filterExpression = [this.dataField, selectedFilterOperation, dateconv(filterValue, this)];
        return filterExpression;
    };
}

export function createExpressionButtonVisible(expr) {
	var visible = (e) => {
		var validator = new condExpressionValidator(expr, getFieldFromGridRow, e);

		return validator.execute()
	}

	return visible;
}

export function GetColorFromValoresCores(value, valoresCores) {
	/* 
		Must leak Globally (Call comes from backend)
	*/
	let valorCor = valoresCores.split('|').find((valorCor) => { return valorCor.match(new RegExp(`^${value}=`)); });
	if (valorCor) {
		return valorCor.split('=')[1];
	}
	else {
		return '#000';
	}
}

/**
 * 
 * @param {Object} gridComponent dxDataGrid instance
 * @param {Array|Object} rows dxDataGrid rows data
 */

export function fixRowValues(gridComponent, rows) {
	const fixData = (oldData) => {
		const data = { ...oldData };

		Object // in-place data fix 
			.entries(data)
			.forEach(([columnName, columnValue]) => {
				let { dataType } = gridComponent.columnOption(columnName);
	
				formatWithDataType(dataType, data, columnName, columnValue);
			});

		return data;
	};

	return 'length' in rows ? rows.map(fixData) : fixData(rows);
}

/**
 * 
 * @param {string} dataType 'string' | 'date' | 'datetime'
 * @param {Object} data dxDataGrid row data
 * @param {string} columnName 
 * @param {*} columnValue 
 * 
 * Modify data in-place
 */
export function formatWithDataType(dataType, data, columnName, columnValue) {
	if (!columnValue) return; 
		
	switch (dataType) {
		case 'string': return data[columnName] = columnValue.replace(/&nbsp/g, ' ');
		case 'date': return data[columnName] = moment(columnValue, 'MM/DD/YYYY').format('DD/MM/YYYY');
		case 'datetime': return data[columnName] = moment(columnValue, 'MM/DD/YYYY h:m:s').format('DD/MM/YYYY HH:m:s');
	}
}

export function rowDataValues(Grid, chaveprimaria, chavecadastro) {
	const mountPkValues = (data, chaveprimaria, chavecadastro) => {
		const arrayToObjectValues = (pkObject, pk, index) => ({ ...pkObject, [pkCadList[index]]: data[pk].replace(/&nbsp/g, ' ') })
		let pkGridList = chaveprimaria.split(';');
		let pkCadList = (chavecadastro || chaveprimaria).split(';');

		return pkGridList.reduce(arrayToObjectValues, {});  // convert ['CHAVE', 'EMP'] to {'CHAVE': 1, 'EMP': 2}
	}

	let data = JSON.parse(JSON.stringify(Grid.row ? Grid.row.data : Grid.data));
	let gridComponent = Grid.component;

	const fixedData = fixRowValues(gridComponent, data);

	return chaveprimaria ? mountPkValues(fixedData, chaveprimaria, chavecadastro) : fixedData;
}

/*
 * 
 * @param {string} _AClassName Class of form
 * @param {function} _AFuncaoFiltroCX Used to build additional filter (lookup, master-detail, etc)
 * @param {Object} options Pass some config to be used on _AFuncaoFiltroCX 
 * @param {JQuery} options.targetButton Button trigger of search modal
 */
export function createdxDatasource({_AClassName, _AAlias, _AFuncaoFiltroCX, options: { targetButton, gridLookupHandler } = {}}) {
	return {
		//key: keygrid,
		load: function (loadOptions) {
			var d = new $.Deferred();
			var params = {};
			params._AClass = _AClassName;
			params._AAlias = _AAlias;

			if (loadOptions.filter) {
				params.filter = JSON.stringify(loadOptions.filter);
			}

			if (_AFuncaoFiltroCX) {
				try {
					_AFuncaoFiltroCX(params, { targetButton, gridLookupHandler });
				}
				catch (e) {
					if (!e.message.includes('Filter Grid: master is not selected')) console.error(e);
					
					d.resolve([], { totalCount: 0 });

					return d.promise();
				}
			}

			if (loadOptions.sort) {
				params.sort = JSON.stringify(loadOptions.sort);
			}
			if (loadOptions.select) {
				params.select = JSON.stringify(loadOptions.select);
			}
			if (loadOptions.group) {
				params.group = JSON.stringify(loadOptions.group);
			}
			params.skip = (loadOptions.skip) ? loadOptions.skip : 0;
			params.take = (loadOptions.take) ? loadOptions.take : 10;
			params.params = PegarParamsBuscaRapida(_AClassName);
			
			$.getJSON('nHttpServicos/DadosConsulta2', params).done(function (result) {
				if (result.data.Erro) {
					FormatErrorMessage(result.data.Erro);
				}
				d.resolve(result.data.items, { totalCount: result.data.totalcount });
				if (result.data.Widgets) {
					loadwidgetsvalues(result.data.Widgets);
				};
			}).fail(verifyAjaxRedirect);

			return d.promise();
		}
	};
}

function PegarParamsBuscaRapida(_AClassName) {
	const form = FormManager.byname(_AClassName);
	let retorno = {};

	if (form) {
		retorno = form.putvaluesin(retorno);//campos da busca rápida ficam como se fossem do cadastro
	}
	return JSON.stringify(retorno);
}

/*
 * @param {string} _AClasse 
 * @param {string} _AAction 
 * @param {string[] | Object[]} _AGrid Set of grid names or set of dxDataGrid instances
 * @param {string[]} _ADataset Set of dataset names
 * @param {boolean} _AEhProcesso 
 * @param {boolean} _AParams 
 */
export function ActionExecutarOperacaoDataSet(props) {
	const { _AClasse, _AAction, _AGrid, _ADataset, _AEhProcesso, _AParams, _AOpcoes, ignoreGlobalAction = false } = props;

	clearAllReponseBox();

	if (!ignoreGlobalAction && typeof window[_AAction] === 'function') {
		window[_AAction](props);
		
		return Promise.resolve()
	}

	const { DataSend, DataReturn } = _AOpcoes;

	const getFormParams = () => {
		let form = FormManager.byname(_AClasse)
		let savedLookups = saveLookupFields(form);

		fixLookupFields(savedLookups);

		return { 
			savedLookups,
			params: form.getValues(), 
		};
	};

	const getData = (gridComponent, dataSend) => {
		switch (dataSend) {
			case 'dsNone': return [];
			case 'dsSelected': return GridActionHelper.getSelectedRows(gridComponent);
			default: return GridActionHelper.getAllWithEditedData(gridComponent);
		}
	};

	KTApp.blockPage();

	let gridComponents = GridActionHelper.getGridComponents(_AGrid);

	if (!GridActionHelper.hasGridComponents(gridComponents)) {
		KTApp.unblockPage();
		return Promise.resolve();
	}

	let dataset = gridComponents.map((gridComponent, index) => ({
		NomeDataset: ((_ADataset || [])[index] || ''),
		Registros: getData(gridComponent, DataSend),
	}));

	if (GridActionHelper.hasSelectedRows(dataset) || DataSend === 'dsAll' || DataSend === 'dsNone') {
		if (_AEhProcesso) {
			var { params, savedLookups } = getFormParams();
		}

		const successCalback = function ({ data: { Action, ClassName, Dataset, Erro, dialogs, MsgTreeLista, MsgLista, ResultFile } }) {
			if (Erro != null) {
				FormatErrorMessage(Erro);
			}

			if (_AEhProcesso) {
				if (Dataset.every(({ DataReturn }) => DataReturn == 'drAll')) {
					Dataset.forEach(({ NomeDataset, Registros }) => {
						const grid = gridManager.get(NomeDataset);
	
						if (!(grid.getDataSource().store() instanceof CustomStoreProcessOnDemand)) {
							GridActionHelper.setData(grid, Registros);
						}

						handleResponseBox(grid, { MsgTreeLista, MsgLista });

						grid.refresh();	
					});
				} else if (Dataset.every(({ ExecutarProcesso }) => ExecutarProcesso == false)) {
                    Dataset.forEach(({ NomeDataset }) => {
                        const grid = gridManager.get(NomeDataset);
                        handleResponseBox(grid, { MsgTreeLista, MsgLista });
                    });
                } else {
					recoverLookupFields(savedLookups);
					const formulario = FormManager.byname(_AClasse);
					
					formulario.executeAction('ActionExecutarExecute');
				}
			}
			else {
				gridComponents.forEach((gridComponent) => {
					gridComponent.refresh();

					handleResponseBox(gridComponent, { MsgTreeLista, MsgLista })
				});
			}

			if (ResultFile) {
 				downloadArquivoTemp({
					Arquivo: ResultFile,
					Caption: ResultFile.replace(/\S+[\\|\/]/, ''),
				});
			}
		};

		if (_AParams) {
			params = _AParams	
		}

		let request = new RequestOperacaoDataset({
			_ADatamodule: _AClasse,
			_AAction: _AAction,
			_ADataset: dataset,
			_AProcessoParams: params
		});

		return request.executar(successCalback);
	}
	else {
		KTApp.unblockPage();
		FormatErrorMessage('Nenhum registro selecionado. Selecione os registros antes de continuar.');
		return Promise.resolve();
	}
}

/**
 * 
 * @param {*} _AClass 
 * @param {*} _AAction 
 * @param {*} _AGrid 
 * @param {*} _ADataset 
 * 
 * Send all data to server and gets all data modified back
 */
export async function ActionReplicarDados(_AClass, _AAction, _AGrid, _ADataset, _AFieldsToClone) {
	const first = (array) => (array || [])[0];
	const handleResponse = ({ data: { Action, ClassName, Dataset, Erro, dialogs, MsgTreeLista, MsgLista } }, gridComponent) => {
		if (Erro) {
			FormatErrorMessage(Erro);
		}
		else {
			GridActionHelper.setDataToUpdate(gridComponent, (first(Dataset) || {}).Registros, _AFieldsToClone);

			handleResponseBox(gridComponent, { MsgTreeLista, MsgLista })
		}
	}
	const validateFields = (selectedRow) => {
		if (!selectedRow) FormatErrorMessage('Nenhum registro selecionado. Selecione os registros antes de continuar.');

		return !selectedRow;
	}

	try {
		KTApp.blockPage();

		let gridComponent = first(GridActionHelper.getGridComponents(_AGrid));
		let selectedRowData = first(GridActionHelper.getSelectedRowsWithEditedData(gridComponent));
		let selectedRowKey = gridComponent.keyOf(selectedRowData);

		if (validateFields(selectedRowData)) {
			KTApp.unblockPage();
			return;
		}

		let data = [{
			NomeDataset: first(_ADataset) || '',
			Registros: GridActionHelper.getAllWithEditedData(gridComponent)
		}];

		let request = new RequestOperacaoDataset({
			_ADatamodule: _AClass,
			_AAction: _AAction,
			_ADataset: data,
			_ASelected: selectedRowKey,
			_AProcessoParams: GridActionHelper.getParams([gridComponent])
		});

		request.executar(handleResponse, gridComponent);
	}
	catch (e) {
		FormatErrorMessage('Não foi possível executar esta operação');
		console.error(e);
		KTApp.unblockPage();
	}
}

export class GridActionHelper {
  static getGridComponents(grid) { //array of dxInstances or string
    const getGridFromName = (gridName) => {
      try {
        return dxDataGrid.getInstance(`#dataGridConteudo${gridName}`);
      }
      catch {
		console.error(gridName, ' doesn\'t exists as instance of dxDataGrid');

        return null;
      }
    }

    return grid.map((grid) => typeof grid === 'string' ? getGridFromName(grid) : grid);
  }

  static getSelectedRows(gridComponent) {
    let rows = gridComponent.option('selectedRowsQuestor') || JSON.parse(JSON.stringify(gridComponent.getSelectedRowsData()));

	return fixRowValues(gridComponent, rows);
  };

	static getSelectedRowsWithEditedData(gridComponent) {
		let rowsData = gridComponent.getSelectedRowsData();

		return GridActionHelper.getUpdatedData(gridComponent, rowsData);
	}

	static getUpdatedData(gridComponent, data) {
		return data.map((row) => (gridComponent.getController('editing').getUpdatedData(row)));
	}

  static hasGridComponents(gridComponents) {
    return gridComponents.reduce((previous, actual) => (previous && actual));
  }

  static hasSelectedRows(dataset) {
    return dataset.reduce((hasRows, { Registros }) => (hasRows || (Registros.length > 0)), false);
  }

  static getAllData(gridComponent) {
		let store = gridComponent.getDataSource().store();

		if (store instanceof CustomStoreArray) {
			return store._dataSourceArray;
		}
		else if (store instanceof ArrayStore) {
			return store._array;
		}
		else {
			throw new Error("Store doesn't contain all data");
		}
	}

	static getAllWithEditedData(gridComponent) {
		let store = gridComponent.getDataSource().store();

		if (store instanceof CustomStoreArray) {
			let rows = GridActionHelper.getUpdatedData(gridComponent, store._dataSourceArray); 

			return fixRowValues(gridComponent, rows);
		}
		else if (store instanceof ArrayStore) {
			let rows = GridActionHelper.getUpdatedData(gridComponent, store._array);

			return fixRowValues(gridComponent, rows);
		}
		else {
			throw new Error("Store doesn't contain all data");
		}
	}

	static setData(gridComponent, data) {
		let store = gridComponent.getDataSource().store();

		if (store instanceof CustomStoreArray) {
			store.setDataSourceArray(data);
		}
		else if (store instanceof ArrayStore) {
			gridComponent.option('dataSource', data);
		}
		else {
			throw new Error("Store doesn't contain all data");
		}

		gridComponent.refresh();
	}

	/**
	 * 
	 * @param {*} gridComponent dxDataGrid instance
	 * @param {[*]} data rowData
	 * 
	 * Utilizes the 'editing' controller method 'updateFieldValue', which is an internal/undocumented.
	 * Quoting support of devexpress "This method is not documented and we cannot guarantee that it will work properly in future versions"
	 * https://supportcenter.devexpress.com/ticket/details/t426643/how-to-update-selected-rows-that-are-not-displayed-on-the-current-page-in-the-batch
	 */
	static setDataToUpdate(gridComponent, data, fieldsToClone) {
		let store = gridComponent.getDataSource().store();

		if (store instanceof CustomStoreArray) {
			const setColumnValue = (gridComponent, newRow, columnName) => {
				const gridField = {
					key: gridComponent.keyOf(newRow),
					column: gridComponent.columnOption(columnName)
				};

				gridComponent
					.getController('editing')
					.updateFieldValue(gridField, newRow[columnName]);
			};

			KTApp.blockPage();

			const awaitForChanges = data.map(async (rowValue) => {
				let [currentData] = await gridComponent.getDataByKeys([gridComponent.keyOf(rowValue)]);

				fieldsToClone.forEach((field) => {
					let fieldToClone = field.toUpperCase();
					let FieldLookupResult = gridComponent.columnOption(fieldToClone).Lookup.FieldLookup;

					if (FieldLookupResult && currentData[FieldLookupResult] != rowValue[FieldLookupResult]) {
						setColumnValue(gridComponent, rowValue, FieldLookupResult);
					}

					if (currentData[fieldToClone] != rowValue[fieldToClone]) {
						setColumnValue(gridComponent, rowValue, fieldToClone);
					}
				});
			});

			Promise
				.all(awaitForChanges)
				.then(() => {
					gridComponent.getController("data").updateItems();

					KTApp.unblockPage();
				});
		}
		else {
			throw new Error("Store doesn't support method");
		}
	}

	static getParams(gridComponents) {
		const paramsReducer = (params, gridComponent) => ({ ...params, ...(gridComponent.option('form.Params') || {}) });
		
		return gridComponents.reduce(paramsReducer, {});
	}
}