import qNotifier from "../ts/components/notifier/q-notifier";
import FormManager from "./FormManager";
import { DialogsSemResposta, ProcessarDialogsRetorno } from "./nDialog";
import { CarregarDataCadastro, defaulRequestError } from "./nWebUtil";
import { UrlParamBuilder } from "./UrlParamBuilder";

export class RequestOperacaoDataset {
	constructor({ _ADatamodule, _AAction, _ADataset, _AProcessoParams, _ASelected }) {
		this.datamodule = _ADatamodule;
		this.action = _AAction;
		this.dataset = _ADataset;
		this.dialogs = null;
		this.processoParams = _AProcessoParams;
		this.selecionado = _ASelected;
	}

	stringifyParams(params) {
		let paramStr = JSON.stringify(params)
			.replace(/\\r/g, ' ')
			.replace(/\\n/g, '')
			.replace(/&nbsp/g, ' ')

		return `{p=${paramStr}}`;
	}

  /**
   * 
   * @param {function} _ASucessFunction Success request handler
   * @param  {any} additionalParams Params passed to _ASucessFunction
   */
	executar(_ASucessFunction, additionalParams) {
		var aself = this;
		var d = new $.Deferred();
		var params = {
			ClassName: this.datamodule,
			Action: this.action,
			Dataset: this.dataset,
		};

		if (this.processoParams) {
			params.Params = this.processoParams;
		}

		if (this.dialogs) {
			params.dialogs = this.dialogs;
		}

		if (this.selecionado) {
			params.selecionado = this.selecionado;
		}

		$.ajax({
			type: 'POST',
			url: "nHttpServicos/ExecutarOperacaoDataset",
			cache: true,
			dataType: 'html',
			data: this.stringifyParams(params),
			error: (jqXHR, ...args) => {
				if (verifyAjaxRedirect(jqXHR)) return;

				defaulRequestError(jqXHR, ...args)
			},
			success: function (data) {
				if (typeof data === 'object') {
					var retorno = data;
				}
				else {
					var retorno = JSON.parse(data);
				}

				KTApp.unblockPage();

				if (retorno.data.dialogs != null) {
					if (DialogsSemResposta(retorno.data.dialogs)) {
						var AOnRequest = function (Scope) {
							aself.dialogs = Scope.dialogs;
							aself.executar(_ASucessFunction, additionalParams);
						};

						ProcessarDialogsRetorno(retorno, AOnRequest, _ASucessFunction, additionalParams);
					}
					else {
						_ASucessFunction(retorno, additionalParams);
					}
				}
				else {
					_ASucessFunction(retorno, additionalParams);
				}
				
				d.resolve();
			}
		});

		return d.promise();
	}
}

export class RequestDadosCadastro{
    constructor(_ACadastro, _AAlias) {
        this.cadastro = _ACadastro;
        this.alias    = _AAlias;
        this.origem   = '';
        this.detalhes = null;
		this.filtros  = {};
		this.idrequest = '0';
        this.chaveprimariamestre = '';
	}

    addFiltro(_ACampo, _AValor){
        this.filtros[_ACampo] = _AValor;
    }

    SetChavePrimariaMestre(_AChavePrimaria){
        if(_AChavePrimaria){
            this.chaveprimariamestre = '&_AChavePrimariaMestre='+_AChavePrimaria;
        }
        else{
            this.chaveprimariamestre = '';
        }          
    }

    addDetalhes(_AFormulario){

    }

    ehDetalhe(){
		return !this.detalhes;
    }

    executar(_ASucessFunction){
		var builder = new UrlParamBuilder("nHttpServicos/DadosCadastro");

		builder.addParam("_AClassName", this.cadastro);
		builder.addParam("_AAlias", this.alias);
		builder.addParam("_AFiltro", JSON.stringify(this.filtros));
		builder.addParam("_AOrigem", this.origem);
		builder.addParam("_ADetalhes", JSON.stringify(this.detalhes || {}));
		builder.addParam("_AEhdetalhe", this.ehDetalhe());
		builder.addParam("_AIDRequest", this.idrequest + this.chaveprimariamestre);
		builder.addParam("_AFiltroFixo", this._AFiltroFixo);

		return $.ajax({
			'type': 'GET',
			'url': builder.buildUrl(),
			'cache': true,
			'error': (jqXHR, ...args) => {
				if (verifyAjaxRedirect(jqXHR)) return;

				defaulRequestError(jqXHR, ...args);
			},
			'success': function (retorno) {
				_ASucessFunction(retorno);
			}
		});
	}
}

export class RequestRespostaCaptcha {
	constructor(_AID, _AResposta, _Atimer, _AonSucess, _AonError) {
		this.id = _AID;
		this.resposta = _AResposta;
		this.timer = _Atimer;
		this.onSucess = _AonSucess;
		this.onError = _AonError;
	}

	executar() {
		var origem = this;
		var builder = new UrlParamBuilder("nHttpServicos/RespostaCaptcha");
		builder.addParam("_AID", origem.id);
		builder.addParam("_AResposta", origem.resposta);
		this.timer.IniciarExecucao();
		if (this.resposta) {
			var req = $.ajax({
				type: 'GET',
				url: builder.buildUrl(),
				cache: false,
				success: function (request) {
					if (request.data.Erro) {
						qNotifier.error(request.data.Erro);
					}
					origem.onSucess(origem.id);
				},
				error: function (jqXHR, textStatus, errorThrown) {
					if (verifyAjaxRedirect(jqXHR)) return;

					qNotifier.error(errorThrown);
					origem.onError(origem.id);
				}
			});
		};

	}
}

export class RequestExecutarExpression{    
    constructor(_ANome, params){
				this.resultado = null;
				this.FUNC = {
					nome: _ANome,
					params: params
				};        
    }
    
    dorequest(){
        var origem = this;
				var builder = new UrlParamBuilder("nHttpServicos/ExecutarExpression");
				builder.addParam("_AFunc", JSON.stringify(this.FUNC));
        return $.ajax({
            type: 'GET',
            url: builder.buildUrl(),
            cache: false,		
            success: function (request) {
                origem.resultado = request.Result;                    
            },
            error: function (jqXHR, textStatus, errorThrown) {
				if (verifyAjaxRedirect(jqXHR)) return;

                origem.resultado = null;
                qNotifier.error(errorThrown);                    
            }		
        });        
    }

    async executar(){
        await this.dorequest();
	    return this.resultado;
    }
}

export class RequestConsultaCaptcha{
    constructor(_AID, _AonSucess, _AonError){
        this.id       = _AID;
        this.onSucess = _AonSucess;
        this.onError  = _AonError;
    }

    executar(){
        var origem = this;
				var builder = new UrlParamBuilder("nHttpServicos/ConsultaCaptcha");
				builder.addParam("_AID", origem.id);
        if (this.id){
            $.ajax({
                type: 'GET',
                url: builder.buildUrl(),
                cache: false,		
                success: function (request) {
                    origem.onSucess(request.data);                    
                },
                error: function (jqXHR, textStatus, errorThrown) {
					if (verifyAjaxRedirect(jqXHR)) return;

                    qNotifier.error(errorThrown);
                    origem.onError();
                }		
            });
        }
    }
}

export class PostWithForm{
    constructor(Action){
        this.localform = document.createElement('form');
        this.localform.action = Action;
        this.localform.method = 'POST';
		this.localform.style = 'display:none';

        document.body.appendChild(this.localform);
    }

    addParam(Name, value){
        let i = document.createElement('input');
        i.type = 'hidden';
        i.name = Name;
        i.value = value;
        this.localform.appendChild(i);
    }

    submit(){
        this.localform.submit();
    }

	remove() {
		this.localform.remove();
	}
}

export const verifyAjaxRedirect = (/** @type {JQuery.jqXHR} */response = {}) => {
	if (response.responseText?.includes('/home/Inicio')) {
		window.location.reload();

		return true;
	}
};

export const verifyAxiosRedirect = (/** @type {import("axios").AxiosResponse} */response = {}) => {
	const { request } = response;
	const responseUrl = new URL(request.responseURL);

	if (responseUrl.pathname.includes('/home/Inicio')) {
		window.location.reload();

		return true;
	}
};

export const verifyAxiosError = (/** @type {import("axios").AxiosError} */response = {}) => {
	const { data } = response;

	if (data?.error?.EMVCApplication?.Message.includes('/home/Inicio')) {
		window.location.reload();

		return true;
	}
};