﻿import axios from "axios";
import dxDataGrid from "devextreme/ui/data_grid";
import broadcastManager from "../ts/utils/broadcast-manager/broadcast-manager";
import FormManager from "./FormManager";
import SessionManager from "./SessionManager";
import WizardManager from "./WizardManager";
import { scrollToElement } from "../ts/components/cad-page/cad-page-dynamic-tab";
import { GotoWizardNext } from "../ts/components/wizard-navigation/wizard-navigation";
import { clearAllReponseBox } from "./grid/actions/messageBox/handleReponseBox";
import { downloadArquivoTemp } from "./nArquivos";
import { FormState } from "./nCadastro";
import { DialogsSemResposta, ProcessarDialogsRetorno, ShowDialogs } from "./nDialog";
import { ControlType } from "./nFormulario";
import { nFraControl } from "./nFraControl";
import { fixRowValues, loadData, rowDataValues } from "./nGrid";
import { FindLookupValue } from "./nLookUp";
import { setDatePicker } from "./setDatepicker";
import { CarregarTreeList } from "./treeview/treeview";
import qNotifier from "../ts/components/notifier/q-notifier";
import { verifyAjaxRedirect } from "./nRequests";
import { getCurrentPageOptions } from "../ts/components/menu/map-config-from-request";

export function defaulRequestError(request, status, error) {
	if ((request.responseText == '') || (request.responseText == null)) {
		FormatErrorMessage(status + ': Sem resposta do servidor');
	} else {
		FormatErrorMessage(status + ': ' + request.responseText);
	}
	console.log('t3');
	KTApp.unblockPage();
}

export function openRequiredSection(field) {
	for (let fieldEl = field.componente[0] || { parentElement: null }; fieldEl.parentElement; fieldEl = fieldEl.parentElement) {
		if ($(fieldEl).data("accordion")) {
			$(fieldEl).collapse('show');
			break;
		}
	}
}

function CheckChangeTab(_ANaPrimeiraTab) {
	var mudou = false;
	var activenext = (_ANaPrimeiraTab != undefined) ? _ANaPrimeiraTab : false;
	if ($('[data-id="tabitens"]').exists()) {
		var tabs = $('[data-id="tabitens"]').first().find('li');
		for (var i = 0; i < tabs.length; i++) {
			if (activenext) {
				$(tabs[i]).find('a')[0].click();
				activenext = false;
				mudou = true;
			} else {
				for (var j = 0; j < tabs[i].classList.length; j++) {
					if (tabs[i].classList[j] == 'active') {
						activenext = true;
					}
				}
			}
		}
		if (!(mudou)) {
			let itens = $(tabs[1]).find('a');
			if (itens.length > 0) {
				itens[0].click();
			}
		}
	}
}

export async function CarregarDataCadastro(retorno, masterData = {}) {
	let { Class, _AAlias } = (retorno.data || retorno);
	let form = FormManager.byname(Class, _AAlias);

	FormManager.podeEventoChangeFields = false;
	try {
		let dados = Object.assign((retorno.data || retorno), masterData);

		if (dados.dialogs) ShowDialogs(dados.dialogs);
		if (dados.CarregarDadosUltimoRegCadastro == 'sim') FormManager.ultimosDadosCadastro = dados;
		if (dados.State != null) form.changeState(dados.State);

		FormManager.SetarValoresInputs(dados);

		if (dados.datadetalhes != null) {
			dados.datadetalhes.forEach(data => {
				FormManager.SetarValoresInputs(data);
				CarregarConteudo(data.Conteudo);
			})
		}

		const canRefreshGrid = Object.values(dados).map(data => data === "Registro não encontrado").reduce((acc, current) => acc && current, true);

		for (let formdet of FormManager.itens) {
			if (formdet.masterclass == Class) {
				if (formdet.grid && canRefreshGrid) {
					formdet.grid.refresh();
				}
			}
		}

		CarregarConteudo(dados.Conteudo);

		if (dados.StatusRegistro != "naoencontrado") {
			loadwidgetsvalues(dados.Widgets);
		}

		form.CarregarDefinicaoHistorico();
	} finally {
		KTApp.unblockPage();
		FormManager.podeEventoChangeFields = true;
	}
}

function CarregarConteudo(Conteudo) {
	if (Conteudo != null) {
		Conteudo.grids.forEach(({ items, nomegrid }) => {
			const gridEl = $(`#dataGridConteudo${nomegrid}`).exists();

			if (gridEl) {
				const gridComponent = dxDataGrid.getInstance(gridEl);

				gridComponent.beginUpdate();
				gridComponent.option('dataSource', []);
				gridComponent.option('dataSource', items);
				gridComponent.endUpdate();
			}
		});
	}
}

export function loadwidgetsvalues(Widgets) {
	if (Widgets != null) {
		const funcload = function (_AItem) {
			if (_AItem != null) {
				for (let i = 0; i < _AItem.length; i++) {
					for (let j = 0; j < _AItem[i].Itens.length; j++) {
						const item = _AItem[i].Itens[j];

						if (item.Valor != null) {
							const awidget = $('#widget_valor_' + item.Name);
							
							if (awidget.length > 0) {
								awidget.html(item.Valor);

								if (item.ValorColor != null) {
									awidget[0].style.color = '#' + item.ValorColor;
								}
							}
						}

						if (item.Text != null) {
							const awidget = $('#widget_title_' + item.Name);

							if (awidget.length > 0) {
								awidget.html(item.Text);

								if (item.TextColor != null) {
									awidget[0].style.color = '#' + item.TextColor;
								}
							}
						}

						if (item.SubTitulo != null) {
							const awidget = $('#widget_SubTitulo_' + item.Name);

							if (awidget.length > 0) {
								awidget.html(item.SubTitulo);
							}
						}

						if (item.image != null) {
							const awidget = $('#widget_image_' + item.Name);

							if (awidget.length > 0) {
								$(awidget[0]).css('background-image', 'url(data:image/png;base64,' + item.image + ')');
							}
						}

						if (item.ProgressPosition) {
							const awidget = $('#wizard_progress' + item.Name);
							const valor = "calc(" + item.ProgressPosition + "%)";

							$(awidget[0]).css("width", valor);

							VoltarEnabled(item.ProgressPosition != 2);
							AvancarEnabled(item.ProgressPosition != 100);

							for (let step of item.Steps) {
								if (step.StepEnabled) {
									const wizard = WizardManager.getFirst();

									if (wizard) wizard.goTo(step.StepIndex);
								}
							}
						}
						if (item.twTreeView) {
							const awidget = $('#widget_tree_' + item.Name);

							console.log(item);

							if (item.nTreeView) {
								import('jstree').then(() => {
									if (awidget.jstree(true)) {
										awidget.jstree().destroy();
									}
	
									awidget.data('tree-content', item.nTreeView);
	
									CarregarTreeList('widget_tree_' + item.Name);
								});
							}
						}
						if (item.twLista) {
							const awidget = $('#widget_title_' + item.Name);
							const color = "var(--" + item.CorTitulo + ")";
							const aitens = $('#widget_itens_' + item.Name);
							
							awidget.css("border-color", color);
							awidget.css("background-color", color);
							
							aitens.empty();
							
							for (let indice = 0; indice < item.Itens.length; indice++) {
								const novo = '<div class="m-list-timeline__item"> ' +
									'<div id="widget_itens_' + item.Itens[indice].Name + '" class="m-list-timeline__item"> ' +
									'     <span class="m-list-timeline__badge m-list-timeline__badge--success"></span>' +
									'     <span class="m-list-timeline__text">' +
									'         ' + item.Itens[indice].Caption +
									'     </span>' +
									'     <span class="m-list-timeline__time">' +
									'         ' + item.Itens[indice].Valor +
									'     </span>' +
									' </div>' +
									' </div>';

								aitens.append(novo);
							}
						}
					}
				}
			}
		};

		funcload(Widgets.top);
		funcload(Widgets.left);
		funcload(Widgets.rigth);
		funcload(Widgets.bottom);
		funcload(Widgets.client);
	}
}

export function VoltarEnabled(Enabled) {
	const btnvoltar = $('#btnVoltar');

	if (btnvoltar.length != 0) {
		if (Enabled) {
			btnvoltar.removeClass('hide');
			btnvoltar.removeClass('disabled');
		}
		else {
			btnvoltar.addClass('hide');
			btnvoltar.addClass('disabled');
		}
	}
}

export function AvancarEnabled(Enabled) {
	const tnavancar = $('#btnAvancar');

	if (tnavancar.length != 0) {
		if (Enabled) {
			tnavancar.html('AVANÇAR');
			tnavancar.toggleClass('btn-q-primary', true)
			tnavancar.off('click');
			tnavancar.on('click', function() {
				GotoWizardNext(WizardManager.context.DetalheProximo, 'useTimer');
			});
		}
		else {
			tnavancar.html('CONCLUIR');
			tnavancar.toggleClass('btn-q-action', true)
			tnavancar.off('click');
			tnavancar.on('click', function() {
				GotoWizardNext(WizardManager.context.DetalheHome, 'useTimer');
			});
		}
	}
}

export function PegarValorSelecao(_ANome) {
	let result = null;

	if (SessionManager.selecao != null) {
		result = (SessionManager.selecao.hasOwnProperty(_ANome)) ? SessionManager.selecao[_ANome] : null;
	}
	if ((result == null) && (WizardManager.context)) {
		if (WizardManager.context.Params) {
			for (let prop in WizardManager.context.Params) {
				if (prop.toUpperCase() == _ANome.toUpperCase()) {
					result = WizardManager.context.Params[prop];
				}
			}
		}
	}
	if (result) {
		return result;
	} else {
		return '';
	}
}

export function RetornarData() {
	const QuemChamou = FormManager.QuemChamou;
	const gridComponent = GetModalDataTable();

	if (gridComponent) {
		const selectedRows = fixRowValues(gridComponent, gridComponent.getSelectedRowsData());
		
		QuemChamou.isFromGridLookup ? loadGridLookup(QuemChamou, selectedRows) : loadFormLookup(QuemChamou, selectedRows, gridComponent);
	}
	else {
		FormatErrorMessage('Erro ao Selecionar');
	}
}

function loadGridLookup(lookupHandler, selectedRows) {
	if (selectedRows.length) lookupHandler.observer.next(selectedRows);
}

function loadFormLookup(QuemChamou, selectedRows, gridComponent) {
	let Class = QuemChamou.data('dmclassname') || QuemChamou.data('class');
	let Alias = QuemChamou.data('alias');
	let form = FormManager.byname(Class, Alias);

	if (selectedRows.length) {
		if ($(QuemChamou).attr('title') == "Consultar") {
			loadCadastroActConsultar({ Class, Alias, form, selectedRows, gridComponent });
		}
		else {
			PreencherKeyFields({
				selectedRows, 
				form,
				Name: QuemChamou.data('id').slice(4, 50),
				LinkKeyFields: QuemChamou.data('linkkeyfields'),
				KeyFieldControls: QuemChamou.data('keyfieldcontrols'),
				LinkLookupKeyFields: QuemChamou.data('linklookupkeyfields')
			});

			form.CarregarDefinicaoHistorico();
		}
	}

	RestaurarMaskaras();

	if (QuemChamou.data("linklookupdatasetnome")) QuemChamou.focus();
}

function loadCadastroActConsultar({ Class, Alias, form, selectedRows }) {
	const _AFiltros = form.chaveprimaria.reduce((filter, pKey) => ({
		...filter,
		[pKey]: GetModalDataTableSelectedColumnSingleValue({ form, selectedRows, _AColumnName: pKey }),
	}), {});

	$('#myModal').modal("hide");

	FormManager.loadCadastro({ _ACadastro: Class, _AAlias: Alias, _AFiltros }).then(() => form.changeState(FormState.cadNavegando))
}

function PreencherKeyFields({ selectedRows, form, LinkLookupKeyFields, KeyFieldControls, LinkKeyFields, Name }) {
	let Afield = null;
	let _linkLookupKeyFields = LinkLookupKeyFields.split(';');
	let _linkKeyFields = (KeyFieldControls || LinkKeyFields).split(';');

	_linkLookupKeyFields.forEach((linkLookupKeyField, index) => {
		const getColumnValues = (_AColumnName) => GetModalDataTableSelectedColumnValues({ _AColumnName, selectedRows });
		const getColumnSingleValues = (_AColumnName) => GetModalDataTableSelectedColumnSingleValue({ selectedRows, form, _AColumnName });
		
		let isLast = (index == (_linkLookupKeyFields.length - 1));
		let ItemLinkKeyField = isLast ? Name : _linkKeyFields[index];
		let valor = isLast ? getColumnValues(linkLookupKeyField) : getColumnSingleValues(linkLookupKeyField);

		Afield = (form.fieldbyName(ItemLinkKeyField) || form.fieldbyNamerDic(ItemLinkKeyField));

		if (Afield != null) {
			if ([ControlType.ntcLinkAnnotation, ControlType.ntcMemoAnnotation].includes(Afield.controlType)) {
				const component = Afield.componente.get(0);
				const { selectionStart, selectionEnd } = component;
				const oldValue = Afield.getvalue();
				const valBeforeSelection = oldValue.slice(0, selectionStart);
				const valAfterSelection = oldValue.slice(selectionEnd);

				Afield.setvalue(`${valBeforeSelection}${valor}${valAfterSelection}`); 
				
				component.selectionEnd = component.selectionStart = selectionEnd + valor.length;
			}
			else {
				Afield.setvalue(valor);
			}
			//Preenchendo o result field com a consulta
			let searchButton = Afield.linkbutton;
			if (searchButton != null) {
				let LinkLookupResultField = searchButton.data('linklookupresultfield');

				if (LinkLookupResultField) {
					let valorresult = (selectedRows.lenght > 1) 
						? 'Seleção Multipla' 
						: LinkLookupResultField.split(";")
							.map((columnName) => getColumnSingleValues(columnName))
							.join(' | ');

					$(Afield.resultcomponente.find('div.spinner')).remove();
					
					if (valor != '' && valorresult == '' && Afield.controlType == ControlType.ntcLinkEdit) {
						FindLookupValue(Afield.nome, Afield.InstanciaFormulario.classe, Afield.InstanciaFormulario.alias);
					}
					else {
						Afield.resultcomponente.val(valorresult);
						Afield.resultcomponentetext.text(valorresult);
						Afield.componente.data('lookup', valorresult);
					}
				};
			};
		};
	});

	if (Afield) {
		const shouldResetSelection = [ControlType.ntcLinkAnnotation, ControlType.ntcMemoAnnotation].includes(Afield.controlType);
		
		Afield.componente.change();

		const component = Afield.componente.get(0);
		const { selectionStart, selectionEnd } = component;

		setTimeout(() => {
			Afield.focus();

			if (shouldResetSelection) {
				component.selectionEnd = selectionEnd;
				component.selectionStart = selectionStart;
			}
		}, 200);
	}
}

export function GetModalDataTable() {
	return $("#dataGridModal").length ? dxDataGrid.getInstance($("#dataGridModal")) : null;
}

export function GetModalDataTableSelectedColumnSingleValue({ _AColumnName, form, selectedRows }) {
	let value = selectedRows[selectedRows.length - 1][_AColumnName];

	if (form) {
		var field = form.fieldbyNamerDic(_AColumnName);

		if ((field != null) && (field.iscombobox())) {
			var valueAux = value;
			value = field.combovaluefromdescr(value);
			//Verifica para pegar pelo valor(DICSIMNAOINT) 
			if (value === "") {
			  value = field.combocheckvalue(valueAux);
			}
		}
	}

	return (value || '').trim();
}

export function GetModalDataTableSelectedColumnValues({ _AColumnName, selectedRows }) {
	return selectedRows
		.map((row) => (row[_AColumnName] || '').trim())
		.filter(value => value)
		.join(';');
}

export function setarevento() {
	let refDate = setDatePicker();

	configureLookupModal();

	$('#ctrlCODIGOHISTCTB').attr('data-haschange', 'true');
	$('#ctrlCODIGOHISTCTB').change(function (event, options = {}) {
		let { classname, alias } = $(this).data();
		let form = FormManager.byname(classname, alias);

		event.shiftKey = options.shiftKey;

		form.CarregarDefinicaoHistorico(event, { fromKeyPress: !options.loadFromData })
	});

	$(document).on('keydown', '.cad-control', nFraControl.FraControlKeyPress);
	$(document).on('change', '.cad-logchange', nFraControl.FraControlChange);

	return refDate;
}

function configureLookupModal() {
	const isLookupFromGrid = () => (FormManager.QuemChamou && FormManager.QuemChamou.isFromGridLookup);

	$('#myModal').on('hide.bs.modal', function (e) {
		dxDataGrid.getInstance($('#dataGridModal')).dispose()
	});

	$('#myModal').on('show.bs.modal', function (e) {
		var mymodal = $(this);
		var targetButton = $(e.relatedTarget)

		$("#modal-dt").html("");

		if (targetButton.length != 0) {
			var LinkLookupDataSetNome = 'T' + targetButton.data('linklookupdatasetnome');

			let selection = {
				allowSelectAll: true,
				selectAllMode: "page",
				mode: (targetButton.data('linklookupmultiselecao') == false) ? "single" : "multiple"
			};

			loadData(LinkLookupDataSetNome, { selection, targetButton });

			FormManager.QuemChamou = targetButton;
		}
		else if (isLookupFromGrid()) {
			let lookupConfig = FormManager.QuemChamou.Lookup
			let LinkLookupDataSetNome = `T${lookupConfig.LinkLookupDataSetNome}`;
			let LinkLookupMultiSelecao = lookupConfig.LinkLookupMultiSelecao;
			let selection = {
				allowSelectAll: true,
				selectAllMode: "page",
				mode: LinkLookupMultiSelecao ? "multiple" : "single"
			}

			loadData(LinkLookupDataSetNome, { selection, gridLookupHandler: FormManager.QuemChamou });
		}

		mymodal.find('.modal-title').text('');
	});
}

export function CadastroGetImputData(form) {
	if (!form) {
		console.error('Form is undefined');
	
		return {};
	}

	var Scope = {};

	Scope.Class = form.classe;
	
	Scope = form.putvaluesin(Scope);

	if (form.masterclass != null) {
		const masterform = FormManager.byname(form.masterclass);

		Scope = form.putmastervaluesin(masterform, Scope);
	}

	CadastroIncluirDataSelecaoEFiltroFixo(Scope, form);

	return Scope;
};

function CadastroIncluirDataSelecaoEFiltroFixo(Scope, form) {
	if (form.filtrofixo != null) {
		for (var prop in form.filtrofixo) {
			Scope[prop] = form.filtrofixo[prop];
		}
	}
	if (SessionManager.selecao != null) {
		for (var prop in Scope) {
			if ((Scope[prop] == '') || (Scope[prop] == null)) {
				var valor = PegarValorSelecao(prop);
				if ((valor != '') && (valor != null)) {
					Scope[prop] = valor;
				}
			}
		}
	}
	if (form.idrequest) {
		Scope.idrequest = form.idrequest
	}
	else {
		Scope.idrequest = 0
	}
}

export function CadastroExecuteAction(props) {
	const { _AClasse, _AAlias, _AMasterClass, _AAction, _AParams } = props;
	
	var ponteiroFuncao = window[_AAction];
	var form;

	clearAllReponseBox();

	if (ponteiroFuncao) {
		ponteiroFuncao(props);
		return null;
	}
	KTApp.blockPage();

	let Scope = {};
	Scope.Class = _AClasse;

	form = FormManager.byname(_AClasse, _AAlias);
	Scope = form.putvaluesin(Scope);

	if (form.masterclass != null) {
		const masterform = FormManager.byname(form.masterclass);

		Scope = form.putmastervaluesin(masterform, Scope);
	}

	form = FormManager.byname(_AMasterClass);
	
	Scope = form.putvaluesin(Scope);

	form = FormManager.byname(_AClasse, _AAlias);

	CadastroGetImputData(form);

	Scope._AAlias = _AAlias
	Scope._AAction = _AAction;

	const ARequest = MontarRequestDadosCadastro(Scope, 'CadastroExecuteAction', '&_AAction="' + _AAction + '"');

	$.ajax(ARequest);
}

export function OperacaoCadastro({ _AClasse, _Aoperacao, _AAlias, _AAdicParam, _AFileName, _AAfterPostAction }) {
	const valueOrDefault = (val, def) => (val !== undefined && val !== null ? val : def);
	let form = FormManager.byname(_AClasse, _AAlias);
	let Scope = CadastroGetImputData(form);

	KTApp.blockPage();

	Scope._AAlias = _AAlias;
	Scope._EstadoCadastro = form.estado;
	Scope._AfterPostAction = valueOrDefault(_AAfterPostAction, -1);

	if (_Aoperacao == 'CadastroSalvar') { // request with json
		return SaveRegistration({ 
			operation: _Aoperacao,
			data: Scope,
		})
	}
	else { // request with formParam
		let ARequest = MontarRequestDadosCadastro(Scope, _Aoperacao, _AAdicParam, _AFileName);
		
		return $.ajax(ARequest);
	}
}

function SaveRegistration({ data }) {
	let operation = 'CadastroSalvar';

	return axios
		.post(
			`nHttpServicos/${operation}`,
			{
				p: data
			}
		)
		.then((response) => {
			const { headers, data: responseData } = response;
			const { data } = responseData;

			if (data.dialogs && DialogsSemResposta(data.dialogs)) {
				ProcessarDialogsRetorno(responseData, function (Scope) {
					SaveRegistration({
						data: Scope 
					});
				});
			}
			else {
				ProcessarRetornoDados(responseData, operation);
			}
		})
		.catch(({ request }) => {
			const { status, responseText } = request;

			FormatErrorMessage(status + ': ' + ((responseText == '') || (responseText == null)) ? 'Sem resposta do servidor' : responseText);

			KTApp.unblockPage();
		})
}

function MontarRequestDadosCadastro(_AScope, _Aoperacao, _AAdicParam, _AFileName) {
	let ADicional = (_AAdicParam != null) ? _AAdicParam : '';

	return {
		'type': 'POST',
		'url': "nHttpServicos/" + _Aoperacao,
		'cache': true,
		'data': "{p=" + JSON.stringify(_AScope) + ADicional + "}",
		'error': function (request, status, error) {
			if (verifyAjaxRedirect(request)) return;

			console.log(request.responseText);
			FormatErrorMessage(status + ': ' + ((request.responseText == '') || (request.responseText == null)) ? 'Sem resposta do servidor' : request.responseText);

			KTApp.unblockPage();
		},
		'success': function (retorno, textStatus, request) {

			if (request.getResponseHeader('content-type') == "application/octet-stream") { // campo ntcAction 
				const a = document.createElement('a');
				a.href = retorno;
				a.download = _AFileName;
				a.style.display = 'none';
				document.body.appendChild(a);
				a.click();
				a.parentNode.removeChild(a);
				KTApp.unblockPage();
			}
			else {
				if (retorno && retorno.data && retorno.data.dialogs != null) {
					if (DialogsSemResposta(retorno.data.dialogs)) {
						ProcessarDialogsRetorno(retorno, function (Scope) {
							$.ajax(MontarRequestDadosCadastro(Scope, _Aoperacao));
						});
					} else {
						ProcessarRetornoDados(retorno, _Aoperacao);
					}
				} else if (retorno && retorno.data && retorno.data.ArquivosTemp) { // op = CadastroExecuteAction ACtion do tipo TnActSalvarArquivo
					downloadArquivoTemp(retorno.data.ArquivosTemp[0]);
				} else {
					ProcessarRetornoDados(retorno, _Aoperacao);
				}
			}
		}
	};
}

function ProcessarRetornoDados(_ARetorno, _Aoperacao) {
	let formClass = _ARetorno.Class || _ARetorno.data.Class;
	let formAlias = _ARetorno._AAlias || _ARetorno.data._AAlias;
	let form = FormManager.byname(formClass, formAlias);
	let propagateResult = {
		wizardContext: WizardManager.context,
		success: false
	};


	if (_ARetorno.data.Erro != null) {
		FormatErrorMessage(_ARetorno.data.Erro);
		document.location.href = '#result' + _ARetorno.Class + _ARetorno._AAlias;
		KTApp.unblockPage();
		if ((_ARetorno.data.CampoErro) && (form)) {
			var afield = form.fieldbyName(_ARetorno.data.CampoErro);
			if (afield) {
				afield.setatencao();
			}
		}
	}
	else {

		propagateResult.success = true;

		if (form != null) {
			if (_Aoperacao == 'CadastroExcluir') {
				FormManager.CadastroClearFields({ form: form, clearDetails: true });
				loadwidgetsvalues(_ARetorno.data.Widgets);
				form.changeState(FormState.cadLimpo);
			} else {
				if (_Aoperacao != 'CadastroExecuteAction') {
					FormManager.CadastroClearFields({ form: form, clearDetails: false });
				}

				if (_Aoperacao != 'CadastroExecuteAction') {
					if (_Aoperacao != 'CadastroIncluir' && _ARetorno.data._AfterPostAction === 1) {
						form.changeState(FormState.cadNavegando);
					}

					CarregarDataCadastro(_ARetorno, _Aoperacao == 'CadastroIncluir' ? form.getMasterValues() : {});
				}
				else {
					form.changeState(FormState.cadLimpo);
				};
				FormManager.IncluirValoresFixosModal(form);
			};
		}
		if ((_Aoperacao == 'CadastroExcluir') && ($("#dataGrid").length > 0)) {
			dxDataGrid.getInstance($("#dataGrid")).refresh();
		}
		if (_ARetorno.data.Mensagem != null) {
			qNotifier.success(_ARetorno.data.Mensagem);
			if (_Aoperacao != 'CadastroExecuteAction') {
				if (form && !form.isFormRepeater) {
					if (_ARetorno.data.autoinsert != undefined && _ARetorno.data.autoinsert != true) {
						CheckChangeTab(!form.ehdetalhe);
					}
				}
			}
		}
		CarregarConteudo(_ARetorno.data.Conteudo);
		if (_ARetorno.data._AfterPostAction == 3) {
			window.location = FormManager.LinkConsultaOrigem;
		}

		KTApp.unblockPage();

        const getNameTab = function(aNome) {
			if (aNome.indexOf("_") > 0) {
				return aNome.substring(0, aNome.indexOf("_"));
			} else {
			    return aNome;	
			}
		}

		if (form) {
			form.focusFirstField();

			if (_Aoperacao == 'CadastroExcluir') {
				if (form.AfterDelete) {
					form.AfterDelete();
				}
				form.afterDelete.next();
			}

			if (_Aoperacao == 'CadastroSalvar') {
				if (form.AfterPost) {
					form.AfterPost();
				}

				broadcastManager.postMessage({
					type: 'save',
					data: {
						Class: form.classe,
						Alias: form.alias,
						chaveprimaria: form.chaveprimaria,
						data: _ARetorno.data.insertedData,
					}
				});

				form.AfterSave.next({
					...propagateResult,
					data: _ARetorno.data.insertedData
				});

				var aDetalhe = FormManager.itens.filter(item => (item.masterclass === form.nome && $('#' + getNameTab(item.nome) + ':visible').length > 0))[0];
				if (aDetalhe != undefined) {
					if (aDetalhe.fields.length != 0) {
						aDetalhe.focusFirstField()
					} else {
						scrollToElement($('#' + aDetalhe.classe + aDetalhe.alias));
					}
				}
			}

			if (_Aoperacao == 'CadastroExecuteAction') {
				form.afterExecuteTnAction.next();
			}
		}
	};

	document.location.href = '#result' + formClass + formAlias;
	RestaurarMaskaras();
}

export function FormatErrorMessage(_AText) {
	var n = _AText.indexOf('Você não esta logado');
	if (n != -1) {
		window.location = "/home/inicio";
	} else {
		qNotifier.error(_AText);
	}
}

export function DoExecuteReport(_AClass, _AAlias, _AReport) {
	let form = FormManager.byname(_AClass, _AAlias)
	let p = CadastroGetImputData(form);

	DoExecuteReportWithParams(_AReport, p);
}

export function DoExecuteReportWithParams(_AReport, _AParams) {
	let AScope = {};
	
	_AParams.Class = _AReport;

	AScope.Params = _AParams;
	AScope.Class = _AReport;

	Object
		.keys(AScope.Params)
		.forEach((key) => {
			if (AScope.Params[key] === null) AScope.Params[key] = '';
		});

	window.open("Relatorio?Scope=" + encodeURIComponent(JSON.stringify(AScope)), '_blank');
}

export function AjaxAutoCompleteField(Field) {
    const form = Field.InstanciaFormulario;

    if (!Field.canAutoComplete) { // Resolve multiple event focus triggering 
        Field.canAutoComplete = true;

        $.ajax({
            type: 'GET',
            url: `nHttpServicos/CadastroExecuteAutoComplete?_AClass=${form.classe}&_AFieldName=${Field.nome}&_AFiltroChave=${JSON.stringify(form.MontarFiltroChavePrimaria())}`,
            cache: true,
            success: (AData) => {
                form.loadfromdata(AData.data);
                Field.canAutoComplete = false;
            },
			error: verifyAjaxRedirect
        });
    }
}

function RestaurarMaskaras() {
	//Necessário fonte abaixo pois o DOM se perde ao mostrar a consulta modal e perde as maskaras
	$('.form-control').each(function (I) {
		if (!$(this).data('haslinklookup')) {
			$(this).unmask();
			$.applyDataMask(this);
		}
		// $(this).unmask();
	});
	// $.applyDataMask();
}

export function AJaxCadastroExecuteMethodGetProperty(_AClass, _AMethod, _AValues, _ANamePropertys, _AFuncaoSucesso, _AFuncaoErro) {
	$.ajax({
		'type': 'GET',
		'url': "nHttpServicos/CadastroExecuteMethodGetProperty?_AClass=" + _AClass +
			"&_AMethod=" + _AMethod +
			"&_AValues=" + _AValues +
			"&_ANamePropertys=" + _ANamePropertys,
		'cache': true,
		'error': function ErroAjax(request, status, error) {
			KTApp.unblockPage();

			if (verifyAjaxRedirect(request)) return;
			
			if (_AFuncaoErro == null) {
				qNotifier.error(status + ': ' + ((request.responseText == '') ? 'Sem resposta do servidor' : request.responseText));
			} else {
				_AFuncaoErro(request, status, error);
			}

		},
		'success': function (retorno) {
			KTApp.unblockPage();
			if ((retorno.data.Erro == '') || (retorno.data.Erro == null)) {
				_AFuncaoSucesso(retorno)
			} else {
				qNotifier.error(retorno.data.Erro);
			}
		}
	});
}

export function AJaxProcessoExecuteMethodGetProperty({ _AClass, _AMethodName, _AValues, _ANamePropertys = '', onSuccess, onError, blockPage = true }) {
	if (blockPage) KTApp.blockPage();

	$.ajax({
		'type': 'GET',
		'url': "nHttpServicos/ProcessoExecuteMethodGetProperty?_AClass=" + _AClass +
			"&_AMethod=" + _AMethodName +
			"&_AValues=" + _AValues +
			"&_ANamePropertys=" + _ANamePropertys,
		'cache': true,
		'error': function ErroAjax(request, status, error) {
			if (verifyAjaxRedirect(request)) return;

			if (blockPage) KTApp.unblockPage();

			if (typeof onError !== 'function') {
				qNotifier.error(`${status}: ${request.responseText == '' ? 'Sem resposta do servidor' : ''}`);
			} 
			else {
				onError(request, status, error);
			}
		},
		'success': function (retorno) {
			if (blockPage) KTApp.unblockPage();

			if ((retorno.data.Erro == '') || (retorno.data.Erro == null)) {
				onSuccess(retorno);
			} 
			else {
				qNotifier.error(retorno.data.Erro);
			}
		}
	});
}

export function RemoverTabCadastro(_ACaption) {
	$(`[data-id="tab${_ACaption}"]`).hide();
}

export function RestoreTabsCadastro() {
	$('[data-id="tabitens"] li').show();
}

export function ConsultaAjaxDeleteReg(e) {
	let { Class, Alias } = e.component.option().form;
	let form = FormManager.byname(Class, Alias);

	if (form.BeforeDelete) {
		let retorno = form.BeforeDelete(e, rowDataValues(e));
		if (retorno == undefined || (typeof retorno === 'boolean' && retorno)) {
			e.cancel = true;
			DeleteRegistroConsulta(e);
		} else if (retorno.then instanceof Function) {
			e.cancel = retorno.then(() => {
				DeleteRegistroConsulta(e);
				return true;
			}).catch(() => {
				return true;
			});
		} else {
			e.cancel = $.Deferred().reject().catch(() => { return true; });
		}
	} else {
		e.cancel = true;
		DeleteRegistroConsulta(e);
	}
}

function DeleteRegistroConsulta(e) {
	let { Class, Alias, consultaDMClass } = e.component.option().form;
	let form = FormManager.byname(Class, Alias);

	KTApp.blockPage();
	
	let Scope = rowDataValues(e);
	
	Scope.Class = consultaDMClass || Class;
	Scope._AAlias = "";

	const ARequest = {
		'type': 'POST',
		'url': "nHttpServicos/CadastroExcluir",
		'cache': true,
		'data': "{p=" + JSON.stringify(Scope) + "}",
		'error': function (request, status, error) {
			if (verifyAjaxRedirect(request)) return;

			KTApp.unblockPage();
			console.log(request.responseText);
			FormatErrorMessage(status + ': ' + ((request.responseText == '') || (request.responseText == null)) ? 'Sem resposta do servidor' : request.responseText);
			KTApp.unblockPage();
		},
		'success': function (retorno) {
			KTApp.unblockPage();
			
			if (retorno.data.dialogs != null) {
				let ASemResposta = false;
				
				for (var i = 0; i < retorno.data.dialogs.length; i++) {
					if (retorno.data.dialogs[i].Resposta == '') {
						ASemResposta = true;
					}
				}

				if (ASemResposta) {
					ProcessarDialogsRetorno(retorno, function (Scope) {
						$.ajax(MontarRequestDadosCadastro(Scope, 'CadastroExcluir'));
					});
				} else {
					ProcessarRetornoDados(retorno, 'CadastroExcluir');
				}
			}

			if (retorno.data.Erro != null) {
				if (!retorno.data.Erro.includes("Dialog a responder")) {
					qNotifier.error(retorno.data.Erro);
				}
			}
			else {
				e.component.refresh();
				qNotifier.success('Registro excluído com sucesso.');

				if (form) {
					if (form.AfterDelete) form.AfterDelete();
					form.afterDelete.next();
				}
			}
		}
	};

	$.ajax(ARequest);
}

export function AbrirProcesso(_AClasse, ANovaJanela, Params) {
	const { pageClass, pagePathName, pageWorkspaceClass } = getCurrentPageOptions();
	const Scope = {
		Class: _AClasse,
		...($IsIntegration ? { IsIntegration: true } : {})
	};

	clearAllReponseBox();

	if (_AClasse.toUpperCase() == pageClass || _AClasse.toUpperCase() == pageWorkspaceClass) {
		Scope.pathName = pagePathName;
	}

	if (Params) {
		Scope.Params = Params;
	}

	if (ANovaJanela) {
		window.open(`Processo?Scope=${JSON.stringify(Scope)}`, '_blank');
	}
	else {
		KTApp.blockPage();

		window.location = `Processo?Scope=${JSON.stringify(Scope)}`;
	};
}

export function AbrirCadastro(_AClasse, ANovaJanela, AFiltros) {
	const { pageClass, pagePathName, pageWorkspaceClass} = getCurrentPageOptions();
	const Scope = {
		Class: _AClasse,
		[AFiltros ? 'Filtros' : 'new']: AFiltros || true,
		...($IsIntegration ? { IsIntegration: true } : {})
	};

	clearAllReponseBox();

	if (pageClass == _AClasse.toUpperCase() || pageWorkspaceClass == _AClasse.toUpperCase()) {
		Scope.pathName = pagePathName;
	}

	if (ANovaJanela) {
		window.open("Cadastro?Scope=" + JSON.stringify(Scope), '_blank');
	}
	else {
		KTApp.blockPage();

		window.location = "Cadastro?Scope=" + JSON.stringify(Scope);
	};
}

export function AbrirProcessoModal({ _AClasse, AParams, AGrid, ACustomWidth, ACustomHeigth, shouldRefresh = true, form, sender , Executar}) {
	$('#detalheModal .modal-dialog').width(ACustomWidth ? ACustomWidth + 'px' : '100%');
	$('#framemodal').data('CustomModalHeigth', ACustomHeigth || null);

	let Scope = {
		Class: _AClasse,
		EhModal: true,
		AbrirExecutar: Executar,
	};

	clearAllReponseBox();
	
	if (AParams) {
		Scope.Params = AParams;
	}

	$('#detalheModal').on('hide.bs.modal', function hideModal (e) {
		const AForm = form || FormManager.byname(sender);

		if (shouldRefresh) {
			if (AGrid) {
				AGrid.refresh();
			}
			else if (AForm && AForm.tipoObjeto != 'ntoProcesso') {
				AForm.executeAction('actDataSetRefreshExecute');
			}
		}

		if (AForm && AForm.AfterModal) {
			AForm.AfterModal($('#framemodal')[0].contentWindow.FormManager.itens.firstUsable());
		}

		$('#detalheModal').off('hide.bs.modal', hideModal);
	});

	FrameModalLoad('Processo?Scope=' + JSON.stringify(Scope));
}

export function FrameModalLoad(_AUrl) {
	$('#framemodal').contents().find('body').html('');
	$('#framemodal').attr('src', _AUrl);
	$('#detalheModal').modal();
}

export function LocateProp(obj, name) {
	var result = null;
	if (Object.prototype.hasOwnProperty.call(obj, name)) {
		return obj[name];
	}

	for (var prop in obj) {
		if (Object.prototype.hasOwnProperty.call(obj, prop)) {
			if (typeof obj[prop] == "object") {
				result = LocateProp(obj[prop], name)
				if (result) {
					return result;
				}
			}
		}
	}
}

export function ProcessoExecuteAction(target, actionname, sender, form) {
	KTApp.blockPage();

	FormManager.podeEventoChangeFields = false;
	
	const params = {};

	params.actionname = actionname;
	params.sender = sender;
	params.FormularioAtual = form.classe;

	if (WizardManager.context) {
		if (WizardManager.context.Datasets) {
			for (let i = 0; i < WizardManager.context.Datasets.length; i++) {
				const dataset = $('#dataGridConteudo' + WizardManager.context.Datasets[i].Nome);
				
				if (dataset.length > 0) {
					WizardManager.context.Datasets[i].Valor = dxDataGrid.getInstance(dataset).option("dataSource");
				};
			};
		};
		params.WizardContext = WizardManager.context;
	};

	if (SessionManager.selecao != null) {
		for (var prop in SessionManager.selecao) {
			var field = form.fieldbyName(prop.toUpperCase());
			if (!field) {
				field = form.fieldbyName('P' + prop.toUpperCase());
			}
			if (field) {
				if (!field.visible) {
					field.setvalue(SessionManager.selecao[prop]);
				}
			}
		}
	}

	if (WizardManager.context) {
		if (WizardManager.context.Params) {
			for (var prop in WizardManager.context.Params) {
				var field = form.fieldbyName(prop.toUpperCase());
				if (!field) {
					field = form.fieldbyName('P' + prop.toUpperCase());
				}
				if (field && !field.isFile) {
					if (!field.visible) {
						field.setvalue(WizardManager.context.Params[prop]);
					}
				}
			}
		}
	}

	params.formparams = {};
	params.formparams = form.putvaluesin(params.formparams);

	$.ajax({
		'type': 'POST',
		'data': '{p= ' + JSON.stringify(params) + '}',
		'url': "nHttpServicos/ProcessoExecuteAction",
		'cache': false,
		'contentType': false,
		'processData': false,
		'error': function (request, status, error) {
			if (verifyAjaxRedirect(request)) return;
			
			try {
				if ((request.responseText == '') || (request.responseText == null)) {
					FormatErrorMessage(status + ': Sem resposta do servidor');
				} else {
					FormatErrorMessage(status + ': ' + request.responseText);
				}
				console.log('t3');
			}
			catch (erro) {
				qNotifier.error('Erro de javascript: ' + erro);
			}
			finally {
				KTApp.unblockPage();
				FormManager.podeEventoChangeFields = true;
			}
		},
		'success': function (retorno) {
			try {
				if (retorno.data.Erro) {
					qNotifier.error(retorno.data.Erro);
				}
				WizardManager.setContext(retorno.data.WizardContext);
				$('.wizardtab').removeClass('active');
				$('#' + WizardManager.context.DetalheAtivo).addClass('active');

				const form = FormManager.byname(WizardManager.context.DetalheAtivo, '');

				form.loadfromdata(retorno.data.formparams);

				if (WizardManager.context) form.AfterGoToWizardItem.next();

				loadwidgetsvalues(retorno.data.Widgets);
				for (var i = 0; i < retorno.data.Detalhes.length; i++) {
					var item = retorno.data.Detalhes[i];
					loadwidgetsvalues(item.Widgets);
					if (item.FiltroFixo) {
						var formitem = FormManager.byname(item.ClasseDetalhe, '');
						formitem.filtrofixo = item.FiltroFixo;
					}
					if (item.grids) {
						for (var j = 0; j < item.grids.length; j++) {
							try {
								var datagrid = dxDataGrid.getInstance($("#dataGridConteudo" + item.grids[j].nomegrid));

								if (datagrid) {
									datagrid.option("dataSource", item.grids[j].items);
								}
							}
							catch (e) {
								continue;
							}

						}
					}
				}

			}
			catch (erro) {
				qNotifier.error('Erro de javascript: ' + erro);
			}
			finally {
				KTApp.unblockPage();
				FormManager.podeEventoChangeFields = true;
			}
		}
	});
}

export function padBase64(str64) {
	return str64.padEnd(str64.length + 4 - ((str64.length % 4) || 4), "=");
}