import { href } from "../ts/components/menu/map-config-from-request";

function getMenuSession() {
  try {
    return JSON.parse(sessionStorage.getItem('itenspesquisamenu'));
  }
  catch {
    return [];
  }
}

async function getUntil(func, attempts, def) {
  const result = func();
  if (result) return result;
  if (attempts < 0) return def;

  return new Promise((resolve) => {
    setTimeout(() => {
      resolve(getUntil(func, attempts -1));
    }, 1000);
  });
}

async function CarregarMenuQS(query) {
  const menuItens = await getUntil(getMenuSession, 10, []);
  const normalize = (str = '') => str.removeDiacritics().replace(/[.*+?^${}()|[\]\\]/g, '\\$&').toLowerCase();
  const isDebug = typeof $ModoDebug == 'boolean';
  let filteredMenu = menuItens.filter(item => (normalize(item.label).match(normalize(query)) || (isDebug && normalize(item.link).match(normalize(query)))));
  let template = `<div class="kt-quick-search__result">`;
  let groupBy = function(xs, key) {
    return xs.reduce(function(rv, x) {
      (rv[x[key]] = rv[x[key]] || []).push(x);
      return rv;
    }, {});
  };

  if (filteredMenu.length) {
    let groupedMenus = groupBy(filteredMenu, 'section');
    
    for (let section in groupedMenus) {
      
      template += `<div class="categoria-quick-search">${section}</div>`;

      groupedMenus[section].forEach(item => {
        template += `<div>
                        <a data-block-page-on-click href="${item.link.replace(/"/g, '&quot;')}">
                            <div class="d-flex align-items-center flex-grow-1 mb-2 justify-content-between">
                                <div class="d-flex flex-column ml-3 mt-2 mb-2">
                                    <span class="titulo-quick-search font-weight-bold text-dark">
                                        ${item.label}
                                    </span>
                                    <span class="localizacao-quick-search font-size-sm text-muted">
                                      ${section} » ${item.label}
                                    </span>
                                </div>
                            </div>
                        </a>
                    </div>`
      });
    }
  }
  else {
    template += `
    <div class="text-muted">
      <h3>Nenhum menu encontrado.</h3>
    </div>`;
  }

  template += `</div>`;
  return template;
}

export function populateQuickSearch({ listaMenu, hasModules, hasTreeview }) {
  const itemspesquisa = [];

  const insertItem = (item, subSectionText) => {
    const description = item.Text.trim();

    itemspesquisa.push({
      label: description,
      value: description,
      link: href(item, hasModules),
      section: subSectionText
    })
  };

  const mapItemsFromSubsection = (subsectionOrMenu, sectionText) => {
    if (hasTreeview && subsectionOrMenu.Text.trim().match(/^outros$/i)) {
      mapItemsFromTreeview(subsectionOrMenu, subsectionOrMenu.Text);
    }
    else if (subsectionOrMenu.Childs) {
      subsectionOrMenu.Childs.forEach((menu) => insertItem(menu, subsectionOrMenu.Text));
    }
    else {
      insertItem(subsectionOrMenu, sectionText);
    }
  };

  const mapItemsFromTreeview = (item, text) => {
    if (item.Childs) {
      item.Childs.forEach((child) => mapItemsFromTreeview(child, text));
    }
    else {
      insertItem(item, text);
    }
  };

  listaMenu.forEach(section => {
    section.Childs.forEach((subsectionOrModule) => { 
      try {
        if (subsectionOrModule.Modulo && subsectionOrModule.Childs) {
          subsectionOrModule.Childs.forEach((subSection) => {
            mapItemsFromSubsection(subSection, section.Text);
          })
        }
        else {
          mapItemsFromSubsection(subsectionOrModule, section.Text);
        }
      }
      catch (e) {
        console.error(e);
        console.log('Verificar a estrutura dos menus'); 
      }
    })
  });

  sessionStorage.setItem('itenspesquisamenu', JSON.stringify(itemspesquisa));
}

export const QuickSearch = function() {
  var target;
  var form;
  var input;
  var closeIcon;
  var resultWrapper;
  var resultDropdown;
  var resultDropdownToggle;
  var inputGroupAppend;
  var query = '';

  var hasResult = false;
  var timeout = false;
  var isProcessing = false;
  var requestTimeout = 200; // ajax request fire timeout in milliseconds 
  var spinnerClass = 'spinner spinner-sm spinner-primary';
  var resultClass = 'quick-search-has-result';
  var minLength = 2;

  var showProgress = function () {
    isProcessing = true;
    if (closeIcon) {
      KTUtil.hide(closeIcon);
    }
    KTUtil.addClass(inputGroupAppend, spinnerClass);
  }

  var hideProgress = function () {
    isProcessing = false;
    KTUtil.removeClass(inputGroupAppend, spinnerClass);

    if (closeIcon) {
      if (input.value.length < minLength) {
        KTUtil.hide(closeIcon);
      } else {
        KTUtil.show(closeIcon, 'flex');
      }
    }
  }

  var showDropdown = function () {
    if (resultDropdownToggle && !KTUtil.hasClass(resultDropdown, 'show')) {
      $(resultDropdownToggle).dropdown('toggle');
      $(resultDropdownToggle).dropdown('update');
    }
  }

  var hideDropdown = function () {
    if (resultDropdownToggle && KTUtil.hasClass(resultDropdown, 'show')) {
      $(resultDropdownToggle).dropdown('toggle');
    }
  }

  var processSearch = async function () {
    if (hasResult && query === input.value) {
      hideProgress();
      KTUtil.addClass(target, resultClass);
      showDropdown();
      KTUtil.scrollUpdate(resultWrapper);

      return;
    }

    query = input.value;

    KTUtil.removeClass(target, resultClass);
    showProgress();
    var res = await CarregarMenuQS(query);
    hasResult = true;
    hideProgress();
    KTUtil.addClass(target, resultClass);
    KTUtil.setHTML(resultWrapper, res);
    showDropdown();
    KTUtil.scrollUpdate(resultWrapper);
  }

  var handleCancel = function (e) {
    input.value = '';
    query = '';
    hasResult = false;
    KTUtil.hide(closeIcon);
    KTUtil.removeClass(target, resultClass);
    hideDropdown();
  }

  var handleSearch = function () {
    if (input.value.length < minLength) {
      hideProgress();
      hideDropdown();

      return;
    }

    if (isProcessing == true) {
      return;
    }

    if (timeout) {
      clearTimeout(timeout);
    }

    timeout = setTimeout(function () {
      processSearch();
    }, requestTimeout);
  }

  return {
    init: function (element) {
      // Init
      target = element;
      form = KTUtil.find(target, '.quick-search-form');
      input = KTUtil.find(target, '.quick-search-input');
      closeIcon = KTUtil.find(target, '.quick-search-close');
      resultWrapper = KTUtil.find(target, '.quick-search-wrapper');
      resultDropdown = KTUtil.find(target, '.dropdown-menu');
      resultDropdownToggle = KTUtil.find(target, '[data-toggle="dropdown"]');
      inputGroupAppend = KTUtil.find(target, '.input-group-append');

      // Attach input keyup handler
      KTUtil.addEvent(input, 'keyup', handleSearch);
      KTUtil.addEvent(input, 'focus', handleSearch);

      // Prevent enter click
      form.onkeypress = function (e) {
        var key = e.charCode || e.keyCode || 0;
        if (key == 13) {
          e.preventDefault();
        }
      }

      KTUtil.addEvent(closeIcon, 'click', handleCancel);

      // Auto-focus on the form input on dropdown form open
      var toggle = KTUtil.getById('kt_quick_search_toggle');
      if (toggle) {
        $(toggle).on('shown.bs.dropdown', function () {
          input.focus();
        });
      }
    }
  };
};