import { CardBuilder } from '../widgets/layout/card';
import { TwIframe } from '../widgets/tw-iframe/tw-iframe';
import './dashboard-products.css';

export interface DashboardProduct {
    description: string;
    icon: string;
    title: string;
    url: string;
    name?: string;
    hasAccess?: boolean;
}

export const Dashboard = (products?: Array<DashboardProduct>) => {
    const dashboardEl = $('[data-dashboard-company-content="true"]');

    if (!dashboardEl.length || !products) return;

    dashboardEl.append(products.map((product) => {
        const { description, url, icon, title, hasAccess } = product;
        const card =  $(/*html*/`
            <div class="col-xl-6">
                <div class="card card-custom gutter-b card-stretch">
                    <div class="card-body d-flex flex-column p-6">
                        <div class="flex-grow-1 text-center" style="/* align-content: center; */">
                            <i class="${icon}"></i>
                            <p class="font-weight-normal font-size-lg mt-4">${description}</p>
                        </div>
                        <div class="text-center">
                            ${
                                hasAccess
                                    ? `<button class="btn btn-q-action font-weight-bold px-10 py-1">ACESSAR</button>`
                                    : `<a href="${url}" target="_blank" class="btn btn-q-action font-weight-bold px-10 py-1">ACESSAR</a>`
                            }
                        </div>
                    </div>
                </div>
            </div>
        `);

        card
            .find('button')
            .on('click', () => {
                createProductView(product);
            });

        return card;
    }));
};

export const createProductView = ({ title, url }: DashboardProduct) => {
    const container = $('#q_dashboard_menu').parent();
    const iframeContainer = $('<div class="container-fluid">');

    const card = CardBuilder.build({
        title: title,
        bodyContent: TwIframe({
            widget: {
                Name: 'test',
                link: url,
                twIframe: true,
                Visible: true,
                Align: '',
            }
        }),
    });

    card.cardBody.addClass('p-1');

    iframeContainer.append(card.cardElement);

    container.hide();

    container.parent().empty().append(iframeContainer)
};

export const verifyCreateView = () => {
    const product = JSON.parse(sessionStorage.getItem('product')!);

    
    if (product) {
        sessionStorage.removeItem('product');

        createProductView(product);
    }
};

export const storeProductForRedirect = (product: DashboardProduct) => {
    sessionStorage.setItem('product', JSON.stringify(product));
}