import axios from "axios";
import SessionManager from "../../../js/SessionManager";
import qNotifier from "../notifier/q-notifier";

declare var KTApp: any;

interface CurrrentFavorite {
    name: string;
    role: string;
    moduleName?: string;
    url?: string; // old version of favorites
    pathName?: string; 
}

class FavoriteButton {
    private _localStorageKey!: string;    

    public init() {
        this._localStorageKey = `favoritePages${SessionManager.hasModules ? 'QuestorCloud' : SessionManager.moduloAtual}`;
    }

    private _favoriteAction(currentFavorite: CurrrentFavorite) {
        KTApp.blockPage();

        const localStorageItem = this._getCache();
        let isFavorite = false;

        let prevFavorites: Array<CurrrentFavorite> = (localStorageItem && localStorageItem != 'undefined' && typeof localStorageItem !== 'undefined')
            ? JSON.parse(localStorageItem)
            : [];

        const exists = prevFavorites.find(({ role, pathName }) => currentFavorite.role === role && currentFavorite.pathName === pathName);

        if (!exists) {
            prevFavorites.push(currentFavorite);
            isFavorite = true;
            qNotifier.success("Página adicionada aos seus favoritos!", "Sucesso");
        }
        else {
            prevFavorites = prevFavorites.filter(({ role, pathName }) => !(role == currentFavorite.role && pathName == currentFavorite.pathName));
            isFavorite = false;
            qNotifier.success("Página removida dos seus favoritos!", "Sucesso");
        }

        this._setCache(prevFavorites);

        const saveString = btoa(this._getCache()?.replace(/=/g, '') || '');

        axios
            .post('nHttpServicos/UpdateFavoritos', {
                _AFavoritos: saveString,
            })
            .catch((e) => qNotifier.error('Falha ao atualizar os favoritos: ' + e));

        KTApp.unblockPage();

        return isFavorite;
    }

    private _setCache(favorites: Array<CurrrentFavorite> | string) {
        localStorage.setItem(this._localStorageKey, typeof favorites === 'string' ? favorites : JSON.stringify(favorites));
    }

    private _getCache() {
        return localStorage.getItem(this._localStorageKey);
    }

    public setCache(favorites: Array<CurrrentFavorite>) {
        this._setCache(favorites);
    }

    public setFavorite(item: CurrrentFavorite) {
        return this._favoriteAction(item);
    }
}

export default new FavoriteButton();