import FormManager from "../../../../js/FormManager";
import { Formulario } from "../../../../js/nFormulario";
import { FraControl } from "../../fra-control/fra-control";
import { Field, GroupColumn, Prop } from "../../fra-control/types";
import { isGroupColumn } from "../../fra-control/util/util";
import { FormRepeaterForm } from "../form-repeater";

interface FormProp extends Prop<(Field | GroupColumn)[]> {
    formEl?: JQuery<HTMLElement>;
    isFormRepeater?: boolean;
    formRepeaterInstance?: FormRepeaterForm;
}

/**
 * 
 * Creates Form as instance of Formulario 
 * 
 * Creates fields as instances of Field
 */
export const Form = ({ formProp, item, isFormRepeater, formRepeaterInstance }: FormProp) => {
    let wrapper = getFormFieldsWrapper();

    let formEl = $('<form>')
        .addClass(`kt-form form${formProp.Class}${formProp.Alias}`)
        .data('class', formProp.Class);
    
    let form = instantiateForm({
        formProp,
        formEl,
        isFormRepeater,
        formRepeaterInstance,
    });
    
    let { row } = getFormFieldsContainer({
        formProp,
        item,
        formInstance: form,
    });

    $(() => {
        FormManager.createAllFields(form) // await "until" element is present on dom 
    })

    return {
        wrapper: wrapper.append(formEl.append(row)),
        component: form
    }
}


export const getFormFieldsWrapper = () =>  $('<div class="col-10">');

export const getFormFieldsContainer = ({ formProp, item, formInstance }: Omit<FormProp, 'formEl' | 'formRepeaterInstance' | 'isFormRepeater'>) => {
    let row = $(`<div class="${formProp.MultiColuna ? 'row' : ''}">`);

    item.forEach((field, index) => {
        if (!isGroupColumn(field) && field.Lookup) return;

        let fieldEl = FraControl({
            formInstance,
            formProp,
            item: { ...field, index },
        });

        if (fieldEl.length) {
            row.append(fieldEl);
        }
    });

    return { row };
}

export const instantiateForm = (props: Omit<FormProp, 'item' | 'formInstance'>): Formulario => {
    const { formProp, formEl, isFormRepeater, formRepeaterInstance } = props;

    let form = new Formulario({
        alias: formProp.Alias || '',
        caption: formProp.caption,
        chavePrimaria: formProp.primaryKey,
        classe: formProp.Class,
        configextra: formProp.extraConfig,
        detailfields: formProp.detailFields,
        element: formEl,
        filtrofixo: formProp.fixedFilter,
        formdetalhe: formProp.isDetailForm,
        idrequest: formProp.requestId,
        masterclass: formProp.masterClass,
        masterfields: formProp.masterFields,
        multiColumn: formProp.MultiColuna,
        tipoObjeto: formProp.objectType,
        isFormRepeater,
        formRepeaterInstance,
        salvarDadosAutomatico: formProp.salvarDadosAutomatico,
    });

    return form;
}