import { ControlElementCreator, Field, Prop } from "../types";
import { InputBlock } from "../input-block/input-block";
import { Label } from "../label/label";
import { getFormId, setAttrIfHasValue } from "../util/util";
import classNames from 'classnames';
import { FraControlElement } from "../fra-control-element/fra-control-element";
import { OperacaoCadastro } from "../../../../js/nWebUtil";

declare var focusImputFunc: any;

interface ButtonProps {
    props: Prop;
    input: JQuery<HTMLElement>
}

export const NtcActBlob: ControlElementCreator = (props) => {
    let inputBlock = InputBlock(props);
    let label = Label(props);
    let wrapper = getWrapper(props);
    let insertFormEl = FraControlElement(props);

    return inputBlock
        .append(label)
        .append(wrapper)
        .append(insertFormEl);
}

const getWrapper: ControlElementCreator = (props) => {
    const wrapper = $('<div>')
        .addClass(classNames({
            'input-group': props.formProp.MultiColuna,
            'col-12 col-sm-9 col-lg-7': !props.formProp.MultiColuna
        }));
    const inputGroup = $('<div class="input-group input-addon-default">');
    const input = getInput(props);
    const buttons = getButtons({ props, input });

    inputGroup
        .append(input)
        .append(buttons)

    return wrapper.append(inputGroup);
}

const getInput: ControlElementCreator = ({ item, formProp }) => {
    const classList = getClassList(item);
    const input = $('<input>')
        .on('focus', ({ target }) => focusImputFunc({ element: target }))

    setAttrIfHasValue(input, {
        autocomplete: 'off',
        class: classList,
        disabled: item.disabled,
        form: getFormId(formProp),
        id: `ctrl${item.Name}`,
        name: `p.${item.Name}`,
        step: 'any',
        style: item.TextCase,
        title: item.Hint,
        type: item.InputType,
        value: item.Valor,
        'data-autoincrementar': item.AutoIncrementar,
        'data-alias': formProp.Alias,
        'data-classname': item.DMClassName,
        'data-dicionario': item.Dicionario,
    });

    return input;
}

const getClassList = ({ ControlClass, Name }: Field) => {
    return classNames({
        "form-control kt-input cad-control cad-logchange ntcActBlob": true,
        [`ctrl${Name}`]: true,
        [`${ControlClass}`]: ControlClass,
    })
}

const getButtons = (buttonProps: ButtonProps): JQuery<HTMLElement> => {
    const { props } = buttonProps;
    const { formProp } = props;
    const buttonGroup = $(`
        <div class="input-group-append">
            <button 
                class="btn btn-text-dark btn-hover-transparent-dark btn-sm btn-accent dropdown-toggle dropdown-toggle-split kt-btn"
                style="border-color: #ababab;"
                data-toggle="dropdown"
                tabindex="-1"
                type="button"
            >
				<i class="la la-search "></i>
			</button>
            <div class="dropdown-menu dropdown-menu-md dropdown-menu-right" x-placement="bottom-end">
				<ul
                    class="navi navi-hover py-1 identCadastro"
                    id="m-nav-${formProp.Class}"
                    data-class="${formProp.Class}"
                    data-alias="${formProp.Alias || ''}"
                    data-cadcons="${formProp.CadCons}"
                    data-topicoajuda="${formProp.TopicoAjuda}"
                >
                </ul>
            </div>
        </div>
    `);

    buttonGroup
        .find('ul')
        .append(getImportButton(buttonProps.props), getExportButton(buttonProps))

    return buttonGroup;
};

const getImportButton: ControlElementCreator = (props) => {
    const wrapper = getButtonWrapper(props);
    const button = $('<a class="navi-link btn btn-text-dark btn-hover-transparent-dark py-2 ml-2">').on('click', () => {
        $('.NOMEARQUIVOBLOBFILE').trigger('click');
    });

    button
        .append(getIcon('upload'))
        .append(getSpan('Importar'))

    wrapper
        .find('div')
        .append(button);

    return wrapper;
};

const getExportButton = ({ props, input }: ButtonProps) => {
    const wrapper = getButtonWrapper(props);
    const button = $('<a class="navi-link btn btn-text-dark btn-hover-transparent-dark py-2 ml-2">')
        .on('click', () => {
            OperacaoCadastro({
                _AClasse: props.formProp.Class,
                _Aoperacao: 'CadastroGetFile',
                _AAlias: props.formProp.Alias,
                _AAdicParam: '',
                _AFileName: input.val(),
            } as any);
        });

    button
        .append(getIcon('download'))
        .append(getSpan('Exportar'))

    wrapper
        .find('div')
        .append(button);

    return wrapper;
};

const getButtonWrapper: ControlElementCreator = ({ formProp }) => {
    return $(`
        <li class="navi-item">
			<div 
                class="kt-portlet__nav-item display-inline"
                data-class="${formProp.Class}"
                data-alias="${formProp.Alias || ''}"
                data-cadcons="${formProp.CadCons}"
                data-topicoajuda="${formProp.TopicoAjuda}"
                style="display: inline"
            >
            </div>
        </li>
    `);
};

const getSpan = (text: 'Exportar' | 'Importar'): JQuery<HTMLElement> => {
    return $('<span class="d-none d-sm-inline">').append(text);
};

const getIcon = (text: 'upload' | 'download'): JQuery<HTMLElement> => {
    return $(`<i class="la la-cloud-${text}">`)
};