import { ControlElementCreator } from "../types";
import { InputBlock } from "../input-block/input-block";
import { Label } from "../label/label";
import { CarregarCheckTreeView } from "../../../../js/treeview/treeview";
import { getFormId, whenElementOnPage } from "../util/util";
import { FraControlElement } from "../fra-control-element/fra-control-element";

export const NtcCheckTreeView: ControlElementCreator = (props) => {
    let inputBlock = InputBlock(props);
    let label = Label(props);
    let wrapper = getWrapper(props);
    let controlElement = FraControlElement(props);

    return inputBlock
        .append(label)
        .append(wrapper)
        .append(controlElement);
}

const getWrapper: ControlElementCreator = (props) => {
    const { formProp } = props;

    let wrapper = $('<div>').addClass(formProp.MultiColuna ? 'col-md-12' : 'col-sm-9');
    let checkTreeViewContainer = getCheckTreeViewContainer(props);
    let inputHidden = getInputHidden(props);

    return wrapper
        .append(checkTreeViewContainer)
        .append(inputHidden);
}

const getCheckTreeViewContainer: ControlElementCreator = ({ item }) => {
    let container = $('<div>')
        .attr('id', `checktreeview${item.Name}`)
        .data('tree-content', item.treeview!);

    whenElementOnPage(container.get(0)!)
        .then((element) => {
            CarregarCheckTreeView({ element: $(element), _ANome: item.Name, selectionMode: item.LinkLookupMultiSelecao ? 'multiple' : 'single' } as any);
        })

    return container;
}

const getInputHidden: ControlElementCreator = ({ item, formProp }) => {
    let input = $('<input>')
        .attr({
            autocomplete: 'off',
            form: getFormId(formProp),
            id: `ctrl${item.Name}`,
            name: `p.${item.Name}`,
            required: item.Required,
            type: 'hidden',
            value: '',
            'data-alias': formProp.Alias,
            'data-classname': item.DMClassName,
        })

    return input;
} 