import classNames from 'classnames';
import { InputBlock } from '../input-block/input-block';
import { ControlElementCreator } from '../types';
import { getFormId, setAttrIfHasValue } from '../util/util';
import { FraControlElement } from '../fra-control-element/fra-control-element';

export const NtcCheckbox: ControlElementCreator = (props) => {
    const inputBlock = InputBlock(props);
    const label = getLabel(props);
    const wrapper = getWrapper(props);
    const input = getInput(props);
    const controlElement = FraControlElement(props);

    wrapper.append(input);

    return inputBlock
        .append(label)
        .append(wrapper)
        .append(controlElement);
};

const getLabel: ControlElementCreator = ({ item, formProp }) => {
    const label = $('<label>')
        .addClass(classNames({
            'col-form-label': true,
            'col-12 col-sm-3': !formProp.MultiColuna,
        }))
        .attr({
            for: `ctrl${item.Name}`,
            id: `labelctrlopt${item.Name}`,
        });

    return label;
};

const getWrapper: ControlElementCreator = ({ formProp: { MultiColuna } }) => {
    const div = $('<div>')
        .addClass(classNames({
            'col-12': true,
            'col-sm-9 col-lg-7': !MultiColuna,
        }));

    return div;
};

const getInput: ControlElementCreator = (props) => {
    const checkboxWrapper = $('<div class="checkbox-wrapper">');
    const inputWrapper = $('<label class="checkbox checkbox-outline checkbox-dark mb-2" style="margin-bottom: 2px">');
    const input = getInputVisual(props);
    const inputHidden = getInputHidden(props);
    const spanText = $('<span class="mr-2"></span>');

    inputWrapper
        .append(input)
        .append(inputHidden)
        .append(spanText)
        .append(props.item.DisplayLabel);

    return checkboxWrapper.append(inputWrapper);
};

const getInputVisual: ControlElementCreator = ({ item, formProp }) => {
    const input = $('<input>')
        .on('change', ({ target }) => {
            $(target)
                .siblings()
                .first()
                .val((<HTMLInputElement>target).checked ? 1 : 0)
                .trigger('change');
        });

    setAttrIfHasValue(input, {
        class: 'cad-control cad-logchange',
        title: item.Hint,
        type: 'checkbox',
        'data-alias': formProp.Alias,
        'data-autoincrementar': item.AutoIncrementar,
        'data-classname': item.DMClassName,
    });

    return input;
};

const getInputHidden: ControlElementCreator = ({ item, formProp }) => {
    const input = $('<input>')
        .on('change', ({ target }) => {
            $(target)
                .siblings()
                .first()
                .prop('checked', (<HTMLInputElement>target).value === '1');
        });

    setAttrIfHasValue(input, {
        form: getFormId(formProp),
        id: `ctrl${item.Name}`,
        name: `p.${item.Name}`,
        type: 'hidden',
        value: '0',
    });

    return input;
};
