import moment from "moment";
import { Field as FormField } from '../../../../js/nFormulario';
import { getFormatOptions } from "../../../../js/setDatepicker";
import { FindLookupValue, itsToUseLookupInside } from "../../../../js/nLookUp";

const hasFocus = (element: JQuery<HTMLElement> | null) => element?.is(':focus');

const getText = (value: string | number = '', lookup: string | number) => {
    if (value == '') return '';
    if (lookup == '') return value;

    return `${value} - ${lookup}`;
};

interface ISetValueProps {
    field: FormField;
    value: string | number;
    lookup: string | number;
    isFromLoadData: boolean;
}

const setValue = ({ field, value, lookup = '' }: ISetValueProps) => {
    if (itsToUseLookupInside(field.multiColumn)) {
        setValueInsideInput(field, value, lookup);
    }
    else {
        if (!hasFocus(field.componente)) field.componente.val(value);

        field.componente.data({ value });

        NtcLinkEditUtil.setDescription(field, value === '' ? 'Nenhum item Selecionado' : lookup);
    }

    if (lookup == '') FindLookupValue(field.nome, field.InstanciaFormulario.classe, field.InstanciaFormulario.alias);
};

const setDescription = (field: FormField, lookup: number | string) => {
    const value = field.getvalue();

    if (itsToUseLookupInside(field.multiColumn)) {
        setValueInsideInput(field, value, lookup);
    }

    field.componente.data({ lookup });
    field.resultcomponente.val(lookup);
    field?.resultcomponentetext?.text(lookup);
};

const getAdditionalProps = (field: FormField, dataToLoad: { [key: string]: any }) => {
    const lookupFieldName = `${field.nome}RESULT`;
    const text = dataToLoad[lookupFieldName] || dataToLoad[lookupFieldName.toUpperCase()] || 'Registro não encontrado'

    return {
        lookupText: text,
    };
};

const setValueInsideInput = (field: FormField, value: number | string, lookup: number | string) => {
    const datePickerEl = field.getDatepickerElement();

    if (datePickerEl) {
        if (!hasFocus(datePickerEl)) {
            const { mask } = datePickerEl.data('mask');
            const { momentFormat } = getFormatOptions(mask);
            const date = (value == '') ? '' : moment(value, 'YYYY-MM-DD').format(momentFormat);
            const description = getText(date, lookup);

            datePickerEl.val(description);
        }
    }
    else if (!hasFocus(field.componente)) {
        const text = getText(value, lookup);

        field.componente.val(text);
    }
};

export class NtcLinkEditUtil {
    static setValue = setValue;
    static setDescription = setDescription
    static getAdditionalProps = getAdditionalProps
};