import { FindLookupValue } from "../../../../js/nLookUp";
import { InputBlock } from "../input-block/input-block";
import { Label as getLabel } from "../label/label";
import { ControlElementCreator, Field, FormProp } from "../types";
import { getFormId, setAttrIfHasValue } from "../util/util";
import classNames from 'classnames';
import { FraControlElement } from "../fra-control-element/fra-control-element";
import { AbrirCadastro } from "../../../../js/nWebUtil";

declare var focusImputFunc: any;

export const NtcLink: ControlElementCreator = (props) => {
    let inputBlock = InputBlock(props);
    let label = getLabel(props);
    let wrapper = getWrapper(props);
    let controlElement = FraControlElement(props);

    return inputBlock
        .append(label)
        .append(wrapper)
        .append(controlElement);
}

const getWrapper: ControlElementCreator = (props) => {
    const { item, formProp } = props;
    
    let wrapper = $('<div>').addClass(classNames({
        'input-group': formProp.MultiColuna,
        'col-12 col-sm-9 col-lg-7': !formProp.MultiColuna
    }));

    let input = getInput(props);
    let buttonGroup = getButtonWrapper(props);

    if (formProp.MultiColuna) {
        wrapper.append(input);
        wrapper.append(buttonGroup);

        if (item.InputType == 'date') {
            setDateInput(wrapper);
        }
    }
    else {
        let divGroup = $('<div class="input-group">');

        wrapper.append(divGroup);
        divGroup.append(input);
        divGroup.append(buttonGroup);

        if (item.InputType == 'date') {
            setDateInput(divGroup)
        }
    }

    return wrapper;
}

const setDateInput = (wrapper: JQuery<HTMLElement>) => {
    let prependDate = getPrependDate();
    let inputHidden = getInputHidden();

    wrapper.prepend(prependDate);
    wrapper.append(inputHidden);
}

const getPrependDate = () => {
    let div = $(`<div class="input-group-prepend">`);
    let span = $('<span class="input-group-text">');
    let icon = $('<i class="far fa-calendar-alt">');

    span.append(icon);

    return div.append(span);
}

const getInputHidden = () => {
    return $();
}


const getInput: ControlElementCreator = ({ item, formProp }) => {
    let attr = getInputAtrr(item, formProp);

    let input = $('<input>')
        .on('focus', (event) => focusImputFunc({ element: event.target, multiColumn: formProp.MultiColuna }))

    if (!item.ntcLinkAnnotation) {
        input.on('blur', () => FindLookupValue(item.Name, item.DMClassName, formProp.Alias))
    }

    setAttrIfHasValue(input, attr);

    return input;
}

const getInputAtrr = (item: Field, formProp: FormProp): Object => {
    let classList = getInputClassList(item);
    let attr = {
        autocomplete: 'off',
        class: classList,
        disabled: item.disabled,
        form: getFormId(formProp),
        id: `ctrl${item.Name}`,
        name: `p.${item.Name}`,
        readonly: item.ReadOnly,
        required: item.Required,
        style: item.TextCase,
        title: item.Hint,
        'data-alias': formProp.Alias,
        'data-autoincrementar': item.AutoIncrementar,
        'data-button': `btn${item.Name}`,
        'data-classname': item.DMClassName,
        'data-dicionario': item.Dicionario,
    }

    if (item.InputType == 'date') {
        attr = {
            ...attr,
            ...{
                type: 'text',
                value: '',
                'data-mask': '00/00/0000',
                'data-original-value': item.Valor
            }
        };
    }
    else {
        attr = {
            ...attr,
            ...{
                type: item.InputType,
                value: item.Valor,
                'data-mask': item.mask,
                'data-mask-reverse': item.maskreverse
            }
        };
    }

    if (item.LinkLookupMultiSelecao) {
        attr = {
            ...attr,
            ...{
                'data-role': 'tags'
            }
        };
    }

    return attr;
}

const getInputClassList = (field: Field): string => {
    return classNames({
        'form-control kt-input cad-control cad-logchange': true,
        'form-control-search border-radius-left': field.SearchStyle,
        [`ctrl${field.Name}`]: true,
        [`${field.ControlClass}`]: field.ControlClass,
    });
}

export const getButtonWrapper: ControlElementCreator = (props) => {
    const { item, formProp } = props;

    let buttonWrapper = $('<div class="input-group-append">');
    let buttonSearch = getButtonSearch(props);

    buttonWrapper.append(buttonSearch);

    if (!item.SearchStyle && item.LinkDirecionaCadastro) {
        let buttonAdd = getButtonAdd(props);

        buttonWrapper.append(buttonAdd);
    }

    return buttonWrapper;
}

const getButtonSearch: ControlElementCreator = ({ item }) => {
    let icon = $('<i>')
        .addClass(classNames({
            'la la-search': true,
            'border-radius-right icon-blue': item.SearchStyle
        }));

    let button = $('<button>')
        .append(icon)
        .addClass(classNames({
            'kinput-group-text btn btn-text-dark btn-hover-transparent-dark': true,
            'input-append-blue border-radius-right': item.SearchStyle
        }));

    setAttrIfHasValue(button, {
        disabled: item.disabled,
        id: `btn${item.Name}`,
        style: 'border-color: #ababab;',
        type: 'button',
        'data-control': `ctrl${item.Name}`,
        'data-dmclassname': item.DMClassName,
        'data-id': `ctrl${item.Name}`,
        'data-keyfieldcontrols': item.KeyFieldControls,
        'data-LinkLookupDataSetNome': item.LinkLookupDataSetNome,
        'data-LinkLookupKeyFields': item.LinkLookupKeyFields,
        'data-LinkKeyFields': item.LinkKeyFields,
        'data-LinkLookupMultiSelecao': item.LinkLookupMultiSelecao,
        'data-LinkLookupResultField': item.LinkLookupResultField,
        'data-NomesValores': item.NomesValores,
        'data-target': '#myModal',
        'data-toggle': 'modal'
    });

    return button;
}

const getButtonAdd: ControlElementCreator = ({ item }) => {
    return $('<button>')
        .append('<i class="la la-plus">')
        .attr({
            class: "input-group-text btn btn-text-dark btn-hover-transparent-dark",
            style: 'border-color: #ababab;',
            tabindex: '-1',
            title: '',
            type: 'button',
            'data-id': `ctrlnew${item.Name}`,
            'data-original-title': 'Incluir um Novo',
            'data-toggle': 'kt-tooltip',
        })
        .on('click', () => AbrirCadastro(item.Cadastro, true, ''))
}