/**
 * Mapped properties from FraControl.partial
 */

import { Formulario } from '../../../js/nFormulario';

export interface Field extends ControlType {
    AlignTH: string;
    Alignment: string;
    AutoIncrementar?: boolean;
    BoldStyle: string;
    ChavePrimaria?: boolean;
    ControlClass?: string;
    DMClassName: string;
    DataType: string;
    Dicionario: string;
    DisplayLabel: string;
    DisplayWidth: number;
    DisplayWidthColHtml: number;
    Expressoes: any;
    Expansivel?: boolean;
    Hint?: string;
    InputType: string;

    Lookup: boolean;
    MaxLength?: number;
    Name: string;
    NumeroColunas: number;
    NumeroColunasLabel: number;
    ReadOnly: boolean;
    Required: boolean;
    TabStop: boolean;
    TextCase: string;
    TipoControle: NtcType;
    Visible: boolean;
    disabled: string;
    mask?: string;
    maskreverse?: boolean;
    oculto: boolean;
    LabelClass?: string;
    SearchStyle?: boolean; // widget busca rápida
    Valor?: any;
    ValorLookup?: any;
    ntcGridFields?: Field[];
    NomesValoresList?: NameValue[];
    checked?: boolean;
    Filter?: string; // arquivo
    TopicoAjuda?: string;
    treeview?: TreeViewItem[];

    LinkKeyFields?: string;
    LinkLookupKeyFields?: string;
    LinkLookupResultField?: string;
    LinkLookupDescFindField?: string;
    LinkLookupDescShowFields?: string;
    LinkLookupDataSetNome?: string;
    LinkLookupMultiSelecao?: boolean;
    NomesValores?: string;
    KeyFieldControls?: string;
    LinkDirecionaCadastro?: boolean;
    Cadastro?: string;

    DICDIRETORIOABRIR?: boolean;
    DICARQUIVOHASH?: boolean;
    NOMEARQUIVOBLOBFILE?: boolean;
    DICARQUIVOABRIR?: boolean;

    ControleWeb?: boolean;
    index?: number; // necessário para criar os campos, equivalente ao {{-index}} no mustache
    Accordion?: string;
    Separador?: string;
}

export interface NameValue {
    nvNome: string;
    nvValor: string;
    nvSelected?: string;
}

export interface FormProp {
    requestId?: number;
    MultiColuna: boolean;
    Alias?: string;
    hasSessionSelecao: boolean;
    Class: string;
    CadCons?: string;
    caption: string;
    primaryKey: Array<string> | null;
    extraConfig: Object | null;
    detailFields?: Array<string> | null;
    fixedFilter?: Object | null;
    isDetailForm?: boolean;
    masterClass?: string | null;
    masterFields?: Array<string> | null;
    objectType: string;
    toolButtons: ToolButton[];
    TopicoAjuda?: string;
    salvarDadosAutomatico?: boolean;
}

export interface ToolButton {
    Caption: string;
    Classe: string;
    CleanCaption: string;
    Custom: boolean;
    disabled?: string; // str "disabled" for use directly in .partial
    ExibirToolBar: boolean;
    Hint: string;
    Name: string;
    NameEvent: string;
    Opcoes: { [key: string]: any };
    Confirmacao?: ActionConfirmation;
    TipoAcao: string;
    icon?: string;
    tbsButton?: boolean;
    tbsDropDown?: boolean;
    DropdownMenu?: ToolButtonDropdownItem[];
    CloudTipoAcao?: 'apNone' | 'apPrincipal' | 'apSecundaria';
}

export interface ActionConfirmation {
    Mensagem: string;
    RespostaDefault: number;
    Tipo: number;
}

export interface ToolButtonDropdownItem {
    Text: string;
    NameItem: string;
    Tipo: string;

    /** Only if not have OnClick */
    Link?: boolean;

    /** Only if not have Link */
    OnClick?: string;
    /** Only if have OnClick, may or not have value */
    SubItens?: [];

    NameItemEvent?: string;

    Separador?: boolean;
}

export interface ControlType {
    ntcActBlob?: boolean;
    ntcActBuscarCEP?: boolean;
    ntcCheckBox?: boolean;
    ntcCheckListBoxHorizontal?: boolean;
    ntcCheckListBox?: boolean;
    ntcCheckTreeView?: boolean;
    ntcComboBox?: boolean;
    ntcDBRichEditEditor?: boolean;
    ntcEdit?: boolean;
    ntcGrid?: boolean;
    ntcHistorio?: boolean;
    ntcImage?: boolean;
    ntcLink?: boolean;
    ntcLinkAnnotation?: boolean;
    ntcLinkEdit?: boolean;
    ntcMemo?: boolean;
    ntcMemoAnnotation?: boolean;
    ntcRichEdit?: boolean;
    ntwEditComLabel?: boolean;
    ntwDiaComData?: boolean;
}

export type NtcType =
    | 'ntcActBlob'
    | 'ntcActBuscarCEP'
    | 'ntcActData'
    | 'ntcActDataHora'
    | 'ntcCheckBox'
    | 'ntcCheckListBoxHorizontal'
    | 'ntcCheckListBox'
    | 'ntcCheckTreeView'
    | 'ntcComboBox'
    | 'ntcDBRichEditEditor'
    | 'ntcEdit'
    | 'ntcGrid'
    | 'ntcHistorio'
    | 'ntcImage'
    | 'ntcLink'
    | 'ntcLinkEdit'
    | 'ntcLinkAnnotation'
    | 'ntcMemo'
    | 'ntcMemoAnnotation'
    | 'ntcRichEdit'
    | 'ntwEditComLabel'
    | 'ntwDiaComData';

export interface GroupColumn {
    GrupoColunas: boolean;
    GrupoNome: string;
    InputType: string;
    MultiColuna: boolean;
    NumeroColunas: number;
    colunas: GroupColumnCol[];
    colunasdados: number;
    linhas: GroupColumnRow[];
    linhascomtitulo: boolean;
}

interface GroupColumnCol {
    caption: string;
    numerocolunas: number;
}

interface GroupColumnRow {
    caption: string;
    GroupFields: Field[];
}

interface TreeViewItem {
    nodes?: TreeViewItem[];
    nome: string;
    valor: string;
}

export function isTreeViewItem(item: TreeViewItem[]): item is TreeViewItem[] {
    if (!item.length) return false;

    for (let i = 0; i < item.length; i++) {
        const it = item[i];

        if (!hasOwnProperty(it, 'valor') && !hasOwnProperty(it, 'nome')) return false;

        if (!hasOwnProperty(it, 'nodes')) return true;

        if (!isTreeViewItem(it.nodes!)) return false;
    }

    return true;
}

function hasOwnProperty(obj: any, property: string) {
    return Object.prototype.hasOwnProperty.call(obj, property);
}

export type ControlElementCreator<T = Field, U = JQuery<HTMLElement>> = (props: Prop<T>) => U;

export interface Prop<T = Field> {
    item: T;
    formProp: FormProp;
    formInstance?: Formulario;
}

export interface DefaultProperties extends DefaultElements {
    nome: string;
    visible: boolean;
    index: number;
    isMultiColumn: boolean;
    isGroupColumn: boolean;
    expressoes: Object;
    controlType: NtcType;
}

interface DefaultElements {
    label: JQuery<HTMLElement>;
    componente: JQuery<HTMLElement>;
    linkbutton: JQuery<HTMLElement>;
    resultcomponente: JQuery<HTMLElement>;
    resultcomponentetext: JQuery<HTMLElement>;
    componenteimage: JQuery<HTMLElement> | null;
}
