import { Field } from '../../../../js/nFormulario';
import { GroupColumn } from '../types';

interface Attribute {
    [key: string]: any;
}

const isEmpty = (value: any) => ['', null, undefined].includes(value);

export function setAttrIfHasValue(el: JQuery<HTMLElement>, attr: Attribute) {
    const attribute = Object
        .entries(attr)
        .reduce((acc, [key, value]) => {
            if (!isEmpty(value)) {
                acc[key] = value;
            }

            return acc;
        }, {} as Attribute);

    el.attr(attribute);
}

export function setDataIfHasValue(el: JQuery<HTMLElement>, data: Attribute) {
    const dataAttribute = Object
        .entries(data)
        .reduce((acc, [key, value]) => {
            if (!isEmpty(value)) {
                acc[key] = value;
            }

            return acc;
        }, {} as Attribute);

    el.data(dataAttribute);
}

export function isGroupColumn(item: any): item is GroupColumn {
    return item.GrupoColunas ?? false;
}

export async function whenElementOnPage(element: HTMLElement): Promise<HTMLElement> {
    return new Promise((resolve) => {
        new MutationObserver((_, observer) => {
            if (document.contains(element)) {
                resolve(element);

                observer.disconnect();
            }
        }).observe(document.documentElement, {
            childList: true,
        });
    });
}

export function getFormId({ Class = '', Alias = '' }: { Class: string, Alias?: string }): string {
    return `form_${Class}_${Alias}`;
}

type ExpressionType = 'ExpressaoValue';

interface ExpressionDependencies {
    [key: string]: {
        [key in ExpressionType]: Array<string>;
    };
}

export function hasDependency(runningField: Field, fieldNameCaller: string, type: ExpressionType) {
    const form = runningField.InstanciaFormulario;
    const dependencies = form.expressionDependecies as ExpressionDependencies;
    const searchead = new Map<string, boolean>();

    const search = (fieldName: string): boolean => {
        const fieldNames = dependencies[fieldName]?.[type];

        if (!fieldNames?.length) {
            searchead.set(fieldName, false);

            return false;
        }

        return Boolean(fieldNames.find((fieldName) => {
            if (searchead.has(fieldName)) return fieldName == fieldNameCaller || searchead.get(fieldName);

            const hasFieldName = search(fieldName);

            searchead.set(fieldName, hasFieldName);

            return fieldName == fieldNameCaller || hasFieldName;
        }));
    };

    try {
        return search(runningField.nome);
    }
    catch (e) {
        console.error(`${e}. With expression from field: ${runningField.nome}`);

        return false;
    }
}

export function generateExpressionDependency (field: Field) {
    const { ExpressaoValue } = field.expressoes || {};

    return {
        ExpressaoValue: generateFieldExpressionDependency(field, ExpressaoValue),
    };
}

function generateFieldExpressionDependency(field: Field, expression: string) {
    try {
        const form = field.InstanciaFormulario;
        const fields = form.fields.map(({ nome }) => nome);
        const words = expression?.match(/\w+/g) || [];
        const dependencies = new Set<string>();
        
        words.forEach((word) => {
            if (field.nome !== word && fields.includes(word)) {
                dependencies.add(word);
            }
        });
        
        return Array.from(dependencies.keys());
    }
    catch (e) {
        console.error(e);

        return [];
    }
}