import toastr from "toastr";
import './q-notifier.css';

class qNotifier {

    private _defaultOptions: ToastrOptions = {
        "closeButton": false,
        "debug": false,
        "newestOnTop": false,
        "progressBar": false,
        "positionClass": "toast-top-right",
        "preventDuplicates": false,
        "onclick": undefined,
        "showDuration": 300,
        "hideDuration": 2000,
        "timeOut": 5000,
        "extendedTimeOut": 2000,
        "showEasing": "swing",
        "hideEasing": "linear",
        "showMethod": "fadeIn",
        "hideMethod": "fadeOut"
    };

    private _fixMessage(message: string) {
        return message.replace(/\\n/g, "<br />");
    }

    private _getOptions = (options: ToastrOptions = {}, message: string) => {
        return Object.assign({}, this._defaultOptions, options, { timeOut: Math.min(Math.max(5000, message.length * 50), 15000) });
    }

    public success(message: string, title: string, options?: ToastrOptions) {
        toastr.success(this._fixMessage(message), title, this._getOptions(options, message));
    }

    public info(message: string, title: string, options?: ToastrOptions) {
        toastr.info(this._fixMessage(message), title, this._getOptions(options, message));
    }

    public warning(message: string, title: string, options?: ToastrOptions) {
        toastr.warning(this._fixMessage(message), title, this._getOptions(options, message));
    }

    public error(message: string, title?: string, options?: ToastrOptions) {
        toastr.error(this._fixMessage(message), title, this._getOptions(options, message));
    }

    public exception(data: any) {
        console.log(data);
    }

    public clear() {
        toastr.clear();
    }

    public infoCaq ({link, message, title}: { message: string, link?: string, title?: string }) {
        const text = /*html*/`<p>${message}</p>${link ? `<a href="${link}" target="_blank">Clique aqui para saber mais!</a>`: ''}`;

        toastr.info(
            text,
            title,
            {
                iconClass: 'toast-info-caq',
            }
        )
    }
}

export default new qNotifier();