
export interface ButtonProps {
    variation: ButtonVariation;
    cssClass?: string;
    type?: 'submit' | 'button';
    iconPosition?: 'before' | 'after';
    attr?: string;
    icon?: string;
    iconImg?: string; //data:image/png;base64,aasd...
    text?: string;
    textAdaptability?: boolean;
}

export type ButtonVariation = 'action' | 'primary' | 'secondary' | 'tertiary';

export const Button = ({ variation, cssClass, type, icon, text, textAdaptability, attr, iconPosition = 'before', iconImg }: ButtonProps): string => {
    const iconStr = iconImg 
        ? `<img src="${iconImg}" class="icon-md ${iconPosition == 'after' ? 'pl-2' : 'pr-2'}">`
        : `${icon ? /*html*/`<span><i class="icon-md la ${icon}"></i></span>` : ''}`

    return /*html*/`
        <button
            class="btn btn-sm btn-q-${variation || 'tertiary'} ${cssClass || ''}"
            type="${type || 'button'}"
            ${attr || ''}
        >
            ${iconPosition == 'before' ? iconStr : ''}
            ${text ? /*html*/`<span class="h4 ${textAdaptability ? 'd-none d-sm-inline': ''}">${text}</span>` : ''}
            ${iconPosition == 'after' ? iconStr : ''}
        </button>
    `;
};