import { ToolButton } from "../../fra-control/types";
import { DefinicaoObjetosConfigForToolButtons, ScopeConfigForToolButtons } from "../toolbuttons";
import { getToolButtonAction } from "../toolbutton-action/toolbutton-action";
import { Button } from "../button";

declare var actCarregarXMLfield: any;

export interface CustomButtonProps {
    DefinicaoObjetos: Pick<DefinicaoObjetosConfigForToolButtons, 'Alias' | 'CadCons' | 'Class' | 'TopicoAjuda' | 'TipoObjeto'>;
    toolButton: ToolButton;
    Params?: {
        [key: string]: string;
    };
};

export const renderCustomButton = ({ DefinicaoObjetos, Params }: ScopeConfigForToolButtons/*ScopeConfig*/) => {
    const { ToolButtons } = DefinicaoObjetos;

    return ToolButtons
        .map((toolButton, index) => {
            if (!toolButton.tbsButton || !toolButton.ExibirToolBar || !toolButton.Custom) return;

            if (toolButton.NameEvent === 'actCarregar_XML') {
                return loadXmlButton(index)
            }

            if (!['nActMoverRegAcimaExecute', 'nActMoverRegAbaixoExecute'].includes(toolButton.NameEvent)) {
                return CustomButton({ DefinicaoObjetos, toolButton, Params });
            }

        })
        .filter(button => button);
}


const loadXmlButton = (index: number) => {
    const input = $('<input>')
        .attr({
            accept: ".xml",
            id: `actCarregarXML${index}`,
            type: "file",
            style: "display:none",
        })
        .css({ display: 'none' })
        .on('change', function () { actCarregarXMLfield(this) });

    (window as any).actCarregar_XML = () => {
        $(`#actCarregarXML${index}`).click();
    };

    return input;
};

export const CustomButton = ({ DefinicaoObjetos, toolButton, Params }: CustomButtonProps) => {
    const { Alias = '', Class = '', CadCons = '', TopicoAjuda = '' } = DefinicaoObjetos;
    const { icon, Caption, Name } = toolButton;

    const button = $(Button({
        variation: 'tertiary',
        attr: `data-alias="${Alias}" data-cadcons="${CadCons}" data-topicoajuda="${TopicoAjuda}" data-action-name="${Name}"`,
        cssClass: `${Class} ml-2`,
        text: Caption.toUpperCase(),
        iconImg: icon,
    }))
    .on('click', getToolButtonAction({ DefinicaoObjetos, toolButton, Params }));

    return button;
}