import FormManager from "../../../../js/FormManager";
import { DoExecuteReport } from "../../../../js/nWebUtil";
import { ConteudoConfig, DefinicaoObjetosConfig, ScopeConfig } from "../../../types/page-config/page-config";
import { ToolButton } from "../../fra-control/types";
import { Button } from "../button";
import { getStringDataAction } from "../util/action-data";
import { dropdownReportButton } from "./dropdown-report-button";

export interface DefinicaoObjetosForDropDownButton extends Pick<DefinicaoObjetosConfig, 'Class' | 'Alias' | 'CadCons' | 'TopicoAjuda' | 'ToolButtons'> { }

interface DropdownButtonProps extends Pick<ScopeConfig, 'Conteudo'> {
    DefinicaoObjetos: DefinicaoObjetosForDropDownButton;
}

export const DropdownButton = ({ DefinicaoObjetos, Conteudo }: DropdownButtonProps) => {
        const { Class, Alias = '', CadCons, TopicoAjuda, ToolButtons } = DefinicaoObjetos;
    const divDropdown = $(/*html*/
        `<div class="dropdown dropdown-inline">
            ${Button({
                cssClass: 'dropdown-toggle ml-2',
                attr: 'data-toggle="dropdown" aria-haspopup="true" aria-expanded="false"',
                icon: 'la-ellipsis-v',
                text: 'AÇÕES',
                variation: "tertiary",
            })}
        </div>`
    );

    const dropdownMenu = $('<div class="dropdown-menu dropdown-menu-md dropdown-menu-right" x-placement="bottom-end">');

    const ul = $(`<ul class="navi navi-hover py-1 identCadastro" id="m-nav-${Class}" data-class="${Class}" data-alias="${Alias}" data-cadcons="${CadCons}" data-topicoajuda="${TopicoAjuda}">`)

    const listItems = ToolButtons
        .filter(({ Name, tbsButton, tbsDropDown, DropdownMenu, Custom }) => (!(Name === 'actDataSetPost') && (tbsButton || (tbsDropDown && DropdownMenu?.length)) && !Custom))
        .map((ToolButton) => {
            if (ToolButton.tbsDropDown) return getDropdownButon({ ToolButton, DefinicaoObjetos });

            return getTbsButton({ ToolButton, DefinicaoObjetos })
        });
    
    const reportItems = getDropdownReportButton({ Conteudo});

    ul.append([...listItems, reportItems]);

    dropdownMenu.append(ul);

    return divDropdown.append(dropdownMenu);
}

type ActionListCreator = (props: {
    ToolButton: ToolButton;
    DefinicaoObjetos: DefinicaoObjetosForDropDownButton;
}) => JQuery<HTMLElement>;

const getTbsButton: ActionListCreator = ({ DefinicaoObjetos, ToolButton }) => {
    const { Class, Alias = '', CadCons, TopicoAjuda, } = DefinicaoObjetos;
    const { disabled, Hint, NameEvent, Name, Caption } = ToolButton;

    return $(
        `<li class="navi-item ${disabled ? 'navi-item-disabled' : ''}">
            <a 
                class="navi-link ${disabled ? 'disabled' : ''} toolbutton" 
                title="${Hint}" 
                style="cursor: pointer;"
                id="tbtn${NameEvent}${Class}${Alias}"
                ${disabled ? 'data-disabled="true"' : ''}  
                data-n-onclick="${NameEvent}"
                ${getStringDataAction(Class, Alias, CadCons, TopicoAjuda)}
            >
                <i class="navi-text navi-icon la la-${Name}"></i>
                <span class="navi-text">${Caption}</span>
            </a>
        </li>`
    )
}

const getDropdownButon: ActionListCreator = ({ DefinicaoObjetos, ToolButton }) => {
    const { Class, Alias = '' } = DefinicaoObjetos;
    const { DropdownMenu, Caption } = ToolButton;
    const dropdownMenu = $('<ul class="dropdown-menu w-100" >');
    const item = $(`
        <li class="navi-item dropdown-submenu pull-left" id="subment">
            <a class="navi-link" style="cursor: pointer;" id="kt-nav-TnCtbDMLoteCtb" title="Lista de relatórios">
                <i class="navi-text navi-icon la la-angle-down"></i>
                <span class="navi-text">${Caption}</span>
            </a>
        </li>`).on('mouseover', () => {
            const { height, top } = dropdownMenu.get(0)!.getBoundingClientRect();
            const windowHeight = window.visualViewport!.height;
            const treshold = height + top - windowHeight + 10;

            if (treshold > 0) dropdownMenu.css('top', `-${treshold}px`);
        });

    const submenuItems = DropdownMenu!.map(({ Text, Tipo, Link, OnClick, NameItem, NameItemEvent = '' }) => {
        const setLink = (el: JQuery<HTMLElement>) => {
            if (Link) {
                return (Tipo == 'Relatorio')
                    ? el.on('click', () => {
                        const customEvent = FormManager.getReportCustomEvent({ Class, Alias, reportName: NameItem });

                        if (customEvent) {
                            customEvent({ Class, Alias, Tipo, NameItem, Text });
                        }
                        else {
                            DoExecuteReport(Class, Alias, NameItem)
                        }
                    })
                    : el.attr('href', `${Tipo}?Scope={Class:"${NameItem}"}`);
            }
            else if (OnClick) {
                return el.on('click', () => (window as any)[NameItemEvent]());
            }
            return el;
        };
        const li = $('<li class="navi-item">');
        const a = $(
            `<a class="navi-link" style="cursor: pointer;">
                <span class="navi-text">${Text}</span>
            </a>`
        );

        return li.append(setLink(a));
    });
    
    dropdownMenu.append(submenuItems);
    
    return item.append(dropdownMenu);
}

const getDropdownReportButton = ({ Conteudo }: { Conteudo?: ConteudoConfig }) => {
    if (Conteudo?.ArquivosRelatorio) return dropdownReportButton({ Conteudo });

    return $();
}