import FormManager from "../../../../js/FormManager";
import { Formulario } from "../../../../js/nFormulario";
import { DefinicaoObjetosConfig } from "../../../types/page-config/page-config";
import { Button } from "../button";
import { getStringDataAction } from "../util/action-data";

interface SaveButtonProps {
    DefinicaoObjetos: Pick<DefinicaoObjetosConfig, 'Class' | 'CadCons' | 'TopicoAjuda' | 'Alias' | 'ToolButtons'>;
}

export const SaveButton = ({ DefinicaoObjetos: { Class, CadCons, TopicoAjuda, Alias = '', ToolButtons } }: SaveButtonProps) => {
    const { disabled, Caption } = ToolButtons.find(({ Name }) => Name === 'actDataSetPost')!;

    const dropdownButtons = [
        {
            text: 'Salvar e Abrir Novo',
            icon: 'la-plus',
            postType: 0,
            action: 'actDataSetPostExecute'
        },
        {
            text: 'Salvar e Editar',
            icon: 'la-pen',
            postType: 1,
            action: 'actDataSetPostExecute'
        },
        {
            text: 'Salvar e Duplicar',
            icon: 'la-copy',
            postType: 2,
            action: 'actDataSetPostExecute'
        },
        {
            text: 'Salvar e Voltar',
            icon: 'la-undo',
            postType: 3,
            action: 'actDataSetPostExecute'
        }
    ];

    FormManager.shortCuts.bind({ 
        Class, 
        Alias, 
        caption: Caption, 
        handler: (event: Event, form: Formulario) => {
            form.executeAction('actDataSetPostExecute');
        }
    });

    return $(/*html*/
        `<li class="kt-portlet__nav-item" data-class="${Class}" data-alias="${Alias}" data-cadcons="${CadCons}" data-topicoajuda="${TopicoAjuda}" style="display: inline">
            <div class="btn-group" data-class="${Class}" data-alias="${Alias}" data-cadcons="${CadCons}" data-topicoajuda="${TopicoAjuda}">
                ${Button({
                    variation: 'tertiary',
                    cssClass: 'ml-2',
                    attr: `id="btnActDataSetPostExecute${Class}${Alias}" data-posttype="1" ${disabled ? 'disabled data-disabled="true"' : ''} data-n-onclick="actDataSetPostExecute" 
                    ${getStringDataAction(Class, Alias, CadCons, TopicoAjuda)}`,
                    icon: 'la-save',
                    text: 'SALVAR'
                })}   
                ${Button({
                    variation: 'tertiary',
                    cssClass: 'dropdown-toggle-split rounded-right',
                    attr: `id="btnActDataSetPostExecuteOpts${Class}${Alias}" ${disabled ? 'disabled' : ''} data-toggle="dropdown"`,
                    icon: 'la-angle-down'
                })}            
                </button>
                <div class="dropdown-menu dropdown-menu-right">
                    ${dropdownButtons
                        .map((prop) => dropdownButtonItem({...prop, Class, Alias, CadCons, TopicoAjuda}))
                        .join('')
                    }
                    <div class="dropdown-divider"> </div>
                    ${dropdownButtonItem({
                        Class,
                        Alias,
                        CadCons,
                        TopicoAjuda,
                        postType: 4,
                        icon: 'la-undo-alt',
                        text: 'Desfazer',
                        action: 'actDataSetCancelExecute',
                    })}

                </div>
            </div>
        </li>`
    );
};

interface DropdownButtonItem {
    text: string;
    icon: string;
    postType: number;
    action: string;
    Class: string;
    Alias: string;
    CadCons: string;
    TopicoAjuda: string;
}

const dropdownButtonItem = ({ text , icon, postType, action, Class, Alias, CadCons, TopicoAjuda}: DropdownButtonItem) => {
    return (/*html*/
        `<a id="btnpost${postType}" class="dropdown-item" data-posttype="${postType}" href="javascript:;" data-n-onclick="${action}" ${getStringDataAction(Class, Alias, CadCons, TopicoAjuda)}> 
            <i class="la ${icon}"></i>
            <span class="kt-nav__link-text ml-1">${text}</span>
        </a>`
    );
};