import dxDataGrid from 'devextreme/ui/data_grid';
import FormManager from "../../../../js/FormManager";
import { VerificaConfirmacaoAction } from "../../../../js/nDialog";
import { DefinicaoObjetosConfig } from "../../../types/page-config/page-config";
import { ToolButton, ActionConfirmation } from "../../fra-control/types";
import { ActionExecutarOperacaoDataSet, rowDataValues } from '../../../../js/nGrid';
import { ActExecuteProcesso } from '../../../../js/form/actions/ActExecuteProcesso';
import { AbrirCadastro, AbrirProcesso, AbrirProcessoModal } from "../../../../js/nWebUtil";
import DevExpress from 'devextreme/bundles/dx.all';
import { ActContabilizar } from "./actions/act-contabilizar";

declare var AbrirCadastroModal: any;
declare var CadastroExecuteAction: (props: CadastroExecuteActionProps) => void | null;

interface ActionProps {
    Class: string;
    Alias?: string;
    toolButton: ToolButton;
}

interface AdditionalOptions {
    shouldRefresh?: boolean;
    isFromGrid?: boolean;
    columnButtonClickEvent?: { // TnActExecuteProcesso
        component?: dxDataGrid | undefined;
        element?: DevExpress.core.dxElement | undefined;
        model?: any;
        event?: DevExpress.event | undefined;
        row?: DevExpress.ui.dxDataGridRowObject | undefined;
        column?: DevExpress.ui.dxDataGridColumn | undefined;
    }
}

interface ConfirmationHandler {
    Confirmacao: ActionConfirmation | undefined;
    fn: () => void;
    actionName?: string;
}

type ActionType =
    | 'TnAction'
    | 'TnActSalvarArquivo'
    | 'TnActAbrirProcesso'
    | 'TnActAbrirCadastro'
    | 'TnActOperacaoDataset'
    | 'TnActExecuteProcesso';

interface ToolButtonActionProps {
    DefinicaoObjetos: Pick<DefinicaoObjetosConfig, 'Class' | 'Alias' | 'TipoObjeto'>;
    toolButton: ToolButton;
    options?: AdditionalOptions;
    Params?: { // use on /RelatorioExibir// twAgenda
        [key: string]: string;
    };
}

export const getToolButtonAction = ({ DefinicaoObjetos, toolButton, options, Params }: ToolButtonActionProps) => {
    const { TipoAcao } = toolButton;
    const { Class, Alias, TipoObjeto } = DefinicaoObjetos;

    verifyExposeCustomAction(toolButton);

    switch (TipoAcao as ActionType) {
        case 'TnActAbrirCadastro': return getTnActAbrirCadastro({
            Alias,
            Class,
            toolButton,
            options,
        });
        case 'TnActAbrirProcesso': return getTnActAbrirProcesso({
            Alias,
            Class,
            toolButton,
            options,
        });
        case 'TnActOperacaoDataset': return getTnActOperacaoDataset({
            Class,
            TipoObjeto,
            toolButton,
            options,
            Params,
        });
        case 'TnActSalvarArquivo': return getTnActSalvarArquivo({
            Alias,
            Class,
            toolButton,
        });
        case 'TnAction': return getTnAction({
            Alias,
            Class,
            toolButton,
            Params,
        });
        case 'TnActExecuteProcesso': return getTnActExecuteProcesso({
            Alias,
            Class,
            toolButton,
            options,
        });
    }
}

const verifyExposeCustomAction = ({ Name, NameEvent }: ToolButton) => {
    if ((window as any)[NameEvent]) return;
    
    if (Name === 'actContabilizarCloud') {
        (window as any)['actContabilizarCloud'] = ActContabilizar;
    }
}

interface ExecuteActioFromPartialProps {
    DefinicaoObjetos: Pick<DefinicaoObjetosConfig, 'Class' | 'Alias' | 'TipoObjeto'>;
    toolButton: ToolButton;
    options?: AdditionalOptions;
}

export const executeActionFromPartial = ({ DefinicaoObjetos, toolButton, options }: ExecuteActioFromPartialProps) => { // call from delphi
    const action = getToolButtonAction({
        DefinicaoObjetos,
        toolButton,
        options,
    });

    action();
};

const getHandler = ({ Confirmacao, fn, actionName }: ConfirmationHandler) => Confirmacao
    ? () => VerificaConfirmacaoAction(Confirmacao, fn, actionName)
    : fn;

interface TcActAbrirCadastroProps extends ActionProps {
    options?: AdditionalOptions;
}

interface TnActAbrirProcessoOptions {
    ClasseDetalhe: string;
    DetailFields: string; // "field1;field2"
    Icon: string;
    CssClass: string;
    OpenModal: boolean;
    MasterFields: string; //"field1;field2"
	InsertOnOpen: boolean;
}
             
const getTnActAbrirCadastro = ({ Class, Alias = '', toolButton, options }: TcActAbrirCadastroProps) => {
    const { Opcoes, Confirmacao } = toolButton;
    const { MasterFields = '', DetailFields = '', ClasseDetalhe = '', OpenModal, InsertOnOpen } = (Opcoes as TnActAbrirProcessoOptions);

    const action = () => {
        const form = FormManager.byname(Class, Alias);
        const params = MasterFields
            ? form.fieldsvalues(MasterFields.split(';'), DetailFields.split(';'))
            : null;

        if (OpenModal) {
            AbrirCadastroModal({
                _AClasse: ClasseDetalhe,
                AFiltros: params,
                AGrid: null,
                ANovo: InsertOnOpen,
                AChaveMestre: MasterFields.split(';'),
                shouldRefresh: options?.shouldRefresh,
                AForm: form,
            });
        }
        else {
            AbrirCadastro(ClasseDetalhe, true, InsertOnOpen ? null : params);
        }
    };

    return getHandler({
        Confirmacao,
        fn: action,
    });
}

interface TnActAbrirProcessoProps extends ActionProps {
    options?: AdditionalOptions;
}

interface TnActAbrirProcessoOptions {
    ClasseProcesso: string;
    MasterFields: string; // "field1;field2;field3"
    Icon: string;
    CssClass: string;
    OpenModal: boolean;
    DetailParams: string; // "field1;field2;field3"
}

const getTnActAbrirProcesso = ({ Class, Alias = '', toolButton, options }: TnActAbrirProcessoProps) => {
    const { Opcoes, Confirmacao } = toolButton;
    const { MasterFields = '', DetailParams = '', ClasseProcesso = '', OpenModal } = (Opcoes as TnActAbrirProcessoOptions);
    const action = () => {
        const form = FormManager.byname(Class, Alias);
        const params = MasterFields
            ? form.fieldsvalues(MasterFields.split(';'), DetailParams.split(';'))
            : null;

        if (OpenModal) {
            AbrirProcessoModal({
                _AClasse: ClasseProcesso,
                AParams: params,
                AGrid: null,
                shouldRefresh: options?.shouldRefresh,
                form,
            } as any);
        }
        else {
            AbrirProcesso(ClasseProcesso, true, params);
        }
    };

    return getHandler({
        Confirmacao,
        fn: action,
    });
}

interface TnActOperacaoDataset extends ActionProps {
    TipoObjeto?: string;
    options?: AdditionalOptions;
    Params?: { // TwAgenda
        [key: string]: string;
    }
}

const getTnActOperacaoDataset = ({ Class, TipoObjeto, toolButton, Params, options: { isFromGrid = false } = {} }: TnActOperacaoDataset) => {
    const { Opcoes, Name, Confirmacao } = toolButton;
    const { datasetNames } = Opcoes;

    const action = () => {
        const params = Params || (isFromGrid ? FormManager.byname(Class)?.getValues() : null);

        ActionExecutarOperacaoDataSet({ 
            _AAction: Name,
            _AClasse: Class,
            _ADataset: datasetNames,
            _AGrid: datasetNames,
            _AEhProcesso: TipoObjeto === 'ntoProcesso',
            _AOpcoes: Opcoes,
            _AParams: params,
        });
    };

    return getHandler({
        Confirmacao,
        fn: action,
        actionName: Name,
    });
}

const getTnActSalvarArquivo = (props: ActionProps) => {
    return getTnAction(props);
}

interface TnActioProps extends ActionProps {
    Params?: { // use on /RelatorioExibir
        [key: string]: string;
    };
}

export interface CadastroExecuteActionProps {
    _AClasse: string;
    _AAlias?: string;
    _AMasterClass: string;
    _AAction: string;
    _AParams?: Pick<TnActioProps, 'Params'>;
}

const getTnAction = ({ Class, Alias = '', toolButton, Params }: TnActioProps) => {
    const { NameEvent, Confirmacao } = toolButton;
    const action = () => {
        CadastroExecuteAction({
            _AClasse: Class,
            _AAlias: Alias,
            _AMasterClass: Class,
            _AAction: NameEvent,
            _AParams: Params,
        });
    };

    return getHandler({
        Confirmacao,
        fn: action,
        actionName: NameEvent,
    });
}

interface TnActExecuteProcesso extends ActionProps {
    options?: AdditionalOptions;
}

const getTnActExecuteProcesso = ({ Class, Alias, toolButton, options = {} }: TnActExecuteProcesso) => {
    const { Opcoes, Confirmacao } = toolButton;
    const { MasterFields = '', ClasseProcesso = '', DetailParams = '' } = Opcoes;
    const { isFromGrid, columnButtonClickEvent } = options;

    const action = async () => {
        const form = FormManager.byname(Class, Alias);
        const params = isFromGrid && columnButtonClickEvent
            ? rowDataValues(columnButtonClickEvent, MasterFields, DetailParams)
            : form.fieldsvalues(MasterFields.split(';'), DetailParams.split(';'));

        ActExecuteProcesso({
            _AClass: ClasseProcesso,
            _AParams: params,
        });
    };

    return getHandler({
        Confirmacao,
        fn: action,
    });
};