import FormManager from "../../../../js/FormManager";
import WizardManager from "../../../../js/WizardManager";

export class Shortcut {
    private _keys: Map<string, Map<string, Function>>;

    constructor ( ) {
        this._keys = new Map();

        this._initHandler();
    }

    private _initHandler() {
        const getForm = (Class: string, Alias = '') => {
            if (Class) return FormManager.byname(Class, Alias);
            if (WizardManager.context) return FormManager.byname(WizardManager.context.DetalheAtivo)
    
            return (FormManager.itens as any).firstUsable();
        };

        $(document).on('keydown', (event) => {
            if (event.ctrlKey && event.key != 'Control') {
                const parent = $(document.activeElement!).closest('[data-shortcut]');
                const Class = parent.data('class');
                const Alias = parent.data('alias');
                const form = getForm(Class, Alias);

                if (!form) return;
                
                const formHandler = this._keys.get(`${form.classe}${form.alias || ''}`);
                const handler = formHandler?.get(event.key);

                if (handler) {
                    try {
                        handler.call(undefined, event, form);
                    }
                    catch (e) {
                        console.error(e);
                    }

                    event.preventDefault();
                };
            }
        });
    }

    bind({Class, Alias = '', handler, caption}: { Class: string, Alias?: string, caption: string, handler: Function }) {
        const shortcutForm = this._keys.get(`${Class}${Alias}`);

        if (shortcutForm) {
            for(const char of caption) {
                if (!shortcutForm.has(char.toLowerCase())) {
                    shortcutForm.set(char.toLowerCase(), handler);
    
                    break;
                }
            }
        }
        else {
            this._keys.set(`${Class}${Alias}`, new Map([[caption[0].toLowerCase() , handler]]));
        }
    }

    unbind({Alias = '', Class, key}: { Class: string, Alias?: string, key?: string}) {

        if (!key) {
            this._keys.delete(`${Class}${Alias}`);
        }
        else {
            const shortcutForm = this._keys.get(`${Class}${Alias}`);
            
            shortcutForm?.delete(key)
        }
    }
}