interface FloatingMenuProps {
    title: string;
    items: FloatingMenuItems;
}

export type FloatingMenuItems = Array<{
    el: JQuery<HTMLElement>;
    caption: string;
}>;

export const FloatingMenu = (props: FloatingMenuProps) => {
    const wrapper = getWrapper();
    const button = getButton();
    const dismissButton = getDismissButton();
    const header = getHeader();
    const text = getText(props.title);
    const content = getContent(props.items);

    header.append([button, text, dismissButton])

    wrapper.append([header, content]);

    setEvent({
        wrapper,
        button,
        dismissButton,
        header,
        content,
        text,
        items: props.items,
    });

    return wrapper;
};

const getWrapper = () => {
    return $('<div>')
        .attr('class', "bg-white floating-menu")
        .css({
            position: 'fixed',
            right: '40px',
            border: '1px solid var(--gray-200)',
            width: '40px',
            'border-radius': '3px',
            'z-index': 100,
            'box-shadow': '0 8px 16px 0 rgba(30, 30, 36, 0.1)',
        });
};

const getHeader = () => {
    return $('<div>')
        .attr('class', 'd-flex align-items-center')
        .css('height', '40px')
};

const getButton = () => {
    return $('<span>')
        .attr('class', 'btn btn-icon')
        .append('<i class="las la-align-left">')
};

const getText = (text: string) => {
    return $(`<span style="display: none;">${text}</span>`)
};

const getDismissButton = () => {
   return $('<span>')
        .attr('class', 'ml-auto mr-2')
        .css({
            cursor: 'pointer',
            display: 'none'
        })
        .append('<i class="las la-times" style="font-size: 1rem">');
}

const getContent = (items: FloatingMenuItems) => {
    const wrapper = $('<div>').css('display', 'none');
    const divider = $('<div class="dropdown-divider mt-0">');
    const ul = $('<ul>')
        .attr({
            class: 'p-1 bm-0',
        })
        .css({
            height: '300px',
            overflow: 'auto',
            'list-style': 'none'
        })

    ul.append(items.map(({ caption, el }, index) => {
        const li = $(
                `<li class="${index ? '' : 'active'}">
                    <a>
                        <span>${caption}</span>
                    <a>
                </li>`
            )
            .on('click', () => {
                setTimeout(() => {
                    const scrollEl = getElementToScroll(el);

                    scrollTo(0, (scrollEl.offset()?.top || 0) - 56 - 48); // -header -subheader
                });
            });

        return li;
    }));

    ul.on('click', 'li', ({ currentTarget }) => {
        $('li', ul).removeClass('active');

        $(currentTarget).addClass('active');
    });

    return wrapper.append([divider, ul]);
}

const getElementToScroll = (el: JQuery<HTMLElement>) => {
    const isTab = el.hasClass('tab-pane');
    const isCard = el.hasClass('card')
    
    if (isTab) return el.closest('.row');
    if (isCard) return el;

    return el.closest('.card');
};

interface eventListenerProps {
    wrapper: JQuery<HTMLElement>;
    button: JQuery<HTMLElement>;
    dismissButton: JQuery<HTMLElement>;
    header: JQuery<HTMLElement>;
    content: JQuery<HTMLElement>;
    text: JQuery<HTMLElement>;
    items: FloatingMenuItems;
}

const setEvent = ({ wrapper, button, dismissButton, content, text, items, header }: eventListenerProps) => {
    button.on('click', () => {
        wrapper.css({
            width: '250px',
            height: '370px'
        });

        content.show();
        text.show();
        dismissButton.show();
        header.css('height', '60px');
    });

    dismissButton.on('click', () => {
        wrapper.css({
            width: '40px',
            height: '40px',
        });

        content.hide();
        text.hide();
        dismissButton.hide();
        header.css('height', '40px');
    });


    $(window).on('resize scroll', () => {
            const elInView = items.find(({ el }) => isElementInViewport(el))!;
            const elInViewIndex = items.indexOf(elInView);
            
            $('li', content).each((index, el) => {
                if (index == elInViewIndex) $(el).addClass('active');
                else $(el).removeClass('active');
            })
    });
}

const isElementInViewport = (el: JQuery<HTMLElement>) => {
    const element = getElementToScroll(el);
    var rect = element.get(0)!.getBoundingClientRect();

    return rect.top >= 0;
}