import widgetManager from "../widget-manager";
import { TwCardDisplayerConfig, WidgetPropsBase } from "../types";
import { WidgetWithExpression } from "../widget-with-expression";

interface TwCardDisplayerProps extends Omit<WidgetPropsBase, 'card' | 'config'> {
	widget: TwCardDisplayerConfig;
    Class: string;
	isLayout?: boolean;
}

export class TwCardDisplayer extends WidgetWithExpression<TwCardDisplayerConfig, TwCardDisplayerProps> {//WidgetBase<TwCardDisplayerConfig, TwCardDisplayerProps> {
	constructor(config: TwCardDisplayerProps) {
		super(config);

		widgetManager.set(config.widget.Name, this);
	}

	protected _expressionValueSetValue(value: string): void {
		this.setValue({ Valor: value });
	}

	protected _build(): JQuery<HTMLElement> {
		const { Action, Title, Text, Name, StyleClass, Valor, CorValorNegativo, CorValorPositivo } = this.getState().widget;
		const { isLayout } = this.getState();
        const color = (Number(Valor) < 0 ? CorValorNegativo : CorValorPositivo) || 'var(--gray-800)';

		return $(`
			<div id="widget_${Name}" class="${isLayout ? 'col-12' : 'col-sm-12 col-md-6 col-lg-3'}" ${Action ? `style="cursor: pointer" onclick="${Action}()"` : ''}>
				<div class="widget-workspace-folha" style="padding-left: 1.8rem; padding-right: 1.8rem;">
					<div class="widget-workspace-folha-details" style="flex-direction: column">
						<div class="widget-workspace-folha-info" style="align-self: flex-start; min-height: 5rem">
							<h4 class="widget-workspace-folha-title">
									${Title}
							</h4>
							<span class="widget-workspace-folha__desc">
								${Text}
							</span>
						</div>
						<span id="widget_valor_${Name}" class="widget-workspace-folha-stats kt--font-${StyleClass}" style="align-self: flex-end; color: ${color};">
							${Valor} 
						</span>	 
					</div> 
					<div class="progress progress--sm">
						<div class="progress-bar kt--bg-${StyleClass}" role="progressbar" style="width: 100%;" aria-valuenow="100" aria-valuemin="0" aria-valuemax="100"></div>
					</div>					   			      
				</div>
			</div>
		`);
	}
}