import { DefinicaoObjetosConfig } from "../../../types/page-config/page-config";
import { TwCardSeletorConfig, WidgetConfigBase, WidgetPropsBase } from "../types";
import { getToolButtonAction } from "../../toolbuttons/toolbutton-action/toolbutton-action";
import { WidgetBase } from "../widget-base";
import widgetManager from "../widget-manager";

interface TwCardSeletorProps extends Omit<WidgetPropsBase, 'card' | 'config'> {
    widget: TwCardSeletorConfig;
    DefinicaoObjetos: DefinicaoObjetosConfig;
}

export class TwCardSeletor extends WidgetBase<TwCardSeletorConfig, TwCardSeletorProps> {

    constructor(config: TwCardSeletorProps) {
        super(config);

        widgetManager.set(config.widget.Name, this);
    }

    protected _build(): JQuery<HTMLElement> {
        const { widget } = this.getState();
        const { DefinicaoObjetos } = this.getState();
        const { Itens, Name } = widget;

        const wrapper = $(`<div id="widget_${widget.Name}" class="d-flex flex-wrap justify-content-center w-100">`);
        
        const items = Itens.map(({ Image, Titulo, Action, Visible }, index) => {
            const footer = $(
                `<div class="card-footer p-4" style="cursor: pointer">
                    <div class="row d-flex align-items-center justify-content-between">
                        <span class="col-10" style="color: var(--gray-800);">
                            <b>Selecionar</b>
                        </span>
                        <div>
                            <i class="float-right las la-angle-right"></i>
                        </div>
                    </div>
                </div>`).on('click', getToolButtonAction({ 
                    DefinicaoObjetos, 
                    toolButton: Action, 
                    options: { 
                        shouldRefresh: false 
                    } 
                }));


            const card = $(
                `<div 
                    id="widget_${index}_${Titulo}"
                    class="card card-custom col-2 p-0 m-4 ${index < (Itens.length - 1) ? 'mr-8' : ''}"
                    style="border: 1px solid; border-color: var(--gray-200); box-shadow: 1px 1px 3px 0px rgb(30 30 36 / 20%);"
                    ${Visible ? '' : ' hidden'}
                >
                    <div class="card-body p-0">
                        <div class="card-header p-0 border-0 pt-5 justify-content-center">
                            <h3 class="text-center" style="color: var(--gray-800);">${Titulo}</h3>
                        </div>
                        <div class="card-body text-center align-items-center">
                            <img src="data:image/png;base64, ${Image}" style="width: 4.5rem; height: 4.5rem;">
                        </div>
                    </div>
                </div>`
            );
            
            return card.append(footer);
        });

        return wrapper.append(items);
    };

    public setVisibleCard(index: number, state: boolean){
        const { widget } = this.getState();
        const { Itens } = widget;
        Itens[index].Visible = state;
        this.setState({});
    };
};