import CustomStore from "devextreme/data/custom_store";
import { FormatErrorMessage } from "../../../../../js/nDialog";
import axios, { AxiosError } from "axios";
import DevExpress from "devextreme/bundles/dx.all";
import { verifyAxiosError } from "../../../../../js/nRequests";

interface Props {
    gridName: string;
    className: string;
    dataId: number | string;
}

interface RequestParams {
    _AID: number | string;
    _ANome: string; // dataset name
    skip: number;
    take: number;
    filter: string;
    sort?: string;
    group?: string;
    groupSummary?: string;
    totalSummary?: string;
    [key: string]: any;
}

export class CustomStoreProcessOnDemand extends CustomStore { }

export const createProcessDataSourceOnDemand: (p: Props) => CustomStore = ({ gridName, dataId }) => new CustomStoreProcessOnDemand({
    load: (loadOptions) => {
        return new Promise(async (resolve, reject) => {
            const params: Partial<RequestParams> = {
                _ANome: gridName,
                _AID: dataId,
                ...getLoadOption(loadOptions, 'skip', 0),
                ...getLoadOption(loadOptions, 'take', Number.MAX_SAFE_INTEGER),
                ...getLoadOption(loadOptions, 'filter', ''),
                ...getLoadOption(loadOptions, 'sort'),
                ...getLoadOption(loadOptions, 'group'),
                ...getLoadOption(loadOptions, 'groupSummary'),
                ...getLoadOption(loadOptions, 'totalSummary'),
                ...getLoadOption(loadOptions, 'requireGroupCount'),
                ...getLoadOption(loadOptions, 'requireTotalCount'),
            };

            try {
                const { data } = await axios.get('nHttpServicos/DadosProcesso', { params });
                const result = data.data;

                if (result.Erro) {
                    FormatErrorMessage(result.Erro);
                }

                resolve({
                    data: result.items.data,
                    totalCount: result.items.totalCount,
                    summary: result.items.summary,
                    groupCount: result.items.groupCount,
                });
            }
            catch (e) {
                console.error(e);

                verifyAxiosError(e as AxiosError);
                
                reject('Erro ao carregar dados');
            }
        });
    }
});

const getLoadOption = (loadOptions: DevExpress.data.LoadOptions, name: keyof DevExpress.data.LoadOptions, def?: string | number) => {
    if (loadOptions[name] != undefined) return { [name]: JSON.stringify(loadOptions[name]) }

    if (def != undefined) return { [name]: def };

    return {};
};