import FormManager from "../../../../js/FormManager";
import { Formulario } from "../../../../js/nFormulario";
import { getFormFieldsContainer, instantiateForm } from "../../form-repeater/form/form";
import { Field, FormProp, GroupColumn } from "../../fra-control/types";
import { getFormId } from "../../fra-control/util/util";
import { createFooterToolbuttons, createPageToolbuttons, createToolButtons } from "../../toolbuttons/toolbuttons";
import { Card, CardBuilder } from "../layout/card";
import { FloatingMenuItems } from "../layout/floating-menu";
import { LayoutScopeConfig, LayoutOptions } from "../layout/layout";
import { TwFieldListConfig } from "../types";

interface TwFieldListProps {
    widget: TwFieldListConfig;
    config: LayoutScopeConfig;
    card: Card;
    floatingMenuItems?: FloatingMenuItems;
    options?: LayoutOptions;
}

declare const SessionManager: any;

export const TwFieldList = ({ widget, config, card, floatingMenuItems, options = {} }: TwFieldListProps) => {
    if (!config.fields) throw new Error('No existing fields');
    
    const { DefinicaoObjetos: { ntfCadPage, Caption, Class, Alias } } = config;
    const { isFrom = {}, isWorkSpace, isWizard } = options;
    const { fields } = config;
    const fieldsConfig = widget.FieldList
        .filter((fieldName) => fields.has(fieldName))
        .map((fieldName) => fields.get(fieldName)) as (Field | GroupColumn)[]; // its guaranteed by configuration to have at least one field

    const content = buildForm({ 
        config,
        fields: fieldsConfig,
    });

    if (widget.isMaster) {
        if (isFrom.twMasterDetail && options.pageOptions) {
            options.pageOptions.toolbarBaseElement.append(createPageToolbuttons({
                scope: config,
                uuid: options.pageOptions.uuid,
            }));
            
            return content;
        }
        else if (ntfCadPage || isWorkSpace || isWizard || isFrom.twMasterDetail) CardBuilder.setToolbar(card, createToolButtons(config));
        else card.cardElement.append(createFooterToolbuttons(config));

        CardBuilder.setTitle(card, config.DefinicaoObjetos.Caption);

        card.cardElement.attr({
            tabindex: "-1",
            'data-alias': Alias || '',
            'data-class': Class,
            'data-shortcut': "true",
        })

        floatingMenuItems?.push({
            el: card.cardElement,
            caption: Caption,
        });

        return content;
    }

    return content;
}

interface BuildFormProps {
    config: Pick<LayoutScopeConfig, 'DefinicaoObjetos'>;
    fields: (Field | GroupColumn)[];
    formAttr?: {
        [key: string]: any;
    };
}

type BuildForm = (props: BuildFormProps) => JQuery<HTMLElement> | JQuery<HTMLElement>[]

export const buildForm: BuildForm = ({ config, fields, formAttr = {} }) => {
    const { DefinicaoObjetos } = config;
    const formProp = {
        Alias: DefinicaoObjetos.Alias,
        Class: DefinicaoObjetos.Class,
        MultiColuna: DefinicaoObjetos.MultiColuna,
        caption: DefinicaoObjetos.Caption,
        detailFields: DefinicaoObjetos.DetailFields,
        extraConfig: DefinicaoObjetos.configextra,
        fixedFilter: DefinicaoObjetos.FiltroFixo,
        isDetailForm: DefinicaoObjetos.FormDetalhe,
        masterClass: DefinicaoObjetos.MasterClass,
        masterFields: DefinicaoObjetos.MasterFields,
        objectType: DefinicaoObjetos.TipoObjeto,
        primaryKey: DefinicaoObjetos.ChavePrimaria,
        hasSessionSelecao: Boolean(SessionManager.selecao),
        toolButtons: DefinicaoObjetos.ToolButtons,
        salvarDadosAutomatico: DefinicaoObjetos.SalvarDadosAutomatico,
    };

    const formInstance = FormManager.byname(DefinicaoObjetos.Class, DefinicaoObjetos.Alias);

    if (!formInstance) {
        const formEl = $('<form>').attr({
            id: getFormId(DefinicaoObjetos),
            class: `kt-form form${DefinicaoObjetos.Class}${DefinicaoObjetos.Alias || ''}`,
            'data-class': DefinicaoObjetos.Class,
            ...formAttr,
        });

        const formInstance = instantiateForm({
            formProp,
            formEl,
        });

        FormManager.insertForm(formInstance);

        const row = createFields({
            formInstance,
            formProp,
            item: fields,
        });

        return [formEl, row];
    }
    else {
        return createFields({
            formInstance,
            formProp,
            item: fields,
        });
    }
}

interface CreateFieldsProp {
    formInstance: Formulario;
    formProp: FormProp;
    item: (Field | GroupColumn)[];
}

type CreateFields = (prop: CreateFieldsProp) => JQuery<HTMLElement>;

const createFields: CreateFields = ({ formInstance, formProp, item }) => { // = ({ formInstance, item, formProp }: { formInstance: Formulario, item: (Field | GroupColumn)[], formProp: FormProp }): JQuery<HTMLElement> => {
    const { row } = getFormFieldsContainer({
        formProp,
        item,
        formInstance,
    });

    $(() => {
        FormManager.createAllFields(formInstance, {
            rootElement: row,
        })
    });

    return row.addClass('kt-form bg-white form-fields');
}