import widgetManager from "../widget-manager";
import { TwImageValueDisplayerConfig, WidgetPropsBase } from "../types";
import { WidgetWithExpression } from "../widget-with-expression";

interface TwImageValueDisplayerProps extends Omit<WidgetPropsBase, 'card' | 'config'> {
    widget: TwImageValueDisplayerConfig;
    Class: string;
}

export class TwImageValueDisplayer extends WidgetWithExpression<TwImageValueDisplayerConfig, TwImageValueDisplayerProps> {
    constructor(config: TwImageValueDisplayerProps) {
        super(config);

        widgetManager.set(config.widget.Name, this);
    }

    protected _expressionValueSetValue(value: string): void {
        this.setValue({ Valor: value });
    }

    protected _build(): JQuery<HTMLElement> {
        const { Action, colunas, Name, TextColor, image, ValueColor, SubTitulo, Valor, Text, CorValorNegativo, CorValorPositivo } = this.getState().widget;

        const color = (Number(Valor) < 0 ? CorValorNegativo : CorValorPositivo) || `#${ValueColor}`;

        return $(`
            <div id="widget_${Name}" class="${colunas ? `col-sm-6 col-lg-${colunas}` : 'col'}">
                <a class="" href="#" ${Action ? `onclick="NavigateWizard(this, '${Action}', 'widget.${Name}')"` : ''}>
                    <div class="widget-thumb widget-bg-color-white text-uppercase margin-bottom-20 ${Text ? 'bordered' : ''}">
                        ${Text ? `<h4 class="widget-thumb-heading" style="color:#${TextColor}" id="widget_title_${Name}">${Text}</h4>` : ''}
                        <div class="widget-thumb-wrap">
                            <i 
                                class="widget-thumb-icon btn-warning"
                                style="background-image:url('data:image/png;base64,${image || ''}')${Text ? '' : `;width:30px;height:30px;`}"
                                id="widget_image_${Name}"
                            ></i>
                            <div class="widget-thumb-body">
                                <span 
                                    class="widget-thumb-subtitle" 
                                    style="color:${color}${Text ? '' : ';font-size:10px;'}"
                                    id="widget_SubTitulo_${Name}"
                                >${SubTitulo}</span>
                                <span 
                                    class="widget-thumb-body-stat"
                                    data-counter="counterup"
                                    data-value="${Valor}"
                                    style="color:${color};text-align:right${Text ? '' : ';font-size:10px;'}"
                                    id="widget_valor_${Name}"
                                    >${Valor}</span>
                            </div>
                        </div>
                    </div>
                </a>
            </div>
        `);
    }
}