import { DefinicaoObjetosConfig } from "../../../types/page-config/page-config";
import { TwMenuConfig, TwMenuItem, WidgetPropsBase } from "../types";

interface TwMenuProps extends Omit<WidgetPropsBase, 'card' | 'config'> {
    widget: TwMenuConfig;
    DefinicaoObjetos: DefinicaoObjetosConfig,
}

export const TwMenu = ({ DefinicaoObjetos, widget }: TwMenuProps) => {
    const { Class, Text, Itens, Name } = widget;

    const li = $(`
        <li id="widget_${Name}" class="nav-item d-inline-block" aria-expanded="true" style="margin-bottom: 5px; margin-top: 5px;">
            <div 
                ${Class ? getActionDataString(Class, DefinicaoObjetos) : 'data-toggle="dropdown"'}
                class="twMenu nav-link active"
                style="cursor: pointer;"
                aria-expanded="false">
                    <span class="d-none d-xs-inline">Ações</span>
                    <h2 class="h2 card-title">${Text}</h2>
            </div>
        </li>
    `)

    if (!Class && Itens) {
        li.append(getDropDown({ DefinicaoObjetos, Itens }));
    }

    return li;
};

interface GetDropDownProps {
    Itens: TwMenuItem[];
    DefinicaoObjetos: DefinicaoObjetosConfig;
}

const getDropDown = ({ Itens, DefinicaoObjetos }: GetDropDownProps) => {
    const dropdown = $('<div class="dropdown-menu " x-placement="bottom-end">');
    const ul = $('<ul class="navi navi-hover py-1 identCadastro">');

    const items = Itens.map((item) => getItem({ item, DefinicaoObjetos }));

    ul.append(items);
    dropdown.append(ul)

    return dropdown;
};

interface GetItemProps {
    item: TwMenuItem;
    DefinicaoObjetos: DefinicaoObjetosConfig;
}

const getItem = ({ DefinicaoObjetos, item }: GetItemProps) => {
    const { Caption, Class, Hint, disabled, Itens } = item;

    if (Itens) return getItemSubmenu({ item, DefinicaoObjetos });

    return $(
        `<li class="navi-item ${disabled ? 'navi-item-disabled' : ''}">
            <a class="navi-link" data-posttype="0" style="cursor: pointer;" ${Hint ? `title="${Hint}"` : ''}${getActionDataString(Class!, DefinicaoObjetos)}> 
                <span class="navi-text">${Caption}</span>
            </a>
        </li>`
    );
};

const getActionDataString = (redirectClass: string, { Class }: DefinicaoObjetosConfig) => {
    return [
        'data-create-naction="true"',
        'data-n-onclick="actOpenWorkSpace"',
        'data-block-page-on-click',
        `data-class="${Class}"`,
        `data-redirect-class="${redirectClass}"`,
    ].join(' ');
};

export const getItemSubmenu = ({ item, DefinicaoObjetos }: GetItemProps) => {
    const { Caption, Itens } = item;
    const itemEL = $(`
        <li class="navi-item dropdown-submenu pull-right">
            <a class="navi-link" style="cursor: pointer;">
                <span class="navi-text">${Caption}</span>
                <i class="navi-text navi-icon la la-angle-down"></i>
            </a>
        </li>`).on('mouseover', () => {
        const { height, top } = dropdownMenu.get(0)!.getBoundingClientRect();
        const windowHeight = window.visualViewport!.height;
        const treshold = height + top - windowHeight + 10;

        if (treshold > 0) dropdownMenu.css('top', `-${treshold}px`);
    });

    const dropdownMenu = $('<ul class="dropdown-menu w-100" >');

    const submenuItems = Itens!.map((item) => getItem({ item, DefinicaoObjetos }));

    dropdownMenu.append(submenuItems);

    return itemEL.append(dropdownMenu);
};