import widgetManager from "../widget-manager";
import { TwMiniCardValueDisplayerConfig, WidgetPropsBase } from "../types";
import { WidgetWithExpression } from "../widget-with-expression";

interface TwMiniCardValueDisplayerProps extends Omit<WidgetPropsBase, 'card' | 'config'> {
    widget: TwMiniCardValueDisplayerConfig;
    isLayout: boolean;
    Class: string;
}

export class TwMiniCardValueDisplayer extends WidgetWithExpression<TwMiniCardValueDisplayerConfig, TwMiniCardValueDisplayerProps> {
    constructor(config: TwMiniCardValueDisplayerProps) {
        super(config);

        widgetManager.set(config.widget.Name, this);
    }

    protected _expressionValueSetValue(value: string): void {
        this.setValue({ value });
    }

    protected _build(): JQuery<HTMLElement> {
        const { isLayout, widget: { Data, action, marker, title, value, Name, CorValorNegativo, CorValorPositivo } } = this.getState();

        const color = (Number(value) < 0 ? CorValorNegativo : CorValorPositivo) || `var(--gray-600)`;

        var el = $(
            `<div id="widget_${Name}" class="btn-group btn-group-toggle twMiniCardValueDisplayer-group"role="group" ${action ? 'data-toggle="button"' : ''}>
                <label 
                    class="btn btn-light btn-elevate btn-square twMiniCardValueDisplayer${isLayout ? '' : ' col-sm-3 col-lg-2 col-xl-1'}"  
                    data-filter-card="${Data}"
                    ${action ? `onclick="${action}(this)"` : ''}
                >
                    <input display="none" type="radio" name="twMiniCardValueDisplayer">
                    <span class="twMiniCardValueDisplayer-title"> 
                        ${marker ? `<span class="twMiniCardValueDisplayer-marker"></span>` : ''}
                        ${title} 
                    </span>
                    <span class="twMiniCardValueDisplayer-value" style="color: ${color}"> 
                        ${value}
                    </span>
                </label>
            </div>`
        );

        if (action) {
            el.on('click', () => {
                $(".twMiniCardValueDisplayer").removeClass("active");
                $(el).addClass("active");
            });
        }

        return el;
    }
}