import dxDataGrid from "devextreme/ui/data_grid";
import style from './tw-msg-lista.scss';
import { TwMsgListaConfig, WidgetPropsBase } from "../types";

interface TwMsgListaProps extends Omit<WidgetPropsBase, 'card' | 'config'> {
    widget: TwMsgListaConfig;
}

export const TwMsgLista = ({ widget: { Name, MsgLista: { Columns, Items } } }: TwMsgListaProps) => {
    const grid = new dxDataGrid($(`<div id="widget_${Name}">`), {
        showBorders: false,
        showRowLines: true,
        showColumnLines: false,
        wordWrapEnabled: true,
        columnAutoWidth: true,
        pager: {
            visible: false,
        },
        paging: {
            pageSize: Items.length + 1,
        },
        dataSource: Items.map((values) => values.reduce((row, value, index) => ({
            ...row,
            [`${index}`]: value
        }), {})),
        columns: Columns.map((columnDescription, index) => ({
            dataField: `${index}`,
            caption: columnDescription,
            allowSorting: false,
            cssClass: style['font'],
            headerCellTemplate: (element, options) => {
                return element
                    .addClass(style['font-bolder'])
                    .append(options.column?.caption || '');
            }
        })),
    });

    return grid.element();
};