import { Field, GroupColumn } from "../../fra-control/types";
import { createFooterToolbuttons, createToolButtons } from "../../toolbuttons/toolbuttons";
import { Card, CardBuilder } from "../layout/card";
import { FloatingMenuItems } from "../layout/floating-menu";
import { LayoutScopeConfig, LayoutOptions } from "../layout/layout";
import { buildForm } from "../tw-field-list/tw-field-list";
import { TwParamlistConfig } from "../types";


interface TwParamListProps {
    card: Card;
    config: LayoutScopeConfig;
    widget: TwParamlistConfig;
    floatingMenuItems?: FloatingMenuItems;
    options?: LayoutOptions;
}

export const TwParamList = ({ card, config, widget, floatingMenuItems, options = {} }: TwParamListProps) => {
    if (!config.fields) throw new Error('No existing fields');

    const { fields, DefinicaoObjetos} = config;
    const { Caption, TipoObjeto, Class, Alias } = DefinicaoObjetos;
    const { isFrom = {}, isWizard } = options;
    const { isMaster, FieldList } = widget;
    const fieldsConfig = (FieldList || [])
        .map((fieldName) => fieldName.toUpperCase())
        .filter((fieldName) => fields.has(fieldName))
        .map((fieldName) => fields.get(fieldName)) as (Field | GroupColumn)[];

    const content = buildForm({
        config,
        fields: fieldsConfig,
        formAttr: {
            action: TipoObjeto === 'ntoRelatorio' ? 'RelatorioImprimir' : 'ProcessoExecutar',
        }
    });

    if (isMaster) {
        if (isFrom.twMasterDetail || isWizard && false) CardBuilder.setToolbar(card, createToolButtons(config));
        else if (isWizard) {
            card.cardHeader.addClass('hidden');
            card.cardElement.css('box-shadow', 'none');
        }
        else card.cardElement.append(createFooterToolbuttons(config));
        
        CardBuilder.setTitle(card, Caption);

        card.cardElement.attr({
            tabindex: "-1",
            'data-alias': Alias || '',
            'data-class': Class,
            'data-shortcut': "true",
        })

        floatingMenuItems?.push({
            el: card.cardElement,
            caption: Caption,
        });

        return [
            ...(Array.isArray(content) ? content : [content]),
            buildResponseContent()
        ];
    }

    return content;
};

const buildResponseContent = () => $(`
    <div id="resultados">
        <div id="Conteudo"></div>
    </div>
`);