import FormManager from "../../../../js/FormManager";
import { Formulario } from "../../../../js/nFormulario";
import { DefinicaoObjetosConfig } from "../../../types/page-config/page-config";
import { Field } from "../../fra-control/types";
import { WidgetConfigBase } from "../types";
import '../../../../css/kt-subheader-search.css';
import { FraControl } from "../../fra-control/fra-control";
import SessionManager from "../../../../js/SessionManager";

interface TwSearchConfigProps {
    widget: TwSearchConfig;
    DefinicaoObjetos: DefinicaoObjetosConfig;
}

export interface TwSearchConfig extends WidgetConfigBase {
    EditLog: {
        Title: string; 
        Text: string;
    };
    Fields: Array<Field>; 
    MultiColuna: boolean;
    twBuscaRapida: true;
}

export const TwSearch = ({ widget, DefinicaoObjetos }: TwSearchConfigProps) => {
    const { EditLog, Fields,MultiColuna } = widget;
    const { Class, Alias } = DefinicaoObjetos;

    const wrapper = $(/*html*/`<div class="row col p-0"></div>`).css({
        margin: '0 -1rem',
        'max-width': 'fit-content',
    });
    
    const log = EditLog 
        ? $(/*html*/
            `<div id="editLog" class="card card-custom col-md-4" style="margin: 0; height: auto">
                <div class="card-header" style="border-bottom: 1px solid #ebedf2!important">
                    <h3 class="card-title">
                        ${EditLog.Title}
                    </h3>
                </div>
                <div class="card-body">
                    <div class="tab-content">
                        <div class="tab-pane active" id="kt_widget4_tab1_content">
                            <div class="scroll" data-scroll="true" id="scrollableTextWidget" data-height="400"
                                style="max-height: 200px;">${EditLog.Text}</div>
                        </div>
                    </div>
                </div>
            </div>`
        )
        : $();

    const search = $(/*html*/
        `<div id="kt-subheader-search" class="kt-subheader-search col-md-${EditLog ? '8' : '12'}">
            <span class="kt-input-icon__icon kt-input-icon__icon--right cursor-pointer float-right toggle-search">
                <i class="la la-angle-down font-size2d2"></i>
            </span>
            <h3 class="kt-subheader-search__title">
                Busca Rápida
                <span class="kt-subheader-search__desc" style="display:block; padding-left: 0px; margin-bottom: 2rem"> Utilize
                    os filtros para facilitar a busca pela transação </span>
            </h3>

            <form class="kt-form">
                <div class="row col-md-12 kt-grid kt-grid--desktop kt-grid--ver-desktop">
                    <div class="col-md-10 kt-grid__item kt-grid__item--middle search-widget-fields-row">
                        <div class="row display-filter">
                        </div>
                    </div>
                    <div class="col-md-2 kt-grid__item kt-grid__item--bottom kt-padding-b-5 kt-padding-t-5 kt-block-center">
                        <div class="kt--visible-tablet-and-mobile"></div>
                        <button type="button"
                            class="btn btn-pill btn-upper btn-bold btn-font-sm kt-subheader-search__submit-btn"
                            style="position:absolute; bottom: 0px;">
                            Consultar
                        </button>
                    </div>
                </div>
            </form>
        </div>`
    );

    search.find('.row.display-filter').append(Fields.map((field) => {
        const wrapper = $(/*html*/` <div class="kt-input-icon kt-input-icon--pill kt-input-icon--right col-md-6 col-lg-4 search-field-item mt-auto ${field.Expansivel ? 'search-expandable' : ''}"
                style="padding: 5px 0px; margin: 0">
            </div>`)

            console.log('formProp', {
                Alias: DefinicaoObjetos.Alias,
                Class: DefinicaoObjetos.Class,
                MultiColuna: MultiColuna,
                caption: DefinicaoObjetos.Caption,
                detailFields: DefinicaoObjetos.DetailFields,
                extraConfig: DefinicaoObjetos.configextra,
                fixedFilter: DefinicaoObjetos.FiltroFixo,
                isDetailForm: DefinicaoObjetos.FormDetalhe,
                masterClass: DefinicaoObjetos.MasterClass,
                masterFields: DefinicaoObjetos.MasterFields,
                objectType: DefinicaoObjetos.TipoObjeto,
                primaryKey: DefinicaoObjetos.ChavePrimaria,
                hasSessionSelecao: Boolean(SessionManager.selecao),
                toolButtons: DefinicaoObjetos.ToolButtons,
            })

        return wrapper.append(FraControl({
            formProp: {
                Alias: DefinicaoObjetos.Alias,
                Class: DefinicaoObjetos.Class,
                MultiColuna: MultiColuna,
                caption: DefinicaoObjetos.Caption,
                detailFields: DefinicaoObjetos.DetailFields,
                extraConfig: DefinicaoObjetos.configextra,
                fixedFilter: DefinicaoObjetos.FiltroFixo,
                isDetailForm: DefinicaoObjetos.FormDetalhe,
                masterClass: DefinicaoObjetos.MasterClass,
                masterFields: DefinicaoObjetos.MasterFields,
                objectType: DefinicaoObjetos.TipoObjeto,
                primaryKey: DefinicaoObjetos.ChavePrimaria,
                hasSessionSelecao: Boolean(SessionManager.selecao),
                toolButtons: DefinicaoObjetos.ToolButtons,
            },
            item: {
                ...field,
                NumeroColunas: 12,
            }
        }));
    }))

    search
        .find('.toggle-search')
        .on('click', (event) => {
            const element = event.currentTarget;

            $(element)
                .find('i')
                .toggleClass('rotate180');

            search
                .find(".kt-input-icon.search-expandable")
                .toggle();

            log
                .find("#scrollableTextWidget")
                .css("max-height", ($(".search-widget-fields-row").height() || 0) + 70) // fields + (padding + margin)
    });

    search
        .find('button.kt-subheader-search__submit-btn')
        .on('click', () => refreshSearchGrid({
            Alias,
            Class
        }));

    (window as any)['searchWidgetFilter'] = searchWidgetFilter;

    return wrapper.append([search, log]);
};

const searchWidgetFilter = (form: Formulario) => {
    form.grid.clearFilter();

    refreshSearchGrid({ form });
}

const refreshSearchGrid = ({ Class, Alias, form }: { Class?: string, Alias?: string, form?: Formulario }) => {
    const formInstance = form || FormManager.byname(Class, Alias);        
    const temp = formInstance.grid.state();

    formInstance.grid.refresh();
    formInstance.grid.state(temp);
};