import { ToolButton } from "../fra-control/types";
import { Card } from "./layout/card";
import { LayoutScopeConfig } from "./layout/layout";
import { TwAgendaConfig } from "./tw-agenda/tw-agenda";
import { TwButtonActiveConfig } from "./tw-button-active/tw-button-active";
import { TwCalendarioConfig } from "./tw-calendario/tw-calendario";
import { TwDatasetTreeViewConfig } from "./tw-dataset-tree-view/tw-dataset-tree-view";
import { TwIframeConfig } from "./tw-iframe/tw-iframe";

export interface WidgetsConfig {
    [position: string]: WidgetsConfigItem[];
}

export interface WidgetsConfigItem {
    Colunas: number;
    Itens: Widget[];
    Layout?: LayoutConfig;
    Color: string;
}

export interface LayoutConfig {
    Linhas?: LayoutConfig[];
    Colunas?: LayoutConfig[];
    Pages?: LayoutPageConfig[];
    SubLayout?: LayoutConfig;

    WidgetName?: string;

    TipoColuna?: number;
    Titulo?: string;
    TipoTamanho: number;
    Tamanho: number;
    TamanhoMinimo: number;
}

export interface LayoutPageConfig {
    Caption: string;
    Name: string;
    Coluna: LayoutConfig;
}

export type Widget = 
    | TwFieldListConfig
    | TwSimpleConfig 
    | TwMasterDetailConfig
    | TwImageConfig
    | TwParamlistConfig
    | TwButtonConfig
    | TwImageTextConfig
    | TwCardDisplayerConfig
    | TwImageValueDisplayerConfig
    | TwListaConfig
    | TwLabelValueConfig
    | TwHomeAreaTrabalhoConfig
    | TwCxGridConfig
    | TwMenuConfig
    | TwCardSeletorConfig
    | TwArquivosTempConfig
    | TwTreeViewConfig
    | TwMsgListaConfig
    | TwTextConfig
    | TwValueDisplayerConfig
    | TwMiniCardValueDisplayerConfig
    | TwAgendaConfig
    | TwCalendarioConfig
    | TwButtonActiveConfig
    | TwDatasetTreeViewConfig
    | TwButtonControlConfig
    | TwIframeConfig;

export const WidgetType = {
    isTwMasterDetail: (widget: any): widget is TwMasterDetailConfig => (widget as TwMasterDetailConfig).twMasterDetail,
    isTwFieldList: (widget: any): widget is TwFieldListConfig => (widget as TwFieldListConfig).twFieldList,
    isTwSimple: (widget: any): widget is TwSimpleConfig => (widget as TwSimpleConfig).twSimple,
    isTwParamList: (widget: any): widget is TwParamlistConfig => (widget as TwParamlistConfig).twParamList,
    isTwImage: (widget: any): widget is TwImageConfig => (widget as TwImageConfig).twImage,
    isTwButton: (widget: any): widget is TwButtonConfig => (widget as TwButtonConfig).twButton,
    isTwImageText: (widget: any): widget is TwImageTextConfig => (widget as TwImageTextConfig).twImageText,
    isTwCardDisplayer: (widget: any): widget is TwCardDisplayerConfig => (widget as TwCardDisplayerConfig).twCardDisplayer,
    isTwImageValueDisplayer: (widget: any): widget is TwImageValueDisplayerConfig => (widget as TwImageValueDisplayerConfig).twImageValueDisplayer,
    isTwLista: (widget: any): widget is TwListaConfig => (widget as TwListaConfig).twLista,
    isTwLabelValue: (widget: any): widget is TwLabelValueConfig=> (widget as TwLabelValueConfig).twLabelValue,
    isTwHomeAreaTrabalho: (widget: any): widget is TwHomeAreaTrabalhoConfig => (widget as TwHomeAreaTrabalhoConfig).twHomeAreaTrabalho,
    isTwCxGrid: (widget: any): widget is TwCxGridConfig => (widget as TwCxGridConfig).twcxGrid,
    isTwMenu: (widget: any): widget is TwMenuConfig => (widget as TwMenuConfig).twMenu,
    isTwCardSeletor: (widget: any): widget is TwCardSeletorConfig => (widget as TwCardSeletorConfig).twCardSeletor,
    isTwArquivosTemp: (widget: any): widget is TwArquivosTempConfig => (widget as TwArquivosTempConfig).twArquivosTemp,
    isTwTreeView: (widget: any): widget is TwTreeViewConfig => (widget as TwTreeViewConfig).twTreeView,
    isTwMsgLista: (widget: any): widget is TwMsgListaConfig => (widget as TwMsgListaConfig).twMsgLista,
    isTwText: (widget: any): widget is TwTextConfig => (widget as TwTextConfig).twText,
    isTwValueDisplayer: (widget: any): widget is TwValueDisplayerConfig => (widget as TwValueDisplayerConfig).twValueDisplayer,
    isTwMiniCardValueDisplayer: (widget: any): widget is TwMiniCardValueDisplayerConfig => (widget as TwMiniCardValueDisplayerConfig).twMiniCardValueDisplayer,
    isTwButtonControl: (widget: any): widget is TwButtonControlConfig => (widget as TwButtonControlConfig).twButtonControl,
    isTwImageButton: (widget: any): widget is any => widget.twImageButton,
    isTwAgenda: (widget: any): widget is TwAgendaConfig => (widget as TwAgendaConfig).twAgenda,
    isTwDatasetTreeView: (widget: any): widget is TwDatasetTreeViewConfig => (widget as TwDatasetTreeViewConfig).twDatasetTreeView,
    isTwCalendario: (widget: any): widget is TwCalendarioConfig => (widget as TwCalendarioConfig).twCalendario,
    isTwButtonActive: (widget: any): widget is TwButtonActiveConfig => (widget as TwButtonActiveConfig).twButtonActive,
    isTwIframe: (widget: any): widget is TwIframeConfig => (widget as TwIframeConfig).twIframe,
}

export interface WidgetPropsBase {
    card: Card;
    config: LayoutScopeConfig;
}

export interface WidgetConfigBase {
    Name: string;
    Visible: boolean;
    Align: string;
    Data?: any;
    Tipo?: number;
    Expressoes?: WidgetExpression; // Only some widgets
    CorValorNegativo?: string;
    CorValorPositivo?: string;
}

export interface WidgetExpression {
    DataSet?: string;
    ExpressaoValue?: string;
}

export interface FieldListBase extends WidgetConfigBase {
    FieldList: string[];

    /** property built at runtime on layout method */
    isMaster: boolean;
} 

export interface TwFieldListConfig extends FieldListBase {
    twFieldList: true;
}

export interface TwParamlistConfig extends FieldListBase {
    twParamList: true;
}

export interface TwSimpleConfig extends WidgetConfigBase {
    twSimple: true;
    Text: string;
    TextColor: string;
    Valor: string;
    ValorColor: string;
    colunas?: number; // Prop only exists when invoked on partial.
}

export interface TwMasterDetailConfig extends WidgetConfigBase {
    DetailClass: string;
    MasterClass: string;

    /** fields separated by ';' ex 'field;field2' */
    MasterFields: string;
    twMasterDetail: true;
}

export interface TwImageConfig extends WidgetConfigBase {
    twImage: true;

    /** Base64 */
    image: string;
}

export interface TwButtonConfig extends WidgetConfigBase {
    twButton: true;
    Action: string;
    BackGroudColor: string;
    Text: string;
    TextColor: string;
}

export interface TwImageTextConfig extends WidgetConfigBase {
    twImageText: true;

    /** Base64 */
    image: string;
    Text: string;
    TextColor: string;

    colunas?: number; // Prop only exists when invoked on partial.
}

export interface TwCardDisplayerConfig extends WidgetConfigBase {
    twCardDisplayer: true;
    Action: string;
    Text: string;
    Title: string;
    Valor: string;
    StyleClass: string;
}

export interface TwImageValueDisplayerConfig extends WidgetConfigBase {
    twImageValueDisplayer: true;
    SubTitulo: string;
    Text: string;
    TextColor: string;
    Valor: string;
    ValueColor: string;
    Action: string;
    
    /** Base64 */
    image: string;
    colunas?: number; // Prop only exists when invoked on partial.
}

export interface TwListaConfig extends WidgetConfigBase {
    twLista: true;
    CorTitulo: "green"
    Itens: Array<{
        Caption: string;
        Valor: string;
    }>;
    Text: "Test";
    colunas?: number; // Prop only exists when invoked on partial.
}

export interface TwLabelValueConfig extends WidgetConfigBase {
    twLabelValue: true;
    Title: {
        Align: 'alTop' | 'alLeft';
        Alignment: 'taLeftJustify' | 'taRight' | 'taCenter' | 'taLeft' | 'twaNone';
        Text: string;
        Font: TextFont;
    };
    Value: {
        Text: string;
        Font: TextFont;
    };
    Symbol: {
        Text: string;
        Font: TextFont;
    };
    ShowShadow: boolean;
    Icon: {
        Picture: string; // Base64 
        Action: ToolButton
    };
    colunas?: number; // Prop only exists when invoked on partial.
}

interface TextFont {
    Color: string;
    Name: string;
    Size: number;
    Style: {
        fsBold: boolean;
    };
}

export interface TwHomeAreaTrabalhoConfig extends WidgetConfigBase {
    twHomeAreaTrabalho: true
    Descricao: string;
    Imagem: string; // Base64
    LinkSaibaMais: string;
    NomeAreaTrabalho: string;
}

export interface TwCxGridConfig extends WidgetConfigBase {
    twcxGrid: true;
    Grid?: string | Object; // Widget from back
    grids?: any[]; // Grids from process result
    Text: string;
    colunas?: boolean; // if the container has "colunas" then grid will have the "col" class. Only existis when invoked on partial
}

export interface TwMenuConfig extends WidgetConfigBase {
    twMenu: true;
    Color: string; // Ex: "rgb(220,93,52)"
    Class?: string; // If hasn't dropdown
    Itens?: Array<TwMenuItem>; // If has dropdown
    Text: string;
}

export interface TwMenuItem { 
    Caption: string;
    Class?: string;
    disabled?: boolean;
    Hint?: string;
    Itens?: Array<TwMenuItem>;
} 

export interface TwCardSeletorConfig extends WidgetConfigBase {
    twCardSeletor: true;
    Itens: Array<{
        Action: ToolButton;
        Image: string; // Base64
        Titulo: string;
        Visible: boolean;
    }>;
}

export interface TwArquivosTempConfig extends Pick<WidgetConfigBase, 'Name' | 'Visible'> {
    twArquivosTemp: true;
    ArquivosTemp: Array<{
        Caption: string;
        Arquivo: string;
    }>;
    Align?: undefined;
}


interface TwTreeViewNode {
    icon?: string;
    state?: {
        opened: boolean;
    };
    text?: string;
    nodes?: Array<TwTreeViewNode>;
}

export interface TwTreeViewConfig extends WidgetConfigBase {
    twTreeView: true;
    nTreeView?: Array<TwTreeViewNode>;
    image?: string; // Base64
    Text?: string;
    colunas?: number; // Prop only exists when invoked on partial.
}

export interface TwMsgListaConfig extends Pick<WidgetConfigBase, 'Name' | 'Visible'> {
    twMsgLista: true;
    MsgLista: {
        Columns: Array<string>;
        Items: Array<Array<string>>;
        datatablesettings: {
            paging: boolean;
            ordering: boolean;
            info: boolean;
            searching: boolean;
        };
    };
    Align?: undefined;
}

export interface TwTextConfig extends WidgetConfigBase {
    twText: true;
    Classe?: string; // for action
    Action?: string; 
    Text: string;
    TextColor: string; // hex ex: "000000"
    colunas?: number; // Prop only exists when invoked on partial.
}

export interface TwValueDisplayerConfig extends WidgetConfigBase {
    twValueDisplayer: true;
    Estilo: string;
    Text: string;
    TextColor: string; // hex ex: "000000"
    Valor: string;
    colunas?: number; // Prop only exists when invoked on partial.
}

export interface TwMiniCardValueDisplayerConfig extends WidgetConfigBase {
    twMiniCardValueDisplayer: true
    action: string;
    marker: boolean;
    title: string;
    value: string;
}

export interface TwButtonControlConfig extends WidgetConfigBase {
    twButtonControl: true;
    Action: string | ToolButton;
    Text: "Adicionar";
    TypeControl: 'Main' | 'Primary' | 'Secondary';
    Width: 100
}