import { Widget } from "./types";

export abstract class WidgetBase<WidgetConfig extends Widget, State extends { widget: WidgetConfig}> {
    protected _element: JQuery<HTMLElement> = $();
    private _state: State;

    get element() {
        return this._element;
    }

    constructor(initialState: State) {
        this._state = initialState;

        this.setState(initialState);
    }

    public setState(newState: Partial<State>) {
        this._state = {
            ...this._state,
            ...newState,
        };

        const newElement = this._build();

        if (this._element.length) this._element.replaceWith(newElement);
        
        this._element = newElement;
    }

    public setValue(newValue: Partial<WidgetConfig>) {
        this._state.widget = {
            ...this._state.widget,
            ...newValue,
        };

        const newElement = this._build();

        if (this._element.length) this._element.replaceWith(newElement);
        
        this._element = newElement;
    }
    
    protected abstract _build(): JQuery<HTMLElement>;
    
    public getState() {
        return this._state;
    };

    public dispose() {
        this._element.remove();
    }
}