import dxDataGrid from "devextreme/ui/data_grid";
import FormManager from "../../../js/FormManager";
import { CustomStoreArray } from "../../../js/grid/CustomStoreArray";
import { Field, Formulario } from "../../../js/nFormulario";
import { PostWithForm } from "../../../js/nRequests";
import SessionManager from "../../../js/SessionManager";
import WizardManager from "../../../js/WizardManager";
import { DefinicaoObjetosConfig } from "../../types/page-config/page-config";
import { Timer } from '../../../js/Timer';
import { ExecuteButton } from "../toolbuttons/execute-button/execute-button";
import { renderCustomButton } from "../toolbuttons/custom-button/custom-button";
import { DropdownButton } from "../toolbuttons/dropdown-button/dropdown-button";
import { getCurrentPageOptions } from "../menu/map-config-from-request";

declare var KTApp: any;
declare var $EhModal: boolean;

interface PropsFromScope extends Pick<DefinicaoObjetosConfig, 'ExibirBotoesNavegacao' | 'Class' | 'WorkSpace'> {}

export interface CreateNavigationToolbarProps extends PropsFromScope{
    element: JQuery<HTMLElement>;
    DefinicaoObjetos: DefinicaoObjetosConfig;
}

export const createWizardNavigationToolbar = (props: CreateNavigationToolbarProps) => {
    const { ExibirBotoesNavegacao, element, Class, WorkSpace } = props;
    const toolbar = !ExibirBotoesNavegacao
        ? $('')
        :  NavigationToolbar(props);

    exposeGlobalHandlers({
        ExibirBotoesNavegacao,
        Class,
        WorkSpace,
    });

    element.replaceWith(toolbar); 
}

const NavigationToolbar = (props: CreateNavigationToolbarProps) => {
    const { DefinicaoObjetos } = props;

    const toolbar = $(
        `<div class="footer bg-white py-1 d-flex flex-lg-column justify-content-center">
            <div class="container-fluid d-flex flex-column flex-md-row align-items-center justify-content-between">
                <div class="nav nav-dark"></div>
                <div class="text-dark order-2"></div>
            </div>
        </div>`
    );

    const previousButtonAction = () => {
        GoToWizardItem(WizardManager.context.DetalheAnterior, 'useTimer');
    };
    const nextButtonAction =   () => {
        GotoWizardNext(WizardManager.context.DetalheProximo, 'useTimer');
    };

    const previousButton = $(/*html*/
            `<button 
                class="btn btn-sm btn-q-tertiary disabled mx-2"
                id="btnVoltar" 
                type="button" 
            >
                <span><i class="icon-md las la-undo"></i></span>
                <span class="h4">VOLTAR</span>
            </button>`
        )
        .on('click', previousButtonAction);

    const nextButton = $(/*html*/
            `<button
                class="btn btn-sm btn-q-primary mx-2"
                id="btnAvancar"
                type="button"
            >
                <span class="h4">AVANÇAR</span>
            </button>`
        )
        .on('click', nextButtonAction);

    // FormManager._shortCuts.bind({
    //     Class: DefinicaoObjetos.Class,
    //     Alias: DefinicaoObjetos.Alias,
    //     caption: 'AVANÇAR',
    //     handler: nextButtonAction
    // });

    
    // FormManager._shortCuts.bind({
    //     Class: DefinicaoObjetos.Class,
    //     Alias: DefinicaoObjetos.Alias,
    //     caption: 'VOLTAR',
    //     handler: previousButtonAction
    // });

    const canRender = (type: 'custom' | 'dropdown' | 'execute') => {
        const { ToolButtons, ProcessoPadrao, ExibirExecutar } = DefinicaoObjetos;

        switch(type) {
            case 'custom': return ToolButtons && ProcessoPadrao;
            case 'dropdown': return ToolButtons && ProcessoPadrao;
            case 'execute': return ToolButtons && ProcessoPadrao && ExibirExecutar;
        }
    };

    toolbar
        .find('.text-dark')
        .append([
            previousButton, 
            canRender('execute') ? ExecuteButton({ ...props, variation: 'secondary' }): $(),
            ...canRender('custom') ? (renderCustomButton(props).map(e => e ? e : $())) : [],
            nextButton,
            canRender('dropdown') ? DropdownButton(props) : $(),
        ]);

    return toolbar;
};


const exposeGlobalHandlers = (propsFromScope: PropsFromScope) => {
    //@ts-ignore
    window.GoToWizardItem = GoToWizardItem;

    //@ts-ignore
    NavigateWizardBounded = NavigateWizardBounded.bind(null, propsFromScope);

    //@ts-ignore
    window.GotoWizardNext = GotoWizardNext;
};

export const GotoWizardNext = (AItem: string, blockPageMode?: 'useTimer') => WizardManager.GoToWizardNext(AItem, blockPageMode);

const GoToWizardItem = (AItem?: string, blockPageMode?: 'useTimer') => {
    if (AItem) {
        const { DetalheAtivo, Params, DetalheAnterior } = WizardManager.context;
        const form = FormManager.byname(DetalheAtivo);

        WizardManager.context.Params = form?.putvaluesin(Params || {}) || Params;

        if (DetalheAnterior != AItem) form?.OnWizardNext.next();

        if (blockPageMode == 'useTimer') (new  Timer()).IniciarExecucao()
        else KTApp.blockPage(); 

        NavigateWizard({
            actionname: 'actShow',
            sender: AItem!,
            form: FormManager.byname(AItem),
        });
    }
};

interface NavigateWizardProps {
    actionname: string;
    sender: string
    element?: HTMLElement;
    form?: Formulario;
}

export var NavigateWizard = (props: NavigateWizardProps) => {
    //@ts-ignore 
    NavigateWizardBounded(props); // 1° parameter must come from NavigateWizardBounded.bind(null, propsFromScope);
}

let NavigateWizardBounded = (propsFromScope: PropsFromScope, props: NavigateWizardProps) => {
    const { actionname, sender, form } = props;

    if (!propsFromScope || !props) {
        throw new Error('Incorrectly bound function');
    }

    FormManager.podeEventoChangeFields = false;

    setFormWithSelecao({ form });
    
    if (WizardManager.context) {
        setWizardContextWithDataset();
        setFormWithWizardContext({ form });
    }

    const params = {
        sender,
        actionname: actionname,
        Class: propsFromScope.Class,
        pathName: getCurrentPageOptions().pagePathName,
        EhModal: $EhModal,
        FormularioAtual: sender,
        formparams: form?.getValues() || {},
        ...getIdRequest(),
        ...getWizardContext(),
        ...getDetailWorkspace(propsFromScope),
    }
    let post = new PostWithForm('Processo');

    try {
        post.addParam('Scope', JSON.stringify(params));
        post.submit();
    }
    catch (e) {
        console.log(e)
        console.log('params', params);
    }
}

const setFormWithSelecao = ({ form }: { form?: Formulario }) => {
    if (!SessionManager.selecao || !form) return;

    Object
        .entries(SessionManager.selecao as { [key: string]: string | number })
        .forEach(([key, value]) => {
            const field = getField(form, key);

            if (field && !field.visible) field.setvalue(value);
        });
};

const getField = (form: Formulario, field: string): Field => {
    const fieldName = field.toUpperCase();
    
    return form.fieldbyName(fieldName) || form.fieldbyName(`P${fieldName}`)!;
};

const setWizardContextWithDataset = () => {
    WizardManager.context?.Datasets?.forEach((dataset: { Nome: string, Valor: any}) => {
        const gridElement =  $(`#dataGridConteudo${dataset.Nome}`);

        if (gridElement.length) {
            dataset.Valor = getDataSourceArray(dxDataGrid.getInstance(gridElement) as dxDataGrid);
        }
    });
};

const getDataSourceArray = (component: dxDataGrid) => {
    const dataSource = component.option('dataSource');

    return (dataSource instanceof CustomStoreArray) ? dataSource._dataSourceArray : dataSource;
};

const setFormWithWizardContext = ({ form }: { form?: Formulario }) => {
    if (!form) return;

    Object
        .entries(WizardManager.context.Params as { [key: string]: string | null })
        .forEach(([key, value]) => {
            const field = getField(form, key);

            if (field && !field.isFile && !field.visible) {
                field.setvalue(value);
            }
        });
};

const getIdRequest = () => {
    if (!WizardManager.context) return {};

    return {
        IDRequest: WizardManager.context.Params.idrequest,
    };
};

const getWizardContext = () =>  {
    if (!WizardManager.context) return {};

    return {
        WizardContext: WizardManager.context,
    };
};

const getDetailWorkspace = ({ WorkSpace }: PropsFromScope) => {
    if (!WorkSpace) return {};
    
    return {
        Detalhe:  `${WizardManager.context.Class}_${WizardManager.context.DetalheHome}`, //Sempre no primeiro item
    };
};