import { criarDialogOk, FormatErrorMessage } from "../../../../js/nDialog";
import { Timer } from "../../../../js/Timer";
import { ToolButton } from "../../../components/fra-control/types";
import { OperacaoDataSetProps, ReportViewScopeConfig } from "../../relatorio-exibir/relatorio-exibir";
import { ReportTypeZen, SendHandlerProps } from "../envio-zen";
import { ReportProps } from "../envio-zen-report-view";
import { handleFeedbackModal } from "../feedback-modal";
import bootbox from 'bootbox';
import { verifyAxiosError } from "../../../../js/nRequests";
import { AxiosError } from "axios";

declare var axios: any;

interface SendQColaboreButtonProps {
    modal: JQuery<HTMLElement>;
    scope: ReportViewScopeConfig;
}

export const getSendQColaboreButton = ({ modal, scope }: SendQColaboreButtonProps) => {
    //@ts-ignore
    window['actionOpenSendReportQColabore'] = actionOpenSendReportQColabore.bind(null, modal, scope);

    return getButtonConfig(scope);
};

const getButtonConfig = (scope: ReportViewScopeConfig): ToolButton => ({
    Caption: 'Enviar QColabore',
    Classe: scope.DefinicaoObjetos.Class,
    CleanCaption: 'Enviar Q-Colabore',
    Custom: true,
    ExibirToolBar: true,
    Hint: 'Enviar Q-Colabore',
    Name: 'actionOpenSendReportQColabore',
    NameEvent: 'actionOpenSendReportQColabore',
    Opcoes: {
        ReportTypeZen: 'qcolabore' as ReportTypeZen,
    },
    TipoAcao: 'TnActOperacaoDataset',
    icon: '',
    tbsButton: true,
});

const actionOpenSendReportQColabore = (modal: JQuery<HTMLElement>, scope: ReportViewScopeConfig, { _AOpcoes }: OperacaoDataSetProps) => {
    const { DefinicaoObjetos, fileNameNhz, Params } = scope;
    const reportName = fileNameNhz;
    const iframeEl = modal.find('iframe');

    if (!hasSplitReportParams(Params!)) {
        bootbox.dialog({
            title: 'Documento Não Enviado',
            message: `<h1>Atenção</h1>
                    <div class="mb-4"> Não foi possível enviar o documento</div>
                    <div>Para publicação de documentos no Q-Colabore é necessário marcar as opções de separar, na tela de geração do relatório, os checkbox: EMPRESA - FILIAL - CONTRATO DE EMPREGADO</div>`,
            buttons: {
                ok: {
                    label: 'Ok',
                    className: 'btn-q-primary',
                }
            },
        });

        return;
    }

    if (!reportName) throw new Error('File name is null');

    const params: ReportProps & { ReportType: ReportTypeZen } = {
        Params: Params!,
        ClassName: DefinicaoObjetos.Class,
        ReportName: reportName,
        ReportType: _AOpcoes.ReportTypeZen,
    };

    const modalParams = `BuscarDadosEnvioZen?Scope=${encodeURIComponent(JSON.stringify(params))}`

    iframeEl.contents().find('body').html('');
    iframeEl.attr('src', modalParams);

    // @ts-ignore;
    modal.modal();
};

const hasSplitReportParams = (Params: { [key: string]: string }) => {
    const splitParams = Params['pSepararArquivosPublicacao'.toUpperCase()];
    const codEmpresa = 'PCODIGOEMPRESA';
    const codFuncContr = 'PCODIGOFUNCCONTR';
    const codEstab = 'PCODIGOESTAB';

    return splitParams && splitParams.includes(codEmpresa) && splitParams.includes(codFuncContr) && splitParams.includes(codEstab);
};

interface ResponseSuccess {
    result: 'Sucesso';
}

interface ResponseError {
    result: 'Falha';
    grids: any[];
    fields: {
        [key: string]: any;
    };
}

type ResponseData = ResponseSuccess | ResponseError;

export const sendQColaboreHandler = async ({ reportProps, formValues, gridValues, modal }: SendHandlerProps) => {
    const { ClassName, Params, ReportName } = reportProps;
    const timer = new Timer();

    try {
        timer.IniciarExecucao();

        const response = await axios.post('nHttpServicos/ExecutarEnvioZen', {
            p: {
                ClassName,
                ReportName,
                Data: {
                    ...formValues,
                    DataSet: {
                        NomeDataset: 'nClientDataSet',
                        Registros: gridValues,
                    },
                },
                Params,
            }
        });

        timer.FinalizarExecucao();

        const { data }: { data: ResponseData } = response.data;

        console.log('responseData', data);

        switch(data.result) {
            case 'Sucesso':
                //@ts-ignore
                modal.modal('hide');

                criarDialogOk('Documento(s) enviado(s) com sucesso', 'SUCESSO');
                
                break;
            case 'Falha':
                //@ts-ignore
                modal.modal('hide');

                handleFeedbackModal({
                    reportProps,
                    formValues,
                    gridsDocument: data.fields.ListaFeedBackEnvio,
                    dataDestinatarios: gridValues,
                });

                break;
            default:
                FormatErrorMessage('Não foi possível enviar o documento');
        }
    }
    catch (e) {
        timer.FinalizarExecucao();

        console.error(e);

        if (verifyAxiosError(e as AxiosError)) return;
        
        FormatErrorMessage('Não foi possível enviar o documento');
    }
};