import dxDataGrid from 'devextreme/ui/data_grid';
import { FormatErrorMessage } from '../../../js/nDialog';
import { CarregarDataCadastro } from '../../../js/nWebUtil';
import { TwCxGrid } from "../../components/widgets/tw-cx-grid/tw-cx-grid";
import { buildForm } from "../../components/widgets/tw-field-list/tw-field-list";
import { DefinicaoObjetosConfig } from "../../types/page-config/page-config";
import { ReportProps } from './envio-zen-report-view';
import { getFields, GetFieldsProps } from './get-fields';

declare var createAccordion: any;
declare var addAccordionClassField: any;

export interface EnvioZenProps {
    ClassName: string;
    DefinicaoObjetos: {
        grids: Array<{ [key: string]: any }>;
        fields: {
            [key: string]: any;
        };
    };
    Params: {
        [key: string]: string;
    };
    ReportName: string;
    ReportType: ReportTypeZen;
}

export type ReportTypeZen = 'edoc' | 'qcolabore';

interface LoadDataProps {
    ClassName: string;
    fields: {
        [key: string]: any;
    };
}

export interface SendHandlerProps {
    reportProps: ReportProps;
    formValues: {
        [key: string]: string | null;
    } | {
        ListaFeedBackEnvio?: {
            NomeDataset: string,
            Registros: Array<{
                [key: string]: string;
            }>;
        };
    };
    gridValues: Array<{
        [key: string]: string;
    }>;
    modal: JQuery<HTMLElement>;
    url?: string;
}

export const ConfigureEnvioZen = (props: EnvioZenProps) => {
    try {
        const { ClassName, Params, DefinicaoObjetos, ReportType } = props;
        const contentWrapper = $('#content-wrapper');
        const gridContainer = $(`<div id="formFieldGridDestinatario${ClassName}" class="input-block form-group kt-form__group">`);
        const definicaoObjetosConfig = getDefinicaoObjetos(props);
        const decriptionEl = getDescription({ ReportType, DefinicaoObjetos });

        console.log('props', props);
        const config: GetFieldsProps = {
            ClassName,
            ReportType,
            Accordion: 'Configurações de Envio',
            Definicao: DefinicaoObjetos.fields.Definicao,
        };

        const fields = getFields(config);

        const form = buildForm({
            config: {
                DefinicaoObjetos: definicaoObjetosConfig,
            },
            fields: fields,
        });

        console.log('fields', fields);

        contentWrapper
            .append(decriptionEl)
            .append(form)
            .find('.form-fields')
            .addClass(`anchor-accordion${ClassName}`)
            .append(gridContainer);

        fields
            .filter(({ Accordion }) => Accordion)
            .forEach((field) => addAccordionClassField(`#formField${field.Name}${ClassName}`, field.Accordion));

        addAccordionClassField(`#formFieldGridDestinatario${ClassName}`, 'Configurar Destinatários');

        createAccordion(
            [
                ...fields
                    .filter(({ Accordion }) => Accordion)
                    .map((field) => {
                        return {
                            divider: field.Accordion,
                            required: field.Required,
                            formClass: ClassName,
                        }
                    }),
                {
                    divider: 'Configurar Destinatários',
                    required: '',
                    formClass: ClassName,
                }
            ],
            false,
        );

        gridContainer.replaceWith(TwCxGrid({
            Params,
            widget: ({
                twcxGrid: true,
                Grid: {
                    ...DefinicaoObjetos.grids[0],
                    editing: {
                        ...DefinicaoObjetos.grids[0].editing!,
                        allowUpdating: true,
                        mode: "batch",
                        selectTextOnEditStart: true,
                        startEditAction: "click",
                    },
                },
                Name: 'GridDestinatario',
            } as any),
            DefinicaoObjetos: definicaoObjetosConfig,
        }));

        const gridElement = contentWrapper.find(`[id*="dataGridConteudo"]`);

        gridElement.closest('.collapse').on('shown.bs.collapse', () => {
            let instance: dxDataGrid = (dxDataGrid.getInstance(gridElement) as dxDataGrid);

            instance.repaint();
            instance.refresh();
        });

        loadData({
            ClassName,
            fields: DefinicaoObjetos.fields,
        });
    }
    catch (e) {
        console.error(e);

        FormatErrorMessage('Carregamento incompleto de informações da página. Tente novamente');
    }
};

const loadData = ({ ClassName, fields }: LoadDataProps) => {
    $(() => {
        CarregarDataCadastro(
            Object
                .entries(fields)
                .reduce((fieldData, [key, value]) => ({
                    ...fieldData,
                    [key.toUpperCase()]: value,
                }), { Class: ClassName })
        );
    });
};

const getDefinicaoObjetos = ({ ClassName, ReportType }: EnvioZenProps): DefinicaoObjetosConfig => ({
    BodyCols: 12,
    CadCons: "",
    Caption: ReportType == 'edoc' ? "Envio E-Doc" : "Envio Q-Colabore",
    Class: ClassName,
    ClassForm: "TnFrmReport",
    ClassParent: "TComponent.TDataModule.TnDM.TnDMPower",
    ConsultaInativo: false,
    Detalhes: [],
    Fields: [],
    IDRequest: 0,
    JavaScript: "",
    MultiColuna: false,
    Name: "",
    ProcessoPadrao: true,
    TipoObjeto: "ntoRelatorio",
    ToolButtons: [],
    TopicoAjuda: ClassName,
    Widgets: false,
    ChavePrimaria: [],
    configextra: null,
});

const getDescription = ({ ReportType, DefinicaoObjetos }: { ReportType: ReportTypeZen } & Pick<EnvioZenProps, 'DefinicaoObjetos'>) => $(`
    <div class="my-4 p-2">
        <h2>Deseja enviar esse Documento para o ${ReportType == 'edoc' ? 'e-Doc' : 'Q-Colabore'}?</h2>
        <span class="d-block">${DefinicaoObjetos?.fields?.Titulo || ''}</span>
    </div>
`);