import { Subject } from "rxjs";
import { filter } from "rxjs/operators";

interface BroadcastMessage {
    type: BroadcastMessageType;
    data: SaveDataType;
}

type BroadcastMessageType = 'save'; 

export interface SaveDataType {
    Class: string;
    Alias?: string;
    chaveprimaria: Array<string>;
    data: {
        [key: string]: any;
    }
}

class BroadcastManager {
    private _channel = typeof BroadcastChannel  != 'undefined' ? new BroadcastChannel('test') : ({} as BroadcastChannel); // Empty when running test on node
    private _subscriber = new Subject<BroadcastMessage>();

    constructor() {
        this._channel.onmessage = (event: MessageEvent) => {
            console.log('onmessage', event);

            this._subscriber.next(event.data);
        }
        this._channel.onmessageerror = (event: MessageEvent) => {
            console.error('onmessageerror', event);
        }
    }

    postMessage(message: BroadcastMessage) {
        this._channel.postMessage(message);
    }

    subscription(type?: BroadcastMessageType) {
        return this._subscriber.pipe(filter((message) => (type ? (message.type == type) : true)));
    }
}

export default new BroadcastManager();