import { CarregarCheckTreeView } from "../../../js/treeview/treeview";
import WizardManager from "../../../js/WizardManager";
import { createFormRepeater, FormRepeaterConfig } from "../../components/form-repeater/create-form-repeater";
import { createNtcGrid, CreateNtcGridProps,  } from "../../components/fra-control/ntc-grid/create-ntc-grid";
import { createNtcLinkAnnotation, CreateNtcLinkAnnotationProps } from "../../components/fra-control/ntc-link-annotation/create-ntc-link-annotation";
import { createNtcMemoAnnotation, CreateNtcMemoAnnotationProps } from "../../components/fra-control/ntc-memo-annotation/create-ntc-memo-annotation";
import qNotifier from "../../components/notifier/q-notifier";
import { CustomButton, CustomButtonProps, renderCustomButton } from "../../components/toolbuttons/custom-button/custom-button";
import { createSplitReportParamsCheckbox, createSplitReportParamsDropdown } from "../../components/toolbuttons/split-report-param/split-report-param";
import { createToolButtons } from "../../components/toolbuttons/toolbuttons";
import { InitLayoutProps, Layout } from "../../components/widgets/layout/layout";
import { createTwAgenda, CreateTwAgendaProps } from "../../components/widgets/tw-agenda/create-tw-agenda";
import { createTwArquivosTemp, CreateTwArquivosTempProps } from "../../components/widgets/tw-arquivos-temp/create-tw-arquivos-temp";
import { createTwButtonControl, CreateTwButtonControlProps } from "../../components/widgets/tw-button-control/create-tw-button-control";
import { createTwButton, CreateTwButtonProps } from "../../components/widgets/tw-button/create-tw-button";
import { createTwCardDisplayer, CreateTwCardDisplayerProps } from "../../components/widgets/tw-card-displayer/create-tw-card-displayer";
import { createTwCardSeletor, CreateTwCardSeletorProps } from "../../components/widgets/tw-card-seletor/create-tw-card-seletor";
import { createTwCxGrid, CreateTwCxGridProps } from "../../components/widgets/tw-cx-grid/create-tw-cx-grid";
import { createTwDatasetTreeView, CreateTwDatasetTreeViewProps } from "../../components/widgets/tw-dataset-tree-view/create-tw-dataset-tree-view";
import { createTwHomeAreaTrabalho, CreateTwHomeAreaTrabalhoProps } from "../../components/widgets/tw-home-area-trabalho/create-tw-home-area-trabalho";
import { createTwImageButton } from "../../components/widgets/tw-image-button/create-tw-image-button";
import { createTwImageText, CreateTwImageTextProps } from "../../components/widgets/tw-image-text/create-tw-image-text";
import { createTwImageValueDisplayer, CreateTwImageValueDisplayerProps } from "../../components/widgets/tw-image-value-displayer/create-tw-image-value-displayer";
import { createTwImage, CreateTwImageProps } from "../../components/widgets/tw-image/create-tw-image";
import { createTwLabelValue, CreateTwLabelValueProps } from "../../components/widgets/tw-label-value/create-tw-label-value";
import { createTwLista, CreateTwListaProps } from "../../components/widgets/tw-lista/create-tw-lista";
import { createTwMenu, CreateTwMenuProps } from "../../components/widgets/tw-menu/create-tw-menu";
import { createTwMiniCardValueDisplayer, CreateTwMiniCardValueDisplayerProps } from "../../components/widgets/tw-mini-card-value-displayer/create-tw-mini-card-value-displayer";
import { createTwMsgLista, CreateTwMsgListaProps } from "../../components/widgets/tw-msg-lista/create-tw-msg-lista";
import { createTwSearch, CreateTwSearchProps } from "../../components/widgets/tw-search/create-tw-search";
import { createTwSimple, CreateTwSimpleProps } from "../../components/widgets/tw-simple/create-tw-simple";
import { createTwText, CreateTwTextProps } from "../../components/widgets/tw-text/create-tw-text";
import { createTwTreeView, CreateTwTreeViewProps } from "../../components/widgets/tw-tree-view/create-tw-tree-view";
import { createTwValueDisplayer, CreateTwValueDisplayerProps } from "../../components/widgets/tw-value-displayer/create-tw-value-displayer";
import { CreateNavigationToolbarProps, createWizardNavigationToolbar } from "../../components/wizard-navigation/wizard-navigation";
import { openPdf, OpenPdfProps } from "../../pages/relatorio-exibir/open-pdf";
import { DefinicaoObjetosConfig, ScopeConfig } from "../../types/page-config/page-config";
import { addAccordionClassField, createAccordion } from '../../../js/nExpandables';

interface Task<T = TaskPayload> {
    type: TaskType;
    payload: T;
}

type TaskType = 
    | 'create-accordion'
    | 'create-custom-toolbutton'
    | 'create-form-repeater'
    | 'create-split-report-params-checkbox'
    | 'create-split-report-params-dropdown'
    | 'create-toolbar'
    | 'create-wizard-navigation-toolbar'
    | 'error'
    | 'layout-initlayout'
    | 'layout-update'
    | 'load-check-treeview'
    | 'open-pdf'
    | 'set-accordion-field-class'
    | 'set-wizard-context'
    | CreateFraControl
    | CreateWizardTaskType;

type CreateFraControl =
    | 'create-ntc-link-annotation'
    | 'create-ntc-memo-annotation'
    | 'create-ntc-grid';

type CreateWizardTaskType = 
    | 'create-tw-arquivos-temp'
    | 'create-tw-button'
    | 'create-tw-button-control'
    | 'create-tw-card-displayer'
    | 'create-tw-card-seletor'
    | 'create-tw-cx-grid'
    | 'create-tw-home-area-trabalho'
    | 'create-tw-image'
    | 'create-tw-image-button'
    | 'create-tw-image-text'
    | 'create-tw-image-value-displayer'
    | 'create-tw-label-value'
    | 'create-tw-lista'
    | 'create-tw-menu'
    | 'create-tw-msg-lista'
    | 'create-tw-tree-view'
    | 'create-tw-text'
    | 'create-tw-value-displayer'
    | 'create-tw-mini-card-value-displayer'
    | 'create-tw-simple'
    | 'create-tw-agenda'
    | 'create-tw-dataset-tree-view'
    | 'create-tw-search';

type TaskPayload =
    | CreateAccordionProps
    | CreateSplitReportParamsProps
    | CreateNavigationToolbarProps
    | CustomButtonConfig
    | FraControlPayload
    | InitLayoutProps
    | LayoutUpdate
    | LoadCheckTreviewProps
    | OpenPdfProps
    | QNotifierError
    | SetAccordionFieldClassProps
    | SetWizardContextProps
    | WidgetTaskPayload;

type FraControlPayload =
    | CreateNtcLinkAnnotationProps
    | CreateNtcMemoAnnotationProps
    | CreateNtcGridProps;

type WidgetTaskPayload =
    | CreateTwArquivosTempProps
    | CreateTwButtonProps
    | CreateTwButtonControlProps
    | CreateTwCardDisplayerProps
    | CreateTwCardSeletorProps
    | CreateTwCxGridProps
    | CreateTwHomeAreaTrabalhoProps
    | CreateTwImageProps
    | CreateTwImageTextProps
    | CreateTwImageValueDisplayerProps
    | CreateTwLabelValueProps
    | CreateTwListaProps
    | CreateTwMenuProps
    | CreateTwMsgListaProps
    | CreateTwTreeViewProps
    | CreateTwTextProps
    | CreateTwValueDisplayerProps
    | CreateTwMiniCardValueDisplayerProps
    | CreateTwSimpleProps
    | CreateTwAgendaProps
    | CreateTwDatasetTreeViewProps
    | CreateTwSearchProps;

interface CreateAccordionProps {
    fields: {
        divider: string;
        required: string;
        formClass: string;
    };
    isMultiColumn: boolean;
}

interface CreateSplitReportParamsProps {
    element: JQuery<HTMLScriptElement> | HTMLScriptElement;
    DefinicaoObjetos: DefinicaoObjetosConfig
}

interface CustomButtonConfig extends CustomButtonProps {
    element: JQuery<HTMLScriptElement>;
}

interface LayoutUpdate {
    scope: ScopeConfig;
}

interface LoadCheckTreviewProps {
    TreeviewName: string;
    element?: JQuery<HTMLElement>;
    _ANome: string;
    selectionMode: 'multiple' | 'single';
}

interface QNotifierError {
    error : string;
}

interface SetAccordionFieldClassProps {
    elementId: string;
    accordion: string;
}

interface SetWizardContextProps {
    [key: string]: any
}

export const ScriptTaskExecutor = () => {
    const tasks: Array<Task> = (window as any).scriptTasksArray?.reverse();

    if (!tasks) return;

    while(tasks.length) {
        runTask(tasks.pop());
    }

    $('script[data-remove-script="true"]').remove()
};

const runTask = (task?: Task) => {
    switch(task?.type) {
        case 'create-accordion': return runScriptCreateAccordion(task as Task<CreateAccordionProps>);
        case 'create-custom-toolbutton': return createCustomToolbutton(task as Task<CustomButtonConfig>);
        case 'create-form-repeater': return runScriptCreateFormRepeater(task as Task<FormRepeaterConfig>);
        case 'create-ntc-grid': return createNtcGrid(task.payload as CreateNtcGridProps);
        case 'create-ntc-link-annotation': return createNtcLinkAnnotation(task.payload as CreateNtcLinkAnnotationProps);
        case 'create-ntc-memo-annotation': return createNtcMemoAnnotation(task.payload as CreateNtcMemoAnnotationProps);
        case 'create-split-report-params-checkbox': return runScriptSplitReportParamsCheckbox(task as Task<CreateSplitReportParamsProps>);
        case 'create-split-report-params-dropdown': return runScriptSplitReportParamsDropdown(task as Task<CreateSplitReportParamsProps>);
        case 'create-tw-arquivos-temp': return createTwArquivosTemp(task.payload as CreateTwArquivosTempProps);
        case 'create-tw-button': return createTwButton(task.payload as CreateTwButtonProps);
        case 'create-tw-button-control': return createTwButtonControl(task.payload as CreateTwButtonControlProps);
        case 'create-tw-card-displayer': return createTwCardDisplayer(task.payload as CreateTwCardDisplayerProps);
        case 'create-tw-card-seletor': return createTwCardSeletor(task.payload as CreateTwCardSeletorProps);
        case 'create-tw-cx-grid': return createTwCxGrid(task.payload as CreateTwCxGridProps);
        case 'create-tw-home-area-trabalho': return createTwHomeAreaTrabalho(task.payload as CreateTwHomeAreaTrabalhoProps);
        case 'create-tw-image': return createTwImage(task.payload as CreateTwImageProps);
        case 'create-tw-image-button': return createTwImageButton(task.payload as any);
        case 'create-tw-image-text': return createTwImageText(task.payload as CreateTwImageTextProps);
        case 'create-tw-image-value-displayer': return createTwImageValueDisplayer(task.payload as CreateTwImageValueDisplayerProps);
        case 'create-tw-label-value': return createTwLabelValue(task.payload as CreateTwLabelValueProps);
        case 'create-tw-lista': return createTwLista(task.payload as CreateTwListaProps);
        case 'create-tw-menu': return createTwMenu(task.payload as CreateTwMenuProps);
        case 'create-tw-msg-lista': return createTwMsgLista(task.payload as CreateTwMsgListaProps);
        case 'create-tw-tree-view': return createTwTreeView(task.payload as CreateTwTreeViewProps);
        case 'create-tw-text': return createTwText(task.payload as CreateTwTextProps);
        case 'create-tw-value-displayer': return createTwValueDisplayer(task.payload as CreateTwValueDisplayerProps);
        case 'create-tw-mini-card-value-displayer': return createTwMiniCardValueDisplayer(task.payload as CreateTwMiniCardValueDisplayerProps);
        case 'create-tw-simple': return createTwSimple(task.payload as CreateTwSimpleProps);
        case 'create-tw-agenda': return createTwAgenda(task.payload as CreateTwAgendaProps);
        case 'create-tw-dataset-tree-view': return createTwDatasetTreeView(task.payload as CreateTwDatasetTreeViewProps);
        case 'create-tw-search': return createTwSearch(task.payload as CreateTwSearchProps);
        case 'create-toolbar': return runScriptCreateToolbar(task);
        case 'create-wizard-navigation-toolbar': return runScriptCreateWizardNavigationToolbar(task as Task<CreateNavigationToolbarProps>);
        case 'error': return runScriptError(task as Task<QNotifierError>);
        case 'layout-initlayout': return runScriptInitLayout(task as Task<InitLayoutProps>);
        case 'layout-update': return runScriptLayoutUptade(task as Task<LayoutUpdate>);
        case 'load-check-treeview': return runScriptLoadCheckTreeview(task as Task<LoadCheckTreviewProps>);
        case 'open-pdf': return runScriptOpenPdf(task as Task<OpenPdfProps>);
        case 'set-accordion-field-class': return runScriptSetAccordionFieldClass(task as Task<SetAccordionFieldClassProps>)
        case 'set-wizard-context': return runScriptSetWizardContext(task as Task<SetWizardContextProps>)
    }
};

const createCustomToolbutton = ({ payload: { element, ...props }}: Task<CustomButtonConfig>) => {
    element.replaceWith(CustomButton(props));
};

const runScriptCreateAccordion = ({ payload }: Task<CreateAccordionProps>) => {
    createAccordion(payload.fields, payload.isMultiColumn);
}

const runScriptCreateFormRepeater = ({ payload }: Task<FormRepeaterConfig>) => {
    createFormRepeater(payload)
};

const runScriptSplitReportParamsCheckbox = ({ payload }: Task<CreateSplitReportParamsProps>) => {
    createSplitReportParamsCheckbox({
        el: payload.element,
        DefinicaoObjetos: payload.DefinicaoObjetos,
    });
};

const runScriptSplitReportParamsDropdown = ({ payload }: Task<CreateSplitReportParamsProps>) => {
    createSplitReportParamsDropdown({
        el: payload.element,
        DefinicaoObjetos: payload.DefinicaoObjetos,
    });
};

const runScriptCreateWizardNavigationToolbar = ({ payload }: Task<CreateNavigationToolbarProps>) => {
    createWizardNavigationToolbar(payload);
};

const runScriptError = ({ payload }: Task<QNotifierError>) => { 
    $(() => qNotifier.error(payload.error));
};

const runScriptOpenPdf = ({ payload }: Task<OpenPdfProps>) => {
    openPdf(payload);
};

const runScriptInitLayout = ({ payload: { element, scope, options } }: Task<InitLayoutProps>) => {
    Layout.initLayout({
        element, 
        scope,
        options
    });
};

const runScriptLayoutUptade = ({ payload }: Task<LayoutUpdate>) => {
    Layout.update(payload.scope);
};

const runScriptLoadCheckTreeview = ({ payload }: Task<LoadCheckTreviewProps>) => {
    $(() => {
        CarregarCheckTreeView(payload as any)
    })
}

const runScriptCreateToolbar = ({ payload: { element, ...scope } }: Task<any>) => {
    $(element).parent().append(createToolButtons(scope as any));
}

const runScriptSetAccordionFieldClass = ({ payload }: Task<SetAccordionFieldClassProps>) => {
    addAccordionClassField(payload.elementId, payload.accordion);
}

const runScriptSetWizardContext = ({ payload }: Task<SetWizardContextProps>) => {
    WizardManager.setContext(payload);
}
