import path from 'path';
import WebpackNotifierPlugin from 'webpack-notifier';
import TerserPlugin from 'terser-webpack-plugin';
import dotenv from 'dotenv';
import { BundleAnalyzerPlugin } from 'webpack-bundle-analyzer';
import { Configuration } from "webpack";

dotenv.config({path: 'development.env'});

export default {
	mode: 'development',
	devtool: 'source-map',
	optimization: {
		minimizer: [
			new TerserPlugin({
				parallel: true
			})
		]
	},
	plugins: [
		new WebpackNotifierPlugin({
			title: 'Cloud webpack',
			contentImage: path.join(__dirname, 'images/logo_questor_q-g.png'),
			excludeWarnings: true,
		}),
		new BundleAnalyzerPlugin({
			analyzerMode: process.env.BUNDLE_ANALYZER_MODE as  "disabled" | "static",
			reportFilename: process.env.BUNDLE_ANALYZER_RESULT_NAME + ".html"
		})
	]
} as Configuration;