import FormManager from "../../../../../js/FormManager";
import dxDataGrid from 'devextreme/ui/data_grid';
import { criarDialogSimNao, FormatErrorMessage } from "../../../../../js/nDialog";
import { Formulario } from "../../../../../js/nFormulario";
import { AbrirCadastroModal } from "../../../../../js/nGrid";
import SessionManager from "../../../../../js/SessionManager";
import { CadastroExecuteActionProps } from "../toolbutton-action";

declare var qNotifier: any;

declare var AJaxCadastroExecuteMethodGetProperty: (
    _AClass: string,
    _AMethod: string,
    _AValues: string,
    _ANamePropertys: string,
    _AFuncaoSucesso: (response: any) => void,
    _AFuncaoErro?: any,
) => void;


export const ActContabilizar = ({ _AClasse, _AAlias }: CadastroExecuteActionProps) => {
    const form: Formulario = FormManager.byname(_AClasse, _AAlias);
    const paramString = JSON.stringify(form.getValues());

    AJaxCadastroExecuteMethodGetProperty(
        _AClasse,
        'ContabilizarCloud',
        paramString,
        '',
        (response) => openModal({
            response,
            Class: _AClasse,
            Alias: _AAlias || '',
        }),
    );
};

const getClassContabilizacao = () => `T${SessionManager.moduloAtual.replace(/^npm/, '')}DMContabilizacaoTemp`;

interface Response {
    data: {
        result: {
            Sequencial?: number;
            InformarLcto?: boolean;
            PossuiDiferenca?: boolean;
        };
    };
}

interface OpenModalProps extends Config {
    response: Response;
}

const openModal = ({ response, Alias, Class }: OpenModalProps) => {
    const { data: { result: { InformarLcto, Sequencial } } } = response;

    if (InformarLcto) {
        configureEventsOnModal({ Class, Alias, Sequencial });

        AbrirCadastroModal({
            _AClasse: getClassContabilizacao(),
            AGrid: null,
            ANovo: false,
            AClasseForm: null,
            AChaveMestre: ['SEQUENCIAL', 'CODIGOEMPRESA', 'CHAVEORIGEM', 'SEQ'],
            AFiltros: {
                SEQUENCIAL: Sequencial,
                INFORMAR: 1,
            },
        } as any);
    }
    else {
        qNotifier.info('Sem lançamentos a contabilizar');
    }
};

interface ModalWindow extends Window {
    FormManager: typeof FormManager;
}

interface FormManagerItems extends Array<Formulario> {
    firstUsable: () => Formulario | null;
}

interface ConfigureEventsProps extends Config {
    Sequencial?: number;
}

const configureEventsOnModal = ({ Alias, Class, Sequencial }: ConfigureEventsProps) => {
    const modalElement = $("#detalheModal");
    const iframeElement = $<HTMLIFrameElement>("#framemodal", modalElement);
    const modalControl = {
        saved: false,
    };

    modalElement.on("hide.bs.modal", getOnHideBsModalEvent({ modalElement, Alias, Class }, modalControl));

    iframeElement.on("load", getOnLoadIframeEvent({ iframeElement, modalElement, Alias, Class, Sequencial }, modalControl));
};

interface ModalControl {
    saved: boolean;
}

interface Config {
    Alias: string;
    Class: string;
}

interface OnLoadIframeEventProps extends Config {
    iframeElement: JQuery<HTMLIFrameElement>;
    modalElement: JQuery<HTMLElement>;
    Sequencial?: number;
}

interface KeyVal { 
    [key: string]: any ;
}

const getOnLoadIframeEvent = ({ iframeElement, modalElement, Sequencial }: OnLoadIframeEventProps, modalControl: any) => {
    const event = async function () {
        iframeElement.off('load', event as any);

        const contentWindow = iframeElement.get(0)!.contentWindow as ModalWindow;

        if (!contentWindow?.FormManager) return;
        
        modalControl.saved = false;

        let sequences = await getSequences(Sequencial);

        contentWindow.FormManager.runAfterCreateAllForms(function (FormManagerModal: typeof FormManager, items: FormManagerItems) {
            const form = items.firstUsable();

            if (form) {
                const isObjectEquals = (baseObj: KeyVal, toCompareObj: KeyVal) => Object.entries(baseObj).reduce<boolean>((equals, [key, value]) => (equals && value == toCompareObj[key]), true);
                const pKeys = form.chaveprimaria.reduce((keys: KeyVal, key: string) => ({
                    ...keys,
                    [key]: form.getvalue(key),
                }), {});

                sequences = sequences.filter((seq) => !isObjectEquals(pKeys, seq));

                form.AfterSave.subscribe(function () {
                    modalControl.saved = true;
                    
                    const nextSequence = sequences.pop();

                    console.log('nexteSequence', nextSequence);
                    
                    if (nextSequence) {
                        const _AFiltros = form.chaveprimaria.reduce((filter: { [key: string]: any }, key: string) => ({
                            ...filter,
                            [key]: nextSequence[key],
                        }), {});

                        FormManagerModal.loadCadastro({
                            _ACadastro: getClassContabilizacao(),
                            _AAlias: '',
                            _AOrigem: undefined,
                            _AFiltros,
                        });
                    }
                    else {
                        (modalElement as any).modal('hide');
                    }
                });

                form.setActionEnabled('actDataSetNextExecute', false);
                form.setActionEnabled('actDataSetPriorExecute', false);
            }
        });
    };

    return event;
};

interface ResponseDadosConsulta2 {
    data: {
        items: Array<{ [key: string]: string }>;
        totalcount: number;
    };
}

const getSequences = async (SEQUENCIAL?: number) => {
    try {
        const filter = SEQUENCIAL !== undefined 
            ? JSON.stringify([['SEQUENCIAL', '=', SEQUENCIAL], 'and', ['INFORMAR', '=', 1]])
            : '';

        const { data }: ResponseDadosConsulta2 = await $.getJSON('nHttpServicos/DadosConsulta2', {
            _AClass: getClassContabilizacao(),
            _AAlias: '',
            filter,
            skip: 0,
            take: 50,
            params: '{}',
        });

        return data.items;
    }
    catch (e) {
        FormatErrorMessage('Falha ao carregar dados para a contabilização');

        return [];
    }
};

interface onHideBsModalEventProps extends Config {
    modalElement: JQuery<HTMLElement>;
}

const getOnHideBsModalEvent = ({ Alias, Class, modalElement}: onHideBsModalEventProps, modalControl: any) => {
    const event = function () {
        modalElement.off('hide.bs.modal', event as any);

        const iframe = $('#framemodal', modalElement).get(0) as HTMLIFrameElement;
        const FormManagerModal = (iframe.contentWindow as ModalWindow).FormManager;
        const form = FormManagerModal.byname(getClassContabilizacao());
        const paramString = JSON.stringify(form.getValues());

        if (modalControl.saved) {
            AJaxCadastroExecuteMethodGetProperty(
                Class,
                'ValidarContabilizacaoCloud',
                paramString,
                '',
                (response) => ValidarContabilizacaoCloud({ Alias, Class, response }));
        }
    };

    return event;
};

interface ValidarContabilizacaoCloudProps extends Config {
    response: Response
}

const ValidarContabilizacaoCloud = ({ Alias, Class, response }: ValidarContabilizacaoCloudProps) => {
    const { data: { result: { PossuiDiferenca, Sequencial } } } = response;

    if (PossuiDiferenca) {
        criarDialogSimNao(
            'Existem diferenças nos lançamentos contábeis! Deseja corrigir os lançamentos?',
            'Confirmação',
            () => openModal({ response, Alias, Class })
        );
    }
    else {
        const form = FormManager.byname(Class, Alias) as Formulario;
        const paramString = JSON.stringify({
            Sequencial,
            params: form.getValues(),
        })

        AJaxCadastroExecuteMethodGetProperty(
            Class,
            'FinalizarContabilizacaoCloud',
            paramString,
            '',
            () => {
                qNotifier.success('Contabilização finalizada!');

                form.executeAction('actDataSetRefreshExecute');
            },
        );
    }
};
